dnl ------------------------------------------------------
dnl ------       TUVISION-LIBRARY acinclude.m4   ---------
dnl ------        (c) Fabian Wenzel 17.12.2001   ---------
dnl ------ This software is part of the tuvision ---------
dnl ------ image processing library. The license ---------
dnl ------           in COPYING applies          ---------
dnl ------------------------------------------------------


dnl TUVISION_ADDCONTRIBIOLIB adds a contrib-directory to
dnl the list of packages which should be compiled
dnl This is necessary whenever e.g. an I/O library is not
dnl present or properly installed on the system
AC_DEFUN([TUVISION_ADDCONTRIBIOLIB],[
	CONTRIB_SRCPATH='$(top_srcdir)/contrib/'$2
	CONTRIB_BUILDPATH='$(top_builddir)/contrib/'$2
	CONTRIB_INCLUDEPATH='$(top_srcdir)/contrib/'$3
	CONTRIB_SUBDIRS="$CONTRIB_SUBDIRS $2"
	TUV_CXXIOFLAGS="$TUV_CXXIOFLAGS -I$CONTRIB_INCLUDEPATH"
	TUV_CONTRIBLIBS="$TUV_CONTRIBLIBS $CONTRIB_BUILDPATH/$1"
	])

dnl TUVISION_ADDEXTRAIOLIB adds an external library to
dnl the include and ld - paths of the tuvision library.
dnl This should happen when a contrib-library (e.g. an
dnl I/O - library is already installed on the system
dnl but in a non-standard location which the compiler
dnl does not find automatically. In such cases,
dnl the configure.in - script should be extended such
dnl that the user can pass the include- and library - paths
dnl to the configure - script by calling --with-pnglib or
dnl --with-pnginclude
AC_DEFUN([TUVISION_ADDEXTRAIOLIB],[
	if test x$3 != "x"; then
		TUV_EXTRALDFLAGS="$TUV_EXTRALDFLAGS -L$3"
	fi
	TUV_EXTRALDFLAGS="$TUV_EXTRALDFLAGS -l$1"

	if test x$2 != "x"; then
		TUV_CXXIOFLAGS="$TUV_CXXIOFLAGS -I$2"
	fi
	])

dnl TUVISION_ADDSTANDARDIOLIB adds an external library to
dnl the ld-flags of the tuvision library compilation files.
dnl This should happen whenever an external library (e.g. an
dnl image I/O - library) has already been installed on the
dnl system such that it can be found automatically by the
dnl compiler.
AC_DEFUN([TUVISION_ADDSTANDARDIOLIB],[
	TUV_EXTRALDFLAGS="$TUV_EXTRALDFLAGS -l$1"
	])


dnl TUVISION_CHECKTIFF checks if the tiff library is installed
dnl on the system if the user has not prevented its use or has
dnl already given an include/lib - location manually
AC_DEFUN([TUVISION_CHECKTIFF],[
	if test "$TIFF_BUILTIN" != 1; then
		if test "$TIFF_GIVEN" != 1; then
			if test "$TIFF_TEST" = 1; then
				AC_CHECK_HEADER(tiffio.h,[
				AC_CHECK_LIB(tiff,TIFFOpen,[
				TUVISION_ADDSTANDARDIOLIB(tiff)],[
				TUVISION_ADDCONTRIBIOLIB(libtiff.la,tiff,tiff)])
				],[TUVISION_ADDCONTRIBIOLIB(libtiff.la,tiff,tiff)],)	
			fi
		else
			TUVISION_ADDEXTRAIOLIB(tiff,$TIFF_INCLUDE,$TIFF_LIB)
		fi
	fi
	])

dnl TUVISION_CHECKJASPER checks if the jasper library is installed
dnl on the system if the user has not prevented its use or has
dnl already given an include/lib - location manually
AC_DEFUN([TUVISION_CHECKJASPER],[
	if test "$JASPER_BUILTIN" != 1; then
		if test "$JASPER_GIVEN" != 1; then
			if test "$JASPER_TEST" = 1; then
				AC_CHECK_HEADER(jasper/jasper.h,[
				AC_CHECK_LIB(jasper,jas_init,[
				TUVISION_ADDSTANDARDIOLIB(jasper)],[
				TUVISION_ADDCONTRIBIOLIB(libjasper.la,jasper,jasper/include)])
				],[TUVISION_ADDCONTRIBIOLIB(libjasper.la,jasper,jasper/include)],)	
			fi
		else
			TUVISION_ADDEXTRAIOLIB(jasper,$JASPER_INCLUDE,$JASPER_LIB)
		fi
	fi
	])

dnl TUVISION_CHECKPNG checks if the png library is installed
dnl on the system if the user has not prevented its use or has
dnl already given an include/lib - location manually
AC_DEFUN([TUVISION_CHECKPNG],[
	if test "$PNG_BUILTIN" != 1; then
		if test "$PNG_GIVEN" != 1; then
			if test "$PNG_TEST" = 1; then
				AC_CHECK_HEADER(png.h,[
				AC_CHECK_LIB(png,png_check_sig,[
				TUVISION_ADDSTANDARDIOLIB(png)
	
				dnl add flags to cygwin if we are not using contrib
				case "$host" in
				*-*-cygwin* | *-*-mingw*)
				TUV_CXXIOFLAGS="-DPNG_STATIC -DZLIB_STATIC"
				;;
				*)
				;;
				esac
				],[TUVISION_ADDCONTRIBIOLIB(libpng.la,png,png)],[-lz -lm])
				],[TUVISION_ADDCONTRIBIOLIB(libpng.la,png,png)])
			fi
		else
			TUVISION_ADDEXTRAIOLIB(png,$PNG_INCLUDE,$PNG_LIB)
			dnl add flags to cygwin if we are not using contrib
			case "$host" in
			*-*-cygwin* | *-*-mingw*)
			TUV_CXXIOFLAGS="-DPNG_STATIC -DZLIB_STATIC"
			;;
			*)
			;;
			esac
		fi
	fi
	])

dnl TUVISION_CHECKJPEG checks if the jpeg library is installed
dnl on the system if the user has not prevented its use or has
dnl already given an include/lib - location manually
AC_DEFUN([TUVISION_CHECKJPEG],[
	if test  "$JPEG_BUILTIN" != 1; then
		if test "$JPEG_GIVEN" != 1; then
			if test "$JPEG_TEST" = 1; then
				AC_CHECK_HEADER(jconfig.h,[
				AC_CHECK_LIB(jpeg,jpeg_destroy_decompress,[
				TUVISION_ADDSTANDARDIOLIB(jpeg)],[
				TUVISION_ADDCONTRIBIOLIB(libjpeg.la,jpeg,jpeg)])
				],[TUVISION_ADDCONTRIBIOLIB(libjpeg.la,jpeg,jpeg)],)
			fi
		else
			TUVISION_ADDEXTRAIOLIB(jpeg,$JPEG_INCLUDE,$JPEG_LIB)
		fi
	fi
	])

dnl TUVISION_CHECKZLIB checks if the zlib library is installed
dnl on the system if the user has not prevented its use or has
dnl already given an include/lib - location manually
AC_DEFUN([TUVISION_CHECKZLIB],[
	if test "$ZLIB_BUILTIN" != 1; then
		if test "$ZLIB_GIVEN" != 1; then
			if test "$ZLIB_TEST" = 1; then
				AC_CHECK_HEADER(zlib.h,[
				AC_CHECK_LIB(z,deflate,[TUVISION_ADDSTANDARDIOLIB(z)],
				[TUVISION_ADDCONTRIBIOLIB(libz.la,zlib,zlib)])
				],[TUVISION_ADDCONTRIBIOLIB(libz.la,zlib,zlib)],)
			fi
		else
			TUVISION_ADDEXTRAIOLIB(z,$ZLIB_INCLUDE,$ZLIB_LIB)
		fi
	fi
	])

dnl TUVISION_CYGWINFLAGS sets thread flags
dnl if the software is compiled for cygwin. This has
dnl to be done when this library should work together
dnl with e.g. wxWindows. fvtable-thunks is deprecated
dnl as gcc3.x don't need it anymore
AC_DEFUN([TUVISION_CYGWINFLAGS],[
	case "$host" in
	*-*-cygwin* | *-*-mingw*)
	TUV_CXXFLAGS="$TUV_CXXFLAGS -mthreads -D_TUVISION_EXPORTS"
	TUV_BUILDCXXFLAGS="$TUV_BUILDCXXFLAGS -mthreads -D_TUVISION_IMPORTS"
	TUV_EXTRALDFLAGS="$TUV_EXTRALDFLAGS -mthreads"
        lt_cv_deplibs_check_method='pass_all'
	;;
	powerpc-*)
	AC_DEFINE(__MACOS__,,"Mac OS")
	;;
	*)
	;;
	esac
	])

AC_DEFUN([TUVISION_DEBUGFLAGS],[
	if test x$debug = xtrue; then
	  TUV_CXXDEBUGRELEASEFLAGS="-g3 -DDEBUG -O0"
	  TUV_BUILDDEBUGRELEASELIB="-ltuvisiond"
	else
	  TUV_CXXDEBUGRELEASEFLAGS="-O3"
	  TUV_BUILDDEBUGRELEASELIB="-ltuvision"
	fi

	CXXFLAGS="${TUV_CXXDEBUGRELEASEFLAGS} -Wall"
	CFLAGS="${TUV_CXXDEBUGRELEASEFLAGS} -Wall"
	AM_CONDITIONAL(DEBUG, test x$debug = xtrue)
	])

dnl TUVISION_LIBVERSION sets the variable which will be used
dnl by automake later.
AC_DEFUN([TUVISION_LIBVERSION],[
	TUV_LDLIBVERSION=`echo $1 | sed -e s"/\\./\\:/g"`
	TUV_RPMLIBVERSION=$1
	AC_SUBST(TUV_LDLIBVERSION)
	AC_SUBST(TUV_RPMLIBVERSION)
	])
