/**
 * erropts.cpp
 * 98/11/22
 * Changes:
 *   98/12/18
 *     reads bit errors
 *   98/12/20
 *     identifies header bit errors. 
 *
 * @author Eric Sommerlade
 * 
 * parses error definition file 
 *
 */
#include <stdio.h>
#include <stdlib.h>
extern "C" {
#include <string.h>
#include "erropts.h"
}


/* this macro lets you view debug output: */
//#define ERR_DEBUG


TBoolean Error_Quiet_Flag=False;
TErrInfo *Err_Info=NULL;   

/** Holds theerror information
 *  if an entry is <0, it is set to -1.
 *  a negative value specifies an error in a 
 *  superstructure, but before a substructure
 *  was identified. (i.e. header information in 
 *  macroblocks)
 */
class CError{
protected:
  /**
   * used for comparison
   */
  static int max(int i) { return(i<0?-1:i); };
  int p, s, mb, b, bit;
public:
  CError(int pp, int ss, int mm, int bb, int bbit) {
    p=max(pp);
    s=max(ss);
    mb=max(mm);
    b=max(bb);
    bit=max(bbit);
  };
  /** checks if argument occurs before or parallel to this
   *  
   */
  TBoolean operator<(CError &e) {
    if(e.p>p) return(True);
    if(e.p<p) return(False);
    if(e.s>s) return(True);
    if(e.s<s) return(False);
    if(e.mb>mb) return(True);
    if(e.mb<mb) return(False);
    if(e.b>b) return(True);
    if(e.b<b) return(False);
    if(e.bit>bit) return(True);
    if(e.bit<bit) return(False);
    return(False);
  };
  TBoolean operator==(CError &e) {
    if(e.p!=p) return(False);
    if(e.s!=s) return(False);
    if(e.mb!=mb) return(False);
    if(e.b!=b) return(False);
    if(e.bit!=bit) return(False);
    return(True);
  };

  void print() {
    printf("p: %d, s: %d, mb: %d, b: %d, bit: %d\n", p, s, mb, b, bit);
  };
};





/** simple minimal list to store error info
 *
 */


template <class T> class CMiniListEntry {
public:
  T& getVal() { return(val); };
  CMiniListEntry(T t): val(t) { n=NULL; };
  CMiniListEntry<T> *getNext() {return(n);};
  void setNext(CMiniListEntry<T> *le) { n=le; };
protected:
  CMiniListEntry *n;
  T val; 
};


template <class T> class CMiniList {
public:
  CMiniList() { head=cur=NULL; ent=0; };
  int entries() { return(ent); } ;
  void print() {
    printf("%d entries:\n", ent);
    cur=head;
    while(cur) {
      cur->getVal().print();
      cur=cur->getNext();
    }
  };
  void start() { cur=head; }
  TBoolean end() { 
    return((TBoolean)(cur->getNext()==NULL)); 
  };
  inline void next() { 
    cur=cur->getNext();
  };
  virtual void insert(T t) { // inserts after cur
    CMiniListEntry<T> *now=new CMiniListEntry<T>(t);
    if(!head) 
      cur=head=now;
    else {
      while(!end()) 
        next();
      cur->setNext(now);
    }
    ent++;
  };
protected:
  int ent;
  CMiniListEntry<T> *head, *cur;
};

template <class T> class CSortedMiniList: public CMiniList<T> {
public:
  using CMiniList<T>::head;
  using CMiniList<T>::cur;
  void insert(T t) {
    CMiniListEntry<T> *now=new CMiniListEntry<T>(t);
    if(!this->head) 
      cur=head=now;
    else {
      CMiniListEntry<T> *last;
      last=cur=head;

      while(cur && (cur->getVal()<t)) {
        last=cur;
        this->next();
      }
      // cur must be > now at this point
      if(head==cur) {
        now->setNext(head);
        head=now;
        return;
      }
      now->setNext(cur);
      last->setNext(now);
      cur=now;
    }
    this->ent++;
  };
  TBoolean valid(T& ee) {
    if(!cur) return(True);
    while(!this->end() && cur->getVal()<ee) {
      this->next();
    }
    return((TBoolean)!(ee==cur->getVal())); // whole T is invalid
  };
};

class CErrorList: public CSortedMiniList<CError> {
protected:
  int errorp, errors, errormb, errorb, errorbit;
  int bstartbit;
  int biterrorp, biterrors, biterrormb, biterrorb;
public:
  CErrorList(): CSortedMiniList<CError>() {
    errorp=errors=errormb=errorb=errorbit=-1;
    bstartbit=0;
  };
  TBoolean validPic(int p) {
    CError ee(p,-1,-1,-1,-1);
    bstartbit=errorbit+1;
    errorp=p;   // only parts of this pic may be invalid
    errors=errormb=errorb=-1;
    return(valid(ee));
  };
  TBoolean validSlc(int s) {
    CError ee(errorp,s,-1,-1,-1);
    bstartbit=errorbit+1;
    errors=s;   // only parts of this slc may be invalid
    errormb=errorb=-1;
    return(valid(ee));
  };
  TBoolean validMb(int mb) {
    CError ee(errorp,errors,mb,-1,-1);
    bstartbit=errorbit+1;
    errormb=mb;   // only parts of this mb may be invalid
    errorb=-1;
    return(valid(ee));
  };
  TBoolean validB(int b) {
    CError ee(errorp,errors,errormb,b,-1);
    bstartbit=errorbit+1;
    errorb=b;   // only parts of this b may be invalid
    return(valid(ee));
  };
  TBoolean validBit(int bit) {
    CMiniListEntry<CError> *tmp;
    TBoolean res;
    int maxbit=bit-bstartbit;
    maxbit=maxbit<0?0:maxbit;
    CError ee(errorp,errors,errormb,errorb,maxbit);
    errorbit=bit;
    tmp=cur;  // save current pos for bitstream decoding
              // check may occur twice!
    
    res=valid(ee); 
    
    cur=tmp;
    return(res);
  };
};


TBoolean validPic(int p, TErrInfo *e) {
  if(!e) return(True);
  return(((CErrorList *)e)->validPic(p));
};
TBoolean validSlc(int s, TErrInfo *e) {
  if(!e) return(True);
  return(((CErrorList *)e)->validSlc(s));
};
TBoolean validMba(int mb, TErrInfo *e) {
  if(!e) return(True);
  return(((CErrorList *)e)->validMb(mb));
};
TBoolean validBlk(int b, TErrInfo *e) {
  if(!e) return(True);
  return(((CErrorList *)e)->validB(b));
};
TBoolean validBit(int b, TErrInfo *e) {
  if(!e) return(True);
  return(((CErrorList *)e)->validBit(b));
};


/* initializes a new error information structure.
 * if an error occurs NULL is returned 
 */

TErrInfo *processErrorOptions(char *filename) {
  FILE *f;
  char line[256];
  char teststring[256];
  int val[5];
#ifdef ERR_DEBUG
  char *str[5]={ "p", "s", "mb", "b", "bit"};
#endif
  char errtype;
  int correct;
  CErrorList *ml;


  if(!filename)
    return(NULL); //default value

  f=fopen(filename, "rt");
  if(!f) {
#ifdef ERR_DEBUG
    fprintf(stderr, "error option file '%.256s' not found!", filename);
#endif
    return(NULL);
  }

  
  ml=new CErrorList();

  while(!feof(f)) {
    fgets(line, 256, f);
#ifdef ERR_DEBUG
    printf("processing: %s", line);
#endif
    if(*line==';') continue; // is it a comment??

    teststring[0]=0;
    sscanf(line, " %256s ", teststring);
    if(!strcmp("quiet",teststring))
      Error_Quiet_Flag=True;

    val[0]=val[1]=val[2]=val[3]=val[4]=-1;
    correct=sscanf(line, "%c : %d : %d : %d : %d : %d", 
                         &errtype, val, val+1, val+2, val+3, val+4);
    if(correct==EOF) continue; // something wrong with the file
#ifdef ERR_DEBUG
     printf("%c\n",errtype);
#endif
    if(errtype!='b') continue;
    if(correct<7 && correct>1)
    {

      CError e(val[0], val[1], val[2], val[3], val[4]);
      ml->insert(e);

#ifdef ERR_DEBUG
      for(int i=0; i<correct-1;i++) 
        printf("%s: %d ", str[i], val[i]);
      if(correct) printf("\n");
#endif
    }
  }

#ifdef ERR_DEBUG
  ml->print();
#endif
  
  ml->start();
  Err_Info=(TErrInfo *)ml;
  return((TErrInfo *)ml);
}

void freeErrorOptions(TErrInfo *e_s) {
  if(e_s)
    delete ((CErrorList *)e_s);
  Err_Info=NULL;
}





  



