/***************************************************************************
                          otherdocumentation.cpp  -  description
                             -------------------
    begin                : |DATE|
    copyright            : (C) |YEAR| by |AUTHOR|
    email                : tuvisionlib@tuvision.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
// this file documents all the code-related stuff
// that does not fit into the original source code.
// stuff documented here:
//
// - doxygen mainpage for the library 
// - namespace tuvision documentation.
// - module documentation - groups the classes.

/*!
 * \mainpage
 * \section codingstyle Coding Style
 *
 * Before contributing to the TUVision Image Library,
 * read the <a href="CodingStyle.html">coding style guide</a>.
 *
 * \section librarystructure Library Structure
 *
 * In case you are wondering where to put your newly 
 * implemented classes, or if you do not know what all
 * these directories are about, 
 * read the <a href="LibraryStructure.html">library structure guide</a>.
 *
 * \section license Software License
 * The TUVision Image Library is put under the
 * GNU Library General Public License (LGPL).
 * For a full text, refer to
 * <a href="http://www.gnu.org/copyleft/lesser.html">LGPL</a>.
 *
 * \section fchanges Future Changes
 * At the moment, the image library is at its very beginning.
 * The approach of duplicating the code
 * of the basic image types is not very promising. <br>
 * The conversion between color models (i.e. s-rgb, yuv, ycrcb..)
 * will be included in the next version. So beware of
 * using the convertToGray method - it will be removed in
 * the very near future.
 *
 * \section miscellaneous Miscellaneous
 * This documentation was generated at datedatedate
 */

//! The namespace that contains all TUVision library elements.
/*!
 *  All classes, struct, 'global' functions that relate to the
 *  library are grouped here.
 *
 *  You must 'use' the namespace, e.g. in such a way:
 *
 <code>
   using namespace TUVision;
   CImageByteGray img(100,100);
 </code>
 *
 * just as you would do with the std namespace.
 *
 */
namespace TUVision {
}

/*! \defgroup image The Image group
 *  This module groups all classes and functions that deal with images only. See 
 * <a href="DocumentationImages\index.html">LGPL</a>.
 */

/*! \defgroup filter The Filter group
 *  It contains abstract classes for inheriting as well as
 *  full implementations of filters like sobel or convolution.
 *  This module groups all classes and functions that filter images.
 */

/*! \defgroup operator The Operator group
 *  This module groups all classes and functions that operate
 *  on images by extracting features or converting them into other
 *  data or color space formats.
 */

/*! \defgroup exception The Exception group
 *  This module groups all exception classes.
 *  These exceptions are thrown by the methods from the
 *  Io group, for example.
 *  
 *  It is still not sure whether to throw an exception in 
 *  CImage methods or not. So do not rely on one being thrown
 *  when passing the parameters - just make sure they are 
 *  specified correctly.
 */

/*! \defgroup io The Io group
 *  This module groups all input/output related classes.
 */

/*! \defgroup geometry The geometry group
 *  This module groups all geometry related classes.
 */

/*! \defgroup extractors The extractor group
 *  This module groups all extraction related classes.
 */

/*! \defgroup 3d The 3d group
 *  This module groups all 3d related classes as e.g. camera models and calibration.
 */
