/***************************************************************************
                          io_mpeg.cpp  -  description
                             -------------------
    begin                : |DATE|
    copyright            : (C) |YEAR| by |AUTHOR|
    email                : tuvisionlib@tuvision.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
/**********************************************************/
/* example for mpeg stream disassembly.
 * \author Eric Sommerlade
 */
/**********************************************************/


#include <iostream>

#include "tuv/CImageByteRGB.h"
#include "tuv/CImageByteYUV.h"
#include "tuv/CException.h"
#include "tuv/CFileWriteTiff.h"
#include "tuv/CFileReadMpeg.h"
#include <stdio.h>
#include "tuv2d/COperatorRGB.h"


using namespace std;
using namespace TUVision;

/*!  main routine.
 *   arguments: 
 *    mpeg-filename          - the source.
 *    output-filename-prefix - number and suffix are appendid by the
 *                             program
 */
int main (int argc, char** argv)
{
    CFileReadMpeg   src;
    CFileWriteTiff  dst;
    CImageByteYUV       *img;

    if(argc<3) {
        cerr << "usage: " << *argv << " infile(mpeg format) outfile-prefix(.tif)"
            << endl
            << "       The tool will separate an mpeg stream to its single pictures."
            << endl << endl;
        return(-1);
    }

    try {

        cout << "Opening file " << argv[1] << endl;
        // the mpeg file source is opened here.
        src.open (argv[1]);

        // Writing all found images to file of the name "argv%n.tif".
        try {
            // as long as there are pictures in the source
            int i;
            for(i=0;;++i) {
                // now convert to rgb-image for saving.
                img = dynamic_cast<CImageByteYUV*> (src.read());
                CImageByteRGB rgb(convertToRGB(*img));
                // delete old yuv image.
                delete img;

                // calculate output file name.
                char *filename=(char*)malloc(strlen(argv[2])+5+4+1);
                sprintf(filename,"%s%05i.tif",argv[2],i);
                // open it
                dst.open (filename);
                cout << "Writing: " << filename << endl; 
                free(filename);

                // write the rgb picture as a tiff
                dst.write (rgb);
                // close the file.
                dst.close();
            }
        }
        catch (CException &e) { 
            // treat the library errors.
            cerr << "Error: " << e.what() << endl; 
        }
        // close the mpeg file source
        src.close();
    }
    catch (std::exception e) {
        // treat std c++ errors.
        cerr << "std exception caught: " << e.what() << endl; 
    } 
    catch (...) {
        // treat unknown errors.
        cerr << "Unknown exception caught." << endl; 
    } 
    return (0);
}
