/***************************************************************************
                          io_png.cpp  -  description
                             -------------------
    begin                : 19.09.2001
    copyright            : (C) 2001 by Fabian Wenzel
    email                : tuvisionlib@tuvision.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
/**********************************************************/
/*
 *   This example shows how to read and write an image
 *   from/to a file using the I/O classes.
 *
 *   It is important to catch the exceptions thrown by
 *   the library. Otherwise your program will be aborted
 *   when an error occurs during the I/O process.
 *
 *                                         Marco Budde
 */
/**********************************************************/

#include <iostream>

#include <tuv/CImageByteGray.h>
#include <tuv/CException.h>
#include <tuv/CFileReadPng.h>
#include <tuv/CFileWritePng.h>


using namespace std;
using namespace TUVision;


int main (void)
{
    CFileReadPng   src;
    CFileWritePng  dst;
    CImageByteGray  *img;


    // Read "test.tif" (gray, 8 bit).
    try {
        src.open ("test.png");

        img = dynamic_cast<CImageByteGray*> (src.read());
        if (img == NULL)
            cerr << "Cannot read PNG file! Wrong color model?" 
            << endl << endl;

        src.close();
    } 
    catch (CException &e) { 
        cerr << "Error: " << e.what() << endl; 
        return (0); 
    } 


    // Write "test2.tif".
    try {
        dst.open ("test2.png");
        dst.write (*img);
        dst.close();
    }
    catch (CException &e) { 
        cerr << "Error: " << e.what() << endl; 
        return (1); 
    } 

    return (0);
}
