/***************************************************************************
                          cline.cpp  -  description
                             -------------------
    begin                : Die May 7 2002
    copyright            : (C) 2002 by Fabian Wenzel
    email                : tuvisionlib@tuvision.de, wenzel@tu-harburg.de
    origin               : PhD Fabian Wenzel

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/

#include "cline.h"
#include <complex>
#include <cstdio>


/***************************************************************************/
/*!
 *  Initializes the line by calculating the normal vector and distance to
 *  the origin of a line based on two points
 *
 ***************************************************************************/
CLine::CLine( const TDataVector &list) :
m_normalvector(list[1].y-list[0].y,list[1].x-list[0].x)
{
    m_normalvector/= m_normalvector.abs();
    m_distance     = m_normalvector.dot(list[0]);
}


/***************************************************************************/
/*!
 *  Destructor does nothing as there is no dynamic data
 *
 ***************************************************************************/
CLine::~CLine(){
}


/***************************************************************************/
/*!
 *  Calculates the distance of a point pair to a line
 *
 ***************************************************************************/
double CLine::calculateDistance( const TDataVector::value_type &sample )
{
    return sample.x*m_normalvector.x+sample.y*m_normalvector.y-m_distance;
}
