/***************************************************************************
                          linefit.cpp  -  description
                             -------------------
    begin                : Die May 7 2002
    copyright            : (C) 2002 by Fabian Wenzel
    email                : tuvisionlib@tuvision.de, wenzel@tu-harburg.de
    origin               : PhD Fabian Wenzel

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/

#include "tuv3d/CRansac.h"
#include "cline.h"

#include <cstdlib>
#include <iostream>
#include <ctime>

using namespace TUVision;
using namespace std;

int main(int argc, char *argv[])
{

    // Randomizer has to be initialized for RANSAC (as ransac picks
    // random data). As this has to be done *only once* in a program,
    // this is done here.
    srand(time(0));


    // Fill sample data vector with some data.
    // The *data type* of the sample data has to be defined
    // via "typedef" in the model itself (see CLine) as
    // RANSAC also uses this data type for building testmodels
    // In this example, the line gets built out of 2D-Points
    // (see below) and there is one outlier in it
    CLine::TDataVector mydata;

    mydata.push_back( CVertex2d(0.0, 1.0) );
    mydata.push_back( CVertex2d(  0,-4.0) ); // Outlier!!!!
    mydata.push_back( CVertex2d(0.4, 1.0) );
    mydata.push_back( CVertex2d(1.9, 1.0) );
    mydata.push_back( CVertex2d( -5, 1.0) );
    mydata.push_back( CVertex2d(  5, 1.0) );

    cout << "Initialized vector" << endl;

    // Initialize RANSAC with the CLine - Datamodel which has to be
    // determined
    CRansac < CLine > myransac( mydata );

    // These two values *have* to be set (or the variance or a
    // normally distributes sample set / the outlier probability)
    myransac.setThreshold(0.01);
    myransac.setSampleCount(6);

    CLine::TDataVector *outliers = myransac.getOutliers();

    cout << outliers->size() << " outliers" << endl;
    for(int i=0;i<outliers->size();i++){
        cout << "Outlier " << i+1 << ": (" << (*outliers)[i].x << "," << (*outliers)[i].y << ")" << endl;
    }

    return EXIT_SUCCESS;
}
