#include <tuv/CFileReadUtils.h>   // We want to read a png - file
#include <tuv/CFileWritePng.h>  // We want to write the output images
#include <tuv/CImageByteGray.h> // We want to use a grayscale image

#include <iostream>

#include <vigra/imageiterator.hxx>
#include <vigra/functorexpression.hxx>
#include <vigra/combineimages.hxx>

using namespace std;
using namespace vigra;
using namespace vigra::functor;


using namespace TUVision;

/* Main function */
int main(int argc, char *argv[]){

    try{
        /* Write all images using CFileWritePng */
        CFileWritePng myFileWriter;
        CImageByteGray *inputimage = readImageByteGray("input.png");

        /* Set up width and height so that the function calls can
           have to be written down only once (makes code nicer) */
        int width =inputimage->getW();
        int height=inputimage->getH();

        /* Set up accessor (access gray value of image) such that
           code looks nicer */
        StandardAccessor<CImageByteGray::PixelType> grayaccess;

        
        /* Horizontal average and difference */
        /*************************************/
        
        /* Image Iterator that skips every second pixel horizontally */
        
        StridedImageIterator<CImageByteGray::Iterator::PixelType>
            subsamp_hor (&(*inputimage->upperLeft())  ,width,2,1);

        /* Output image which has half width */
        CImageByteGray horizontaloutput   (width/2,height);

        /* Builds average */
        combineTwoImages(subsamp_hor,subsamp_hor+Diff2D(width/2, height),grayaccess,
                         subsamp_hor+Diff2D(1,0),grayaccess,
                         horizontaloutput.upperLeft(),grayaccess,
                         (Arg1()+Arg2())/Param(2));

        /* Save image with PNG writer */
        myFileWriter.open("horizontalaverage.png");
        myFileWriter.write(horizontaloutput);
        myFileWriter.close();

        /* Builds difference */
        combineTwoImages(subsamp_hor,subsamp_hor+Diff2D(width/2, height),grayaccess,
                         subsamp_hor+Diff2D(1,0),grayaccess,
                         horizontaloutput.upperLeft(),grayaccess,
                         abs(Arg1()-Arg2())/Param(2));

        /* Save image with PNG writer */
        myFileWriter.open("horizontaldifference.png");
        myFileWriter.write(horizontaloutput);
        myFileWriter.close();

        
        /* Vertical average and difference */
        /*************************************/
        
        /* Image Iterator that skips every second pixel vertically */
        StridedImageIterator<CImageByteGray::Iterator::PixelType>
            subsamp_vert (&(*inputimage->upperLeft())      ,width,1,2);

        /* Output image which has half height */
        CImageByteGray verticaloutput   (width,height/2);

        /* Builds average */
        combineTwoImages(subsamp_vert,subsamp_vert+Diff2D(width, height/2),grayaccess,
                         subsamp_vert+Diff2D(0,1),grayaccess,
                         verticaloutput.upperLeft(),grayaccess,
                         (Arg1()+Arg2())/Param(2));

        /* Save image with PNG writer */
        myFileWriter.open("verticalaverage.png");
        myFileWriter.write(verticaloutput);
        myFileWriter.close();

        /* Builds absolute difference */
        combineTwoImages(subsamp_vert,subsamp_vert+Diff2D(width, height/2),grayaccess,
                         subsamp_vert+Diff2D(0,1),grayaccess,
                         verticaloutput.upperLeft(),grayaccess,
                         abs(Arg1()-Arg2())/Param(2));

        /* Save image with PNG writer */
        myFileWriter.open("verticaldifference.png");
        myFileWriter.write(verticaloutput);
        myFileWriter.close();

        delete inputimage;

    } catch ( CException &e ){
        cout << e.what() << endl;
    }
    return 0;
}
