/***************************************************************************
                          CExceptionFile.h  -  description
                             -------------------
    begin                : 01.01.2001
    copyright            : (C) 2001 by Marco Budde
    email                : tuvisionlib@tuvision.de
    origin               : Diplomarbeit Marco Budde

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
#ifndef __TUVISION_CEXCEPTIONFILE_H__
#define __TUVISION_CEXCEPTIONFILE_H__ 1

#include <string>
#include "TUVisionDef.h"
#include "tuv/CException.h"

namespace TUVision {

/**********************************************************/
//!  Base class defining an file exception.
/*!
 *   This base class defines a file exception. All 
 *   exceptions describing file I/O problems are
 *   derived from this class.
 *
 *   \ingroup exception
 *
 *   \author Marco Budde
 */
/**********************************************************/

class _CCTUVISION CExceptionFile : public CException { 
protected:
    CExceptionFile(std::string what) : CException(what) {}
};


/**********************************************************/
//!  Exception: opening a file
/*!
 *   This exception is used to report a problem with
 *   opening a file.
 *
 *   \ingroup exception
 *
 *   \author Marco Budde
 */
/**********************************************************/

class _CCTUVISION CExceptionFileOpen : public CExceptionFile {

public:

    /******************************************************/
    //! Constructor
    /******************************************************/
    CExceptionFileOpen (const std::string &filename,
        const std::string &add_infos = ""); 
};


/**********************************************************/
//!  Exception: reading from a file
/*!
 *   This exception is used to report a problem with
 *   reading from a file.
 *
 *   \ingroup exception
 *
 *   \author Marco Budde
 */
/**********************************************************/

class _CCTUVISION CExceptionFileRead : public CExceptionFile {

public:

    /******************************************************/
    //! Constructor
    /******************************************************/
    CExceptionFileRead (const std::string &filename,
        const std::string &add_infos = "");
};


/**********************************************************/
//!  Exception: writing to a file
/*!
 *   This exception is used to report a problem with
 *   writing to a file.
 *
 *   \ingroup exception
 *
 *   \author Marco Budde
 */
/**********************************************************/

class _CCTUVISION CExceptionFileWrite : public CExceptionFile {

public:

    /******************************************************/
    //! Constructor
    /******************************************************/
    CExceptionFileWrite (const std::string &filename,
        const std::string &add_infos = "");
};

/**********************************************************/
//!  Exception: handling a file
/*!
 *   This exception is used to report a problem with
 *   unknown contents of a file.
 *
 *   \ingroup exception
 *
 *   \author Eric Sommerlade
 */
/**********************************************************/

class _CCTUVISION CExceptionFileHandle : public CExceptionFile {

public:

    /******************************************************/
    //! Constructor
    /******************************************************/
    CExceptionFileHandle (const std::string &filename,
        const std::string &add_infos = "");
};

} // namespace TUVision

#endif // __TUVISION_CEXCEPTIONFILE_H__
