/***************************************************************************
                          CFile.h  -  description
                             -------------------
    begin                : 01.01.2001
    copyright            : (C) 2001 by Marco Budde
    email                : tuvisionlib@tuvision.de
    origin               : Diplomarbeit Marco Budde

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
#ifndef __TUVISION_CFILE_H_
#define __TUVISION_CFILE_H_ 1

#include "TUVisionDef.h"
#include <string>


namespace TUVision {

/**********************************************************/
//!   Image I/O base class.
/*! 
 *    This base class defines a class for image I/O.
 *    From this class we derive two classes: one for
 *    reading and one for writing a image.
 *
 *    Classes derived from this class should report
 *    errors via exceptions. Normally a function should
 *    throw only exceptions derived from the 
 *    std::exception or CException classes.
 *
 *    \ingroup io
 *
 *    \author Marco Budde
 */
/**********************************************************/

class _CCTUVISION CFile {

public:

    virtual ~CFile() {};

    /******************************************************/
    //!  Open image file.
    /*! 
     *   Opens the image file with the given filename.
     *
     *   \param filename name of the file
     */
    /******************************************************/
    virtual void open (const std::string &filename) = 0;

    /******************************************************/
    //!  Close image file.
    /*! 
     *   Closes an open image file.
     */
    /******************************************************/
    virtual void close (void) = 0;

protected:
    std::string  d_filename;   /*!< name of the file */
};

}

#endif  // __TUVISION_CFILE_H_
