/***************************************************************************
                          CFileReadBmp.h  -  description
                             -------------------
    begin                : 01.01.2001
    copyright            : (C) 2001 by Marco Budde
    email                : tuvisionlib@tuvision.de
    origin               : Diplomarbeit Marco Budde

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
#ifndef __TUVISION_CFILEREADBMP_H_
#define __TUVISION_CFILEREADBMP_H_ 1

#include <exception>

#include "TUVisionDef.h"
#include "tuv/CImageDatatypes.h"
#include "tuv/CFileRead.h"
#include "tuv/CException.h"
#include <cstdio>
#include <string>
#include "tuv/CFileReadRegistrator.h"

namespace TUVision {


/**********************************************************/
/*!
 *   This class read an image as a BMP graphic file.
 *
 *   \ingroup io
 *
 *   \author Fabian Wenzel
 *   \class CFileReadBmp CFileReadBmp.h tuv/CFileReadBmp.h
 */
/**********************************************************/

class _CCTUVISION CFileReadBmp: public CFileRead {

public:
    CFileReadBmp (void);
    ~CFileReadBmp();

    void open (const std::string &filename)
        throw (CException);

    void close (void);

    /******************************************************/
    //!  Read 8 bit RGB image BMP fornmat.
    /******************************************************/
    CImage* read (int image_number=0)
        throw (CException, std::exception);

    int getNumberImages (void);

private:
    std::string d_filename;
    void *jfile;
};

// static class to register the file reader
/*! \class CFileReadBmpRegistrator CFileReadBmp.h tuv/CFileReadBmp.h */
class CFileReadBmpRegistrator: public CFileReadRegistrator {
    static const unsigned char d_id[];
    static const int d_len;    
public:

    TUVision::CFileRead* construct() {
        return(new TUVision::CFileReadBmp());
    }

    const int getLength() { return d_len; }
    const unsigned char *getID() { return d_id; }

};

}

#endif

