/***************************************************************************
                          CFileReadJpeg.h  -  description
                             -------------------
    begin                : 01.01.2001
    copyright            : (C) 2001 by Marco Budde
    email                : tuvisionlib@tuvision.de
    origin               : Diplomarbeit Marco Budde

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
#ifndef __TUVISION_CFILEREADJPEG_H_
#define __TUVISION_CFILEREADJPEG_H_ 1

#include <cstdio>
#include <exception>

#include "tuv/CFileRead.h"
#include "tuv/CException.h"
#include "tuv/CFileReadRegistrator.h"

namespace TUVision {

/**********************************************************/ 
//!  Reads JPEG graphic files.
/*! 
 *   This class reads a JPEG graphic file to memory.
 *   JPEG is an often used graphic file format for 
 *   lossy compressed images. Because the compression 
 *   works really effectively, JPEG is a good format for 
 *   the transmission of images over slow communication 
 *   lines like modems.
 *
 *   The returned image objects could be of type 
 *   CImageByteGray or CImageByteRGB. A JPEG file could
 *   contain only one image.
 *
 *   \ingroup io
 *
 *   \author Marco Budde
 *
 *   \class CFileReadJpeg CFileReadJpeg.h tuv/CFileReadJpeg.h
 */
/**********************************************************/

class _CCTUVISION CFileReadJpeg : public CFileRead {

public:
    CFileReadJpeg (void);
    ~CFileReadJpeg();

    void open (const std::string &filename)
        throw (CException);

    void close (void)
        throw();

    CImage *read (int img_number = 0)
        throw (CException, std::exception);

    int getNumberImages (void)
        throw();

private:
    FILE  *d_hd;          //!< file handle
};

// static class to register the file reader
/*! \class CFileReadJpegRegistrator CFileReadJpeg.h tuv/CFileReadJpeg.h */
class CFileReadJpegRegistrator: public CFileReadRegistrator {
    static unsigned const char d_id[];
    static const int d_len;    
public:

    TUVision::CFileRead* construct() {
        return(new TUVision::CFileReadJpeg());
    }

    const int getLength() { return d_len; }
    const unsigned char *getID() { return d_id; }

};
} 

#endif // __TUVISION_CFILEREADJPEG_H
