/***************************************************************************
                          CFileReadMpeg.h  -  description
                             -------------------
    begin                : 01.01.2001
    copyright            : (C) 2001 by Eric Sommerlade
    email                : tuvisionlib@tuvision.de
    origin               : Studienarbeit/Diplomarbeit Eric Sommerlade

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
#ifndef __TUVISION_CFILEREADMPEG_H__
#define __TUVISION_CFILEREADMPEG_H__ 1

#include "tuv/CFileRead.h"
#include "tuv/CException.h"
#include "tuv/CFileReadRegistrator.h"

struct tstreaminfo;
typedef struct tstreaminfo TStreamInfo;

namespace TUVision {

/**********************************************************/ 
//!  Reads MPEG graphic files.
/*! 
 *   This class reads a MPEG graphic file to memory.
 *   Currently, only one mpeg file may be open at a time.
 *
 *   The returned image objects is of type 
 *   CImageByteYUV. A mpeg file may
 *   contain several images and usually does.
 *
 *   To use this class, you have to have
 *   the library "libmpeg2" installed on your system.
 *
 *   With mpeg files, the difference between
 *   files and streams becomes clear:
 *   streams may start anywhere, files expect a specific
 *   startpoint specified by special bytes.
 *
 *   \ingroup io
 *
 *   \author Eric Sommerlade
 *
 *   \class CFileReadMpeg CFileReadMpeg.h tuv/CFileReadMpeg.h
 */
/**********************************************************/

class _CCTUVISION CFileReadMpeg : public CFileRead {

public:
    CFileReadMpeg ();
    ~CFileReadMpeg();

    void open (const std::string &filename);

    void close ();

    CImage *read (int img_number = 0);

    int getNumberImages ();

private:
    TStreamInfo* d_si;          //!< stream data handle
    bool d_openedstream;        //!< state of the reader
};


//! Registers the Mpeg2 file reader.
/*!
 * Class to register the mpeg2 file reader.
 *
 * \author Eric Sommerlade
 * \class CFileReadMpegRegistrator CFileReadMpeg.h tuv/CFileReadMpeg.h
 */
class CFileReadMpegRegistrator: public CFileReadRegistrator {
    //! id of the file
    static unsigned const char d_id[];
    //! length of the id 
    static const int d_len;
public:
    //! Constructs the file reader.
    TUVision::CFileRead* construct() {
        return(new TUVision::CFileReadMpeg());
    }

    const int getLength() { return d_len; }
    const unsigned char *getID() { return d_id; }

};

} // namespace TUVision

#endif  // __TUVISION_CFILEREADMPEG_H__
