/***************************************************************************
                          CFileReadPng.h  -  description
                             -------------------
    begin                : 19.09.2001
    copyright            : (C) 2001 by Fabian Wenzel
    email                : tuvisionlib@tuvision.de
    origin               : Preparation Bildverarbeitung 1 WS 2001

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
#ifndef __TUVISION_CFILEREADPNG_H_
#define __TUVISION_CFILEREADPNG_H_ 1

#include <cstdio>
#include <exception>

#include "tuv/CFileRead.h"
#include "tuv/CException.h"
#include "tuv/CFileReadRegistrator.h"

namespace TUVision {

/**********************************************************/ 
//!  Reads PNG graphic files.
/*! 
 *   This class reads a PNG graphic file to memory.
 *   PNG is used as as a graphic file format for
 *   lossless compression. It works more effiently than GIF
 *   for example by also providing many features like
 *   transparency. Its power lies in runlength compression
 *   of color-indexed files.
 *
 *   The returned image objects could be of type 
 *   CImageByteGray or CImageByteRGB. A JPEG file could
 *   contain only one image.
 *
 *   \ingroup io
 *
 *   \author Fabian Wenzel
 *
 *   \class CFileReadPng CFileReadPng.h tuv/CFileReadPng.h
 */
/**********************************************************/

class _CCTUVISION CFileReadPng : public CFileRead {

public:
    CFileReadPng (void);
    ~CFileReadPng();

    void open (const std::string &filename)
        throw (CException);

    void close (void)
        throw();

    CImage *read (int img_number = 0)
        throw (CException, std::exception);

    int getNumberImages (void)
        throw();

private:
    FILE  *d_hd;          //!< file handle
};


// static class to register the file reader
/*! \class CFileReadPngRegistrator CFileReadPng.h tuv/CFileReadPng.h */
class CFileReadPngRegistrator: public CFileReadRegistrator {
    static const unsigned char d_id[];
    static const int d_len;    
public:

    TUVision::CFileRead* construct() {
        return(new TUVision::CFileReadPng());
    }
    const int getLength() { return d_len; }
    const unsigned char *getID() { return d_id; }

};
} 

#endif // __TUVISION_CFILEREADPNG_H
