/***************************************************************************
                          CFileReadPnm.h  -  description
                             -------------------
    begin                : 08.04.2003
    copyright            : (C) 2003 by Fabian Wenzel
    email                : tuvisionlib@tuvision.de
    origin               : Maintainance

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
#ifndef __TUVISION_CFILEREADPNM_H_
#define __TUVISION_CFILEREADPNM_H_ 1

#include <exception>

#include "TUVisionDef.h"
#include "tuv/CImageDatatypes.h"
#include "tuv/CFileRead.h"
#include "tuv/CException.h"
#include <cstdio>
#include <string>
#include "tuv/CFileReadRegistrator.h"


namespace TUVision {


/**********************************************************/
/*!
 *   This class read an image as a PNM graphic file.
 *
 *   \ingroup io
 *
 *   \author Fabian Wenzel
 *   \class CFileReadPnm CFileReadPnm.h tuv/CFileReadPnm.h
 */
/**********************************************************/

class _CCTUVISION CFileReadPnm: public CFileRead {

public:
    CFileReadPnm (void);
    ~CFileReadPnm();

    void open (const std::string &filename)
        throw (CException);

    void close (void);

    /******************************************************/
    //!  Read 8 bit RGB image BMP fornmat.
    /******************************************************/
    CImage* read (int image_number=0)
        throw (CException, std::exception);

    int getNumberImages (void);

private:
    std::string d_filename;
    void *jfile;
};

// static class to register the file reader
/*! \class CFileReadPnmRegistratorBinary CFileReadPnm.h tuv/CFileReadPnm.h */
class CFileReadPnmRegistratorBinary: public CFileReadRegistrator {
    static const unsigned char d_id[];
    static const int d_len;    
public:

    TUVision::CFileRead* construct() {
        return(new TUVision::CFileReadPnm());
    }

    const int getLength() { return d_len; }
    const unsigned char *getID() { return d_id; }

};

// static class to register the file reader
/*! \class CFileReadPnmRegistratorASCII CFileReadPnm.h tuv/CFileReadPnm.h */
class CFileReadPnmRegistratorASCII: public CFileReadRegistrator {
    static const unsigned char d_id[];
    static const int d_len;    
public:

    TUVision::CFileRead* construct() {
        return(new TUVision::CFileReadPnm());
    }

    const int getLength() { return d_len; }
    const unsigned char *getID() { return d_id; }

};
}

#endif

