/***************************************************************************
                          CFileReadRegistry.h  -  description
                             -------------------
    begin                : 01.01.2001
    copyright            : (C) 2001 by Eric Sommerlade
    email                : tuvisionlib@tuvision.de
    origin               : Studienarbeit/Diplomarbeit Eric Sommerlade

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
#ifndef __CFILEREADREGISTRY_H__
#define __CFILEREADREGISTRY_H__ 1

#include "TUVisionDef.h"

namespace TUVision {
class CFileReadRegistrator;
//! Registry of filereaders used by CFileReadAll.
/*! 
 *  Registers filereaders.
 *
 *  The filereaders must be passed in by a subclass of 
 *  CFileReadRegistrator, which passes back the correct 
 *  file reader.
 *  The registration is done automatically by the 
 *  constructor of the CFileReadRegistrator class.
 *
 *  To register a new file type, subclass a 
 *  CFileReadRegistrator in the implementation file
 *  of your file reader,
 *  and register it like this:
 *  \code
 *  namespace TUVision {
 *    class CFileReadPNGRegistrator: public CFileReadRegistrator {
 *      ...
 *      // virtual function to implement instantiation.
 *      CFileRead* construct() {
 *        return new CFileReadPNG(...options...);
 *      }
 *    };
 *  }
 *  // static initialization registers file reader automatically.
 *  using namespace TUVision;
 *  static CFileReadPNGRegistrator cfrpr;
 *  \endcode
 *
 *  The class implements a singleton.
 *
 *  \author Eric Sommerlade
 *
 *  \class CFileReadRegistry CFileReadRegistry.h tuv/CFileReadRegistry.h
 */ 
class _CCTUVISION CFileReadRegistry {
public:
    //! call this method to get the single instance of the class.
    static CFileReadRegistry& getRegistry();
    //! registers a CFileRead class.
    void registerReader(CFileReadRegistrator* registrator);
    //! called by the CFileReadAll class.
    CFileReadRegistrator* getReader(int len, const unsigned char* id) const;
    //! returns the maximum length of all file read identifier strings.
    int getMaxLen() const;
    //! destroys the file reader registry.
    ~CFileReadRegistry();

private:
    CFileReadRegistry();
};
} // namespace TUVision

#endif // __CFILEREADREGISTRY_H__
