/***************************************************************************
                          CFileWriteJpeg.h  -  description
                             -------------------
    begin                : 01.01.2001
    copyright            : (C) 2001 by Marco Budde
    email                : tuvisionlib@tuvision.de
    origin               : Diplomarbeit Marco Budde

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
#ifndef __TUVISION_CFILEWRITEJPEG_H_
#define __TUVISION_CFILEWRITEJPEG_H_ 1

#include <cstdio>
#include "TUVisionDef.h"
#include "tuv/CImageDatatypes.h"

#include "tuv/CFileWrite.h"
#include "tuv/CException.h"


namespace TUVision {

/**********************************************************/
//!  Saves an image as JPEG graphic file.
/*! 
 *   Saves an image as JPEG graphic file. JPEG is an
 *   often used graphic file format for lossy compressed 
 *   images. Because the compression works really 
 *   effectively, JPEG is a good format for the 
 *   transmission of images over slow communication lines
 *   like modems.
 *
 *   This class creates JPEG files with a quality of
 *   "75" and no progression. It is possible to change
 *   these default values.
 *
 *   \ingroup io
 *
 *   \author Marco Budde
 */
/**********************************************************/

class _CCTUVISION CFileWriteJpeg: public CFileWrite {

public:
    CFileWriteJpeg (void);
    ~CFileWriteJpeg();

    void open (const std::string &filename)
        throw (CException);

    void close (void)
        throw();

    /******************************************************/
    //! Save 8 bit gray image.
    /******************************************************/
    void write (const CImageByteGray &img)
        throw (CException);

    /******************************************************/
    //! Save 8 bit RGB image.
    /******************************************************/
    void write (const CImageByteRGB &img)
        throw (CException);

    /******************************************************/
    //! Set the quality of the JPEG file.  
    /******************************************************/
    void setQuality (int quality)
        throw();

    /******************************************************/
    //! Enable/disable progression.
    /******************************************************/
    void setProgression (bool progression)
        throw();

private:
    /******************************************************/
    //! Save 8 bit image.
    /******************************************************/
    void write (const CImageByte &img, int color_space)
        throw (CException);

private:
    FILE  *d_hd;          //!< file handle
    int        d_quality;      //!< quality/compression level
    bool       d_progression;  //!< Create progressive JPEG?
};

}

#endif
