/***************************************************************************
                          CFileWriteJpeg2k.h  -  description
                             -------------------
    begin                : 07.04.2003
    copyright            : (C) 2003 by Fabian Wenzel
    email                : tuvisionlib@tuvision.de
    origin               : Maintainance

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
#ifndef __TUVISION_CFILEWRITEJPEG2K_H_
#define __TUVISION_CFILEWRITEJPEG2K_H_ 1

#include <exception>

#include "TUVisionDef.h"
#include "tuv/CImageDatatypes.h"

#include "CFileWrite.h"
#include "CException.h"


namespace TUVision {


/**********************************************************/
/*! 
 *   This class saves an image as a Jpeg2k graphic file.
 *
 *   \ingroup io
 *
 *   \author Fabian Wenzel
 */
/**********************************************************/

class _CCTUVISION CFileWriteJpeg2k: public CFileWrite {

public:
    CFileWriteJpeg2k (void);
    ~CFileWriteJpeg2k();

    void open (const std::string &filename)
        throw (CException);

    void close (void)
        throw();


    /******************************************************/
    //!  Save 8 bit gray image in BMP fornmat.
    /******************************************************/
    void write (const CImageByteGray &img)
        throw (CException);

    /******************************************************/
    //!  Save 8 bit RGB image BMP fornmat.
    /******************************************************/
    void write (const CImageByteRGB &img)
        throw (CException);

    /******************************************************/
    //! Set the quality of the JPEG file.  
    /******************************************************/
    void setQuality (float quality)
        throw();

private:
    void *jfile;
    float       d_quality;      //!< quality/compression level

};

}

#endif

