/***************************************************************************
                          CFileWriteTiff.h  -  description
                             -------------------
    begin                : 01.01.2001
    copyright            : (C) 2001 by Marco Budde
    email                : tuvisionlib@tuvision.de
    origin               : Diplomarbeit Marco Budde
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
#ifndef __TUVISION_CFILEWRITETIFF_H_
#define __TUVISION_CFILEWRITETIFF_H_ 1

#include <exception>
typedef struct tiff TIFF;

#include "TUVisionDef.h"
#include "tuv/CImageDatatypes.h"

#include "tuv/CFileWrite.h"
#include "tuv/CException.h"


namespace TUVision {

/**********************************************************/
/*! 
 *   This class saves an image as a TIFF graphic file.
 *   The TIFF format is supported by most image processing
 *   programs. TIFF is a loseless image format.
 *
 *   The following CImageAttributes are supported:
 *   xdpi, ydpi, and description.
 *
 *   \ingroup io
 *
 *   \author Marco Budde
 */
/**********************************************************/

class _CCTUVISION CFileWriteTiff: public CFileWrite {

public:
    CFileWriteTiff (void);
    ~CFileWriteTiff();

    void open (const std::string &filename)
        throw (CException);

    void close (void) 
        throw();

    /******************************************************/
    //!  Save 8 bit bilevel image.
    /******************************************************/
    void write (const CImageByteBilevel &img)
        throw (CException, std::exception);

    /******************************************************/
    //!  Save 8 bit gray image.
    /******************************************************/
    void write (const CImageByteGray &img)
        throw (CException);

    /******************************************************/
    //!  Save 8 bit RGB image.
    /******************************************************/
    void write (const CImageByteRGB &img)
        throw (CException);

    /******************************************************/
    //!  Turn compression on or off?
    /******************************************************/
    void setParameters (bool compression)
        throw();

private:
    /******************************************************/
    //!  Save the attributes of a CImage object.
    /******************************************************/
    bool writeAttributes (const CImage &img);

private:
    TIFF  *d_tif_hd;        //!< handle of libtiff
    bool  d_compression;    //!< Should we compress the image?
};

}

#endif
