/***************************************************************************
                          CImageData.h  -  description
                             -------------------
    begin                : 01.01.2001
    copyright            : (C) 2001 by Marco Budde, Eric Sommerlade, Islam Shdaifat
    email                : tuvisionlib@tuvision.de
    origin               : Diplomarbeit Marco Budde, Eric Sommerlade, PhD Shdaifat

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
#ifndef __TUVISION_CIMAGEDATA_H__
#define __TUVISION_CIMAGEDATA_H__ 1


namespace TUVision {

/**********************************************************/
//! A reference counter and data handler.
/*! 
 *  This class counts references to itself. 
 *  Only if its reference counts reaches zero, the
 *  managed memory is released.
 *
 *  \author Eric Sommerlade
 */
/**********************************************************/

template <class T>
class _CCTUVISION CImageData {

private:

    T* d_data;         //!< the managed data
    int *d_referees;   //!< the number of objects refering here

    /******************************************************/
    //! Private assignment constructor.
    /*! 
     *  Should not be needed.
     */
    /******************************************************/
    CImageData& operator=(const CImageData& ref);

public:

    /******************************************************/  
    //! Constructs an array of the specified size. 
    /*! 
     *  Explicit constructor to avoid implicit type 
     *  conversion (yes, it happens).
     *
     *  \param size  The size number of elements to be 
     *               allocated the elements are all of type
     *               T.
     */
    /******************************************************/  

    explicit CImageData (int size)
    {
        d_data=new T[size];
        d_referees=new int[1];
        *d_referees=1;
    }


    /******************************************************/  
    //! References another array.
    /*! 
     *  \param ref  The object which already handles the 
     *              memory. Only the reference count is 
     *              increased.
     */
    /******************************************************/  

    CImageData (const CImageData& ref) 
    {
        d_referees=ref.d_referees;
        d_data=ref.d_data;
        (*d_referees)++;
    }

    /******************************************************/  
    //! Destroys the object.
    /*! 
     *  Decreases the reference count, if it reaches zero, 
     *  the memory is released.
     */
    /******************************************************/  

    ~CImageData()
    {
        (*d_referees)--;
        if(*d_referees==0) {
            delete [] d_referees;
            delete [] d_data;
        }
    }


    /******************************************************/  
    //! Array subscript operator.
    /*! 
     *  Returns the element pointed to by the specified 
     *  position.
     *
     *  \param pos the position
     */
    /******************************************************/  

    T& operator[] (int pos) 
    {
        return d_data[pos];
    }


    /******************************************************/  
    //! Type conversion operator.
    /*! 
     *  Useful to handle CImageData<T> objects as arrays 
     *  of type T:
     *
     *  \code 
     *    CImageData<char> data(100);
     *    char *ptr=(char*)data;
     *  \endcode
     */
    /******************************************************/  

    operator T*() 
    {
        return d_data;
    }

};
} // namespace TUVision

#endif //__TUVISION_CIMAGEDATA
