/***************************************************************************
                          CMinpack.h  -  description
                             -------------------
    changed              : 06.08.2003
    copyright            : (C) 2003 Fabian Wenzel
    email                : tuvisionlib@tuvision.de
    origin               : PhD Fabian Wenzel

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
#ifndef __TUVISION_CMINPACK_H__
#define __TUVISION_CMINPACK_H__ 1

#include "TUVisionDef.h"
#include <vector>

namespace TUVision {

/**********************************************************/
//!  Encapsulates calls to minpack optimization routines
/*!  
 *   This class is not thread safe! run() calls have
 *   to be serialized when running more threads, otherwise
 *   the cost function would be called with undefined user
 *   data pointers and cost function pointers.
 *
 *   Usage: Initialize an instance with a pointer
 *   to the cost function, a vector only containing
 *   the initial guess (no more parameters, otherwise
 *   they would be optimized, too, so please watch out
 *   for the correct size!), and the length of the
 *   cost vector. This correctly sized cost vector
 *   gets
 *
 * 
 *   \todo If somebody wants to change minpack's optional
 *         parameters, please add functionality
 *
 *   \author Fabian Wenzel
 */
/**********************************************************/
class _CCTUVISION CMinpack
{
public:
    typedef void (*Tcostfunction)(const std::vector<double> &intermediate,
        std::vector< double > *costvector, void *userdata);

    CMinpack(Tcostfunction costfunction, std::vector<double> initialguess, int costvectorlength);
    std::vector<double> &run(void *userdata = NULL);

    void setMaxIterations  (int maxiterations);
    int  getNumOfIterations();

    void                  *m_userdata;
    Tcostfunction          m_costfunction;
    std::vector< double >  m_costvector;
    std::vector< double >  m_solution;
    int                    m_iterations;
    int                    m_maxiterations;
};

}

#endif
