/***************************************************************************
                          CPointShape.h  -  description
                             -------------------
    begin                : 10.04.2001
    copyright            : (C) 2001 by Fabian Wenzel
    email                : tuvisionlib@tuvision.de, f.wenzel@gmx.net
    origin               : Diplomarbeit Fabian Wenzel

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
#ifndef __TUVISION_CPOINTSHAPE_H__
#define __TUVISION_CPOINTSHAPE_H__ 1

#include "tuv/CRect.h"
#include "tuv/CPoint.h"
#include <list>

using namespace std;

namespace TUVision{

/**********************************************************/ 
//!  Defines a class for the contour of an object in an image
/*! 
 *   When finding objects in an image, one possibility is
 *   by running around it's shape (see CExtractShape). This class
 *   serves as a data type for storing its information. It has a list of
 *   points which belong to an object's shape and a bounding box which
 *   has the maximum and minimum coordinates of the list for x- and y-direction.
 *
 *   \author Fabian Wenzel
 */
class _CCTUVISION CPointShape {
public:
    CPointShape();
    ~CPointShape();
    CPointShape(const CPointShape &other);
    list<CPoint> *getShapePoints() { return d_ShapePoints; };
    void setShapePoints( list<CPoint> *newPointList ) { delete d_ShapePoints; d_ShapePoints = newPointList; }
    const CRect &getBoundingBox() const { return d_BoundingBox; };
    void addPointUpdateBBX(CPoint &newPoint);
    void addPoint(CPoint &newPoint) { d_ShapePoints->push_back(newPoint); }
    bool containsPoint(CPoint &testPoint);
    void setBBX(CRect BBX) { d_BoundingBox = BBX; }
private:
    bool d_empty;
    CRect d_BoundingBox;
    list<CPoint> *d_ShapePoints;
};
}

#endif
