/***************************************************************************
                          CRect.h  -  description
                             -------------------
    begin                : 01.01.2001
    copyright            : (C) 2001 by Eric Sommerlade
    email                : tuvisionlib@tuvision.de
    origin               : Diplomarbeit Eric Sommerlade

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
#ifndef __TUVISION_CRECT_H__
#define __TUVISION_CRECT_H__ 1

#include "TUVisionDef.h"
#include "tuv/CPoint.h"

namespace TUVision {

/*! This class encapsulates a rectangular area bounded by integer values.
 *  It is useful for image applications.
 *
 *  The rectangle starts from the upper left corner
 *  and contains all points up to 
 *  (not including) the lower right corner.
 *
 *  \author Eric Sommerlade
 */
class _CCTUVISION CRect {
public:
    //! constructs a rectangle ( 0, 0 ) - ( w, h )
    explicit CRect(int w, int h);
    explicit CRect(int ulx, int uly, int lrx, int lry);
    explicit CRect(CPoint ul, CPoint lr);

    //! moves the upper left corner
    CRect& setUpperLeft(int x, int y);
    //! moves the lower right corner
    CRect& setLowerRight(int x, int y);
    //! adjusts the upper left corner 
    CRect& setCenter(int x, int y);
    //! sets the new coordinates of the rectangle
    void setRect(int ulx, int uly, int lrx, int lry);
    //! sets the new coordinates of the rectangle
    void setRect(CPoint ul, CPoint lr);
    //! adjusts the size of the rectangle
    CRect& setSize(int w, int h);
    //! fits the corners to contain the point (w,h)
    CRect& fitSize(int w, int h);

    //! returns the intersection of both rectangles.
    CRect intersect(const CRect& r) const;
    //! returns true if the rectangles intersect.
    bool intersects(const CRect& r) const;

    //! returns true if the point in inside of the rectangle.
    bool contains(int x, int y) const;
    //! returns true if the rectangle in inside of this rectangle.
    bool contains(const CRect& r) const;
    //! returns true if the point is inside the rectangle
    bool contains(const CPoint point) const;

    //! returns the width of the rectangle.
    int getW() const;
    //! returns the height of the rectangle.
    int getH() const;
    //! returns the x position of the rectangle's center.
    int getCenterX() const;
    //! returns the y position of the rectangle's center.
    int getCenterY() const;
    //! returns the x position of the rectangle's upper left corner.
    int getX() const;
    //! returns the y position of the rectangle's upper left corner.
    int getY() const;
    //! returns the lower right x coordinate of the rectangle
    int getLRX() const;
    //! returns the lower right y coordinate of the rectangle
    int getLRY() const;
    //! returns the upper left point of the rectangle
    CPoint getUpperLeft() const;
    //! returns the lower right point of the rectangle
    CPoint getLowerRight() const;

    //! returns true if the rectangle contains no point at all.
    bool isNull() const;

private:
    int d_x; //!< x coordinate of upper left corner position  
    int d_y; //!< x coordinate of upper left corner position  
    int d_w; //!< rectangle width (in pixels) 
    int d_h; //!< rectangle height (in pixels) 
};

bool _CCTUVISION operator==(const CRect& r1, const CRect& r2);

void _CCTUVISION dump(const CRect& r);
} // namespace TUVision


#endif
