/***************************************************************************
                          CCorrelator.h  -  description
                             -------------------
    begin                : 01.01.2001
    copyright            : (C) 2001 by Islam Shdaifat
    email                : tuvisionlib@tuvision.de
    origin               : PhD - Thesis Islam Shdaifat

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
#ifndef __TUVISION_CCORRELATOR_H__
#define __TUVISION_CCORRELATOR_H__ 1

#include "TUVisionDef.h"
#include "tuv/CImageDatatypes.h"

namespace TUVision {

/**********************************************************/
//!  calculate the correlation coeffecient of two mages.
/*!
 *   This class can be used to calculate the average, variance,
 *   and correaltion between two images.
 *
 *   \ingroup operator
 *
 *   \author Eric Sommerlade
 *   \author Islam Shdaifat
 */
/**********************************************************/

/******************************************************/
//! calculate the average of image.
/******************************************************/
double _CCTUVISION 
    calcAvg(const CImageByteGray& img, const int step = 1);

/******************************************************/
//! calculate the variance of image.
/******************************************************/
double _CCTUVISION 
    calcVar(const CImageByteGray& img,const double& avg, const int step = 1);

/******************************************************/
//! calculate the correlation of two images, unknown avr, var.
/******************************************************/
double _CCTUVISION 
    correlateImg1Img2(const CImageByteGray& im1,const CImageByteGray& im2, const int step = 1);

/******************************************************/
//! calculate the correlation of two images, known avr, var.
/******************************************************/
double _CCTUVISION 
    correlateImg1Img2(const CImageByteGray& im1,const CImageByteGray& im2,
    const double& im1avg,const double& im1var, const int step = 1);
/******************************************************/
//! calculate the correlation of two images, unknown avr, var.
/******************************************************/
double _CCTUVISION 
    correlateImg1Img2(const CImageByteGray& im1,const CImageByteGray& im2,
    const double& im1avg,const double& im1var,
    const double& im2avg,const double& im2var, const int step = 1);

/******************************************************/
//! returs an image contains the corr. coef. by shifting
//! the small image one pixel.
/******************************************************/
CImageByteGray  _CCTUVISION 
    *correlateImgSmallImgLarge(
    CImageByteGray& imLarge,
    const CImageByteGray& imSmall,
    int* x_max, int* y_max, double* corr_max,
    int step = 1);

}  // namespace TUVision

#endif //__TUVISION_CCORRELATOR_H__
