/***************************************************************************
                          CExtractorShape.h  -  description
                             -------------------
    begin                : 10.04.2001
    copyright            : (C) 2001 by Fabian Wenzel
    email                : tuvisionlib@tuvision.de, f.wenzel@gmx.net
    origin               : Diplomarbeit Fabian Wenzel

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
#ifndef __TUVISION_CEXTRACTORSHAPE_H__
#define __TUVISION_CEXTRACTORSHAPE_H__ 1

#include "tuv/CImageByteGray.h"
#include "tuv/CImageIntGray.h"
#include "TUVisionDef.h"
#include "tuv/CPointShape.h"
#include "tuv/CPoint.h"
#include <list>
#include <vector>

using namespace std;

namespace TUVision{

/**********************************************************/ 
//!  Defines a class for extracting an object's contour
/*! 
 *   Objects in an image can be searched by an automaton which
 *   runs around connected components (see Diplomarbeit of Fabian Wenzel).
 *   This class searches a gray image for objects which are characterized
 *   by non-zero intensities (i.e. the "background" is black). It returns
 *   a list of CPointShapes which each contains the contour pixels of an object
 *   in the image. This method works fine but may fail if there are complex setups
 *   with intersecting non-convex objects (also described in the Diplomarbeit).
 *
 *   \ingroup extractors
 *
 *   \author Fabian Wenzel
 */

class _CCTUVISION CExtractorShape{
public:
    CExtractorShape();
    ~CExtractorShape();
    list <CPointShape *> *extractShapes(const CImageByteGray *image, bool ObjectContentIsZero=false, bool ShapeInsideOfObject=true);
private:
    void runAroundOutside(int firstdir, CPoint startpoint, bool firstrun);
    void runAroundInside(int firstdir, CPoint startpoint, bool firstrun);
    int updateShapesMaskInside(int startline);
    int updateShapesMaskOutside(int startline);
    void updateHashVectorInside(CPoint &newPoint);
    void updateHashVectorOutside(CPoint &newPoint);
    int extractNextShapeOutside(CPoint startPoint);
    int extractNextShapeInside(CPoint startPoint);
    list <CPointShape *> *d_shapes;
    int d_nextfreecoordinate;
    CPointShape *d_currentShape;
    const CImageByteGray *p_image;
    CImageIntGray *d_detectedShapesMask;
    CPoint d_startPoint;
    bool d_objectContentIsZero;
    bool d_shapeInsideOfObject;
    struct minmaxline{
        int min;
        int max;
    };
    minmaxline *d_maskHashVector;
    std::vector< const unsigned char *> d_row;
};

}


#endif
