/***************************************************************************
                          CFilterConvolution.h  -  description
                             -------------------
    begin                : 01.01.2001
    copyright            : (C) 2001 by Marco Budde
    email                : tuvisionlib@tuvision.de
    origin               : Diplomarbeit Marco Budde

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
#ifndef __TUVISION_CFILTERCONVOLUTION_H__
#define __TUVISION_CFILTERCONVOLUTION_H__ 1

#include "TUVisionDef.h"
#include "tuv2d/CFilterGray.h"

namespace TUVision 
{

/**********************************************************/
//!  A generic convolution filter.
/*! 
 *   This class implements a generic convolution filter.
 *   It uses the "centered, zero boundary" method
 *   described in Pratt's "Digital Image Processing",
 *   page 175. The other methods could be emulated by
 *   creating a image with a matching boundary using 
 *   insertSubImage() and setWindow().
 *
 *   \warning At the moment the result is not scaled!
 *            So you can get a lot of overflows. We
 *            should fix this in the next release.
 *
 *   \ingroup filter
 *
 *   \author Marco Budde
 */
/**********************************************************/

class _CCTUVISION CFilterConvolution : public CFilterGray {

public:
    CFilterConvolution();
    ~CFilterConvolution();

    /******************************************************/
    //! Sets the convolution matrix (byte type).
    /******************************************************/
    void setParameters (int x, int y, const char *matrix_c,
        char scaling_c);

    /******************************************************/
    //! Sets the convolution matrix (float type).
    /******************************************************/
    void setParameters (int x, int y, const float *matrix_f,
        float scaling_f);

    /******************************************************/
    //! Filter the given image (byte gray).
    /******************************************************/
    void filter (CImageByteGray &img);

    /******************************************************/
    //! Filter the given image (int gray).
    /******************************************************/
    void filter (CImageIntGray &img);

    /******************************************************/
    //! Filter the given image (double gray).
    /******************************************************/
    void filter (CImageDoubleGray &img);

private:
    char  *d_matrix_c;        //!< convolution matrix (char)
    float *d_matrix_f;        //!< convolution matrix (float)
    char  d_scaling_c;        //!< scaling factor (char)
    float d_scaling_f;        //!< scaling factor (float)
    bool  d_matrix_float;     //!< Use the float conv. matrix?
    int   d_width, d_height;  //!< dimensions of the conv. matrix
};


}

#endif // __TUVISION_CFILTERCONVOLUTION_H__
