/***************************************************************************
                          CFilterHistogram.h  -  description
                             -------------------
    begin                : 01.01.2001
    copyright            : (C) 2001 by Marco Budde
    email                : tuvisionlib@tuvision.de
    origin               : Diplomarbeit Marco Budde

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
#ifndef __TUVISION_CFILTERHISTOGRAM_H__
#define __TUVISION_CFILTERHISTOGRAM_H__ 1

#include "tuv2d/CFilterByteGray.h"
#include "tuv2d/CFilterIntGray.h"

#include "TUVisionDef.h"
#include "tuv/CImageDatatypes.h"


namespace TUVision {


/**********************************************************/
//! Filter an image based on its histogram.
/*!
 *  This filter manipulates an image based on its
 *  histogram. The filtering takes place in three steps:
 *  
 *  1.) Streching: set the black and white points in
 *      the original image.
 *  2.) Gamma Correction
 *  3.) Compression: set the min and max values used by
 *      the filtered output image.
 *
 *  \ingroup filter
 *
 *  \author Marco Budde
 */
/**********************************************************/

class _CCTUVISION CFilterHistogram : public CFilterByteGray, 
    public CFilterIntGray {

public:

    /******************************************************/
    //! Constructor.
    /******************************************************/
    CFilterHistogram();

    /******************************************************/
    //! Set streching values.
    /******************************************************/
    void setStreching (int black, int white);

    /******************************************************/
    //! Set gamma correction value.
    /******************************************************/
    void setGammaCorrection (double gamma);

    /******************************************************/
    //! Set compression values.
    /******************************************************/
    void setCompression (int black, int white);

    /******************************************************/
    //! Disables streching, gamma, and compression.
    /******************************************************/
    void reset (void);

    void filter (CImageByteGray &img);
    void filter (CImageIntGray &img);

private:
    int d_strech_black;    //!< streching black point
    int d_strech_white;    //!< streching white point
    bool d_streching;      //!< Use streching?
    double d_gamma;        //!< gamma correction value
    bool d_gamma_correction;  //!< Use gamma correction?
    int d_comp_black;      //!< compression min
    int d_comp_white;      //!< compression max
    bool d_compression;    //!< Use compression?
    };
}

#endif  // __TUVISION_CFILTERHISTOGRAM_H_
