/***************************************************************************
                           CFilterMaxMin.h  -  description
                              -------------------
     begin                : 30.08.2002
     copyright            : (C) 2002 Marc Hensel
     email                : tuvisionlib@tuvision.de, marc.hensel@web.de
     origin               : Diploma thesis Marc Hensel
 
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/

#ifndef __TUVISION_CFILTERMAXMIN_H__
#define __TUVISION_CFILTERMAXMIN_H__	1

#include <tuv/CImageByteGray.h>

#include <TUVisionDef.h>

namespace TUVision 
{

/**********************************************************/
//!  Maximum and minimum filter.
/*! 
 *   This class implements filters setting pixels with the 
 *   maximum or minimum value in the filter window. Window 
 *   dimensions must be odd. The image remains unchanged when 
 *   trying to set to an even value.
 *
 *   \ingroup filter
 *
 *   \author Marc Hensel
 *
 *   \class CFilterMaxMin CFilterMaxMin.h tuv2d/CFilterMaxMin.h
 */
/**********************************************************/

class _CCTUVISION CFilterMaxMin
{
private:
    unsigned char	filterWidth;	//!< Filter width (must be odd value)

public:
    //! Constructor (sets filter width to 3 if called without argument)
    CFilterMaxMin	(unsigned char width=3);

    //! Destructor
    ~CFilterMaxMin	();

    //! Set filter width (must be odd value)
    bool			setWidth	(unsigned char width);

    //! Returns filter width
    unsigned char	getWidth	();

    //! Filter the image (maximum)
    void			filterMax	(CImageByteGray &img);

    //! Filter the image (minimum)
    void			filterMin	(CImageByteGray &img);
};

}

#endif	// __TUVISION_CFILTERMAXMIN_H__
