/***************************************************************************
                          CLineSearch.h  -  description
                             -------------------
    begin                : 1.1.2000
    copyright            : (C) 2000 by Islam Shdaifat
    email                : tuvisionlib@tuvision.de
    origin               : PhD - Thesis Islam Shdaifat
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
//CLineSearch.h
#ifndef __TUVISION_CLINESEARCH_H_
#define __TUVISION_CLINESEARCH_H_ 1

#include "tuv2d/CCurveSearch.h"
#include "tuv2d/CLine.h"
#include "TUVisionDef.h"
#include "tuv/CImageDatatypes.h"


#include <vector>
#include "tuv2d/CPointXY.h"
#include "tuv2d/CCurve.h"

namespace TUVision {


/**********************************************************/
/*! 
 *    This class implements the search method for line
 *    in a binary image i.e edge image. The edge image is
 *    definded as 0 no edge and 255 edge.
 *
 *   \ingroup extractors
 *    \author Islam Shdaifat
 */
/**********************************************************/
class _CCTUVISION CLineSearch: public CCurveSearch{

public:

    //! default constructor
    CLineSearch(){};

    //! constructor of line search object with a line curve object
    CLineSearch(CLine& line,
        double dThr0 = 0.2,
        double dThr1 = 0.8);

    //! special search for the line
    inline void searchSpecial();

    void connectLineSegments();
    void search(CImageByteGray *img, int x0, int y0);

protected:

    //! for the line check if the current pixel is 255 (on)
    inline bool checkSpecialCondition ();
    inline void getHitAndNonHitCount(int pos);
    //! write the search result in output stream
    void writeToFeatureStream();


};

} // namespace TUVision

#endif  // __TUVISION_CLINESEARCH_H_
