/***************************************************************************
                          COperatorDraw.h  -  description
                             -------------------
    begin                : 01.01.2001
    copyright            : (C) 2001 by Marco Budde
    email                : tuvisionlib@tuvision.de
    origin               : Diplomarbeit Marco Budde

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
#ifndef __TUVISION_COPERATORDRAW_H__
#define __TUVISION_COPERATORDRAW_H__ 1

#include <string>

#include "TUVisionDef.h"
#include "tuv/CImageDatatypes.h"


namespace TUVision {

/**********************************************************/
//!  Class to draw in an image.
/*!
 *   This class could be used to draw in an image.
 *   For example it is possible to draw lines and circles.
 *
 *   \ingroup operator
 *
 *   \author Marco Budde
 */
/**********************************************************/

class _CCTUVISION COperatorDraw
{
public:

    /******************************************************/
    //! Draws a line.
    /******************************************************/
    virtual void drawLine (int x_0, int y_0, int x_d, int y_d);

    /******************************************************/
    //! Draws a circle.
    /******************************************************/
    virtual void drawCircle (int x_0, int y_0, int r, 
        bool filled = false);

    /******************************************************/
    //! Draws an ellipse.
    /******************************************************/
    virtual void drawEllipse (int x_0, int y_0, int a, int b, 
        bool filled = false);

    /******************************************************/
    //! Draws a text.
    /******************************************************/
    virtual void drawText (int x, int y, const std::string &text,
        int scale = 1);

    /******************************************************/
    //! Draws a crosshair.
    /******************************************************/
    virtual void drawCrosshair (int x, int y, int length, 
        int scale, int distance);

    /******************************************************/
    //! Draws a single pixel.
    /******************************************************/
    virtual void drawPixel(int x, int y);


protected:

    /******************************************************/
    //! Sets the value of the specified pixel.
    /*!
     *  Sets the value of the specified pixel. It is ok
     *  to use this function with pixels outside the
     *  image dimensions of the CImage object. Such calls 
     *  are simply ignored.
     *
     *  \param x  x-coordinate of the pixel
     *  \param y  y-coordinate of the pixel
     */
    /******************************************************/
    virtual void setPixel (int x, int y) =0;

    /******************************************************/
    //! Get the CImage object used by this object.
    /*!
     *  Returns the CImage object used by this class.
     *  This could be useful to check the dimensions of
     *  the image.
     *
     *  \return pointer to the internal CImage object
     */ 
    /******************************************************/
    virtual const CImage *getImage (void) const =0;
};



/**********************************************************/
//!  Class to draw in a CImageByteGray object.
/*!
 *   This class could be used to draw in CImageByteGray 
 *   object. For example it is possible to draw lines and 
 *   circles.
 *
 *   \ingroup operator
 *
 *   \author Marco Budde
 */
/**********************************************************/

class _CCTUVISION COperatorDrawByteGray : public COperatorDraw
{
public:

    /******************************************************/
    //! Constructor.
    /******************************************************/
    COperatorDrawByteGray();

    /******************************************************/
    //! Sets the color used to draw in the image.
    /******************************************************/
    void setColor (unsigned char color);

    /******************************************************/
    //! Sets the CImageByteGray object to be used.
    /******************************************************/
    void setImage (CImageByteGray *img);

private:
    void setPixel (int x, int y);
    const CImage *getImage (void) const;

private:
    CImageByteGray *d_img;   //!< image to be used
    unsigned char  d_color;  //!< draw color to be used
};



/**********************************************************/
//!  Class to draw in a CImageByteRGB object.
/*!
 *   This class could be used to draw in CImageByteRGB 
 *   object. For example it is possible to draw lines and 
 *   circles.
 *
 *   \ingroup operator
 *
 *   \author Marco Budde
 */
/**********************************************************/

class _CCTUVISION COperatorDrawByteRGB : public COperatorDraw
{
public:

    /******************************************************/
    //! Constructor.
    /******************************************************/
    COperatorDrawByteRGB();

    /******************************************************/
    //! Sets the color used to draw in the image.
    /******************************************************/
    void setColor (unsigned char r, unsigned char g,
        unsigned char b);

    /******************************************************/
    //! Sets the CImageByteRGB object to be used.
    /******************************************************/
    void setImage (CImageByteRGB *img);

private:
    void setPixel (int x, int y);
    const CImage *getImage (void) const;

private:
    CImageByteRGB  *d_img;     //!< image to be used
    unsigned char  d_color_r;  //!< draw color to be used
    unsigned char  d_color_g;  //!< draw color to be used
    unsigned char  d_color_b;  //!< draw color to be used
};

}

#endif  // __TUVISION_COPERATORDRAW_H__

