/***************************************************************************
                          COperatorHistogram.h  -  description
                             -------------------
    begin                : 01.01.2001
    copyright            : (C) 2001 by Islam Shdaifat
    email                : tuvisionlib@tuvision.de
    origin               : PhD - Thesis Islam Shdaifat

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
#ifndef __TUVISION_COPERATORHISTOGRAM_H__
#define __TUVISION_COPERATORHISTOGRAM_H__ 1

#include "TUVisionDef.h"
#include "tuv/CImageDatatypes.h"

namespace TUVision {


/**********************************************************/
//!  Class to calculate the histogram of an image.
/*!
 *   This class calculates the histogram of a given
 *   image. Additional there are some functions to
 *   analyse the calculated histogram.
 *
 *   \ingroup operator
 *
 *   \author Islam Shdaifat
 *   \author Marco Budde
 */
/**********************************************************/

class _CCTUVISION COperatorHistogram {

public:

    /******************************************************/
    //! Constructor.
    /******************************************************/
    COperatorHistogram();

    /******************************************************/
    //! Destructor.
    /******************************************************/
    ~COperatorHistogram();

    /******************************************************/
    //! Sets the image to be used (byte gray).
    /******************************************************/
    void setImage (const CImageByteGray &img);

    /******************************************************/
    //! Sets the image to be used (integer gray).
    /******************************************************/
    void setImage (const CImageIntGray &img);

    /******************************************************/
    //! Get a pointer to the histogram.
    /******************************************************/
    const int *getHistogramData (void) const;

    /******************************************************/
    //! Get the size of the histogram.
    /******************************************************/
    int getHistogramSize (void) const;

    /******************************************************/
    //! Get max index of a nonzero value.
    /******************************************************/
    int getMaxNonZero(void) const;

    /******************************************************/
    //! Get the black point of the histogram.
    /******************************************************/
    int getBlackpoint (int threshold) const;

    /******************************************************/
    //! Get the white point of the histogram.
    /******************************************************/
    int getWhitepoint (int threshold) const;

    /******************************************************/
    //! Histogram Equalization.
    /******************************************************/
    void equalizeImage(CImageByteGray *img1, CImageByteGray *img2 = 0);

    /******************************************************/
    //! create Histogram-Image in Range [min..max]
    /******************************************************/
    CImageByteGray*
    getHistogramImage(const int min, const int max, const bool scale = false);

private:
    int *d_histogram_data;  //!< histogram data
    int d_histogram_size;   //!< number of histogram data elements
};
}

#endif  // __TUVISION_COPERATORHISTOGRAM_H_
