/***************************************************************************
                          COperatorMaxColDiff.h  -  description
                             -------------------
    begin                : 01.01.2001
    copyright            : (C) 2001 by Islam Shdaifat
    email                : tuvisionlib@tuvision.de
    origin               : PhD - Thesis Islam Shdaifat

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
#ifndef __TUVISION_COPERATORMAXCOLDIFF_H__
#define __TUVISION_COPERATORMAXCOLDIFF_H__ 1

#include "TUVisionDef.h"
#include "tuv/CImageDatatypes.h"

namespace TUVision {


/*! \file COperatorMaxColDiff.h
 *  This header file contains the definition of COperatorMaxColDiff,
 *  which imlements the maximum color difference operator. The
 *  operator work on each color channel R, G, and B for each pixel
 *  the color difference is calculated in the horizontal and vertical
 *  direction. For the horizontal for each pixel we calculate the
 *  absolute difference of the average of the three pixels before and
 *  after the current pixel. ---p---  (---p) -> avg1 (p---) avg2
 *  diff = abs(avg1 - avg2) horizontal for each channel. Add the diff
 *  for the whole channels.
 *  The same is done the vertical direction, and the vertical and horizontal
 *  diff are added together.
 *  
 *  \author Islam Shdaifat
 */

//! calculate the maximum color difference.
CImageByteGray _CCTUVISION
    getMaxColDiff (const CImageByteRGB &rgb);

//! calculate the maximum difference of the image.
CImageByteGray _CCTUVISION
    getMaxDiff (const CImageByteGray &img);

//! calculate the maximum difference of the image.
CImageIntGray _CCTUVISION
    getMaxColDiffInt (const CImageByteRGB &rgb);

CImageByteGray _CCTUVISION
    getMaxDiffGray (const CImageByteGray &img);


} // namespace TUVision


#endif // __TUVISION_COPERATORMAXCOLDIFF_H__
