/***************************************************************************
                          COperatorScaling.h  -  description
                             -------------------
    begin                : 01.01.2001
    copyright            : (C) 2001 by Islam Shdaifat, Marco Budde
    email                : tuvisionlib@tuvision.de
    origin               : Diplomarbeit Marco Budde

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
#ifndef __TUVISION_COPERATORSCALING_H__
#define __TUVISION_COPERATORSCALING_H__ 1


#include "TUVisionDef.h"
#include "tuv/CImageDatatypes.h"

namespace TUVision {

/**********************************************************/
//!  Scaling of a CImage.
/*!
 *   This class can be used to generate a scaled
 *   version of an existing CImage object. The size of
 *   the output image is defined by a scaling factor.
 *
 *   \ingroup operator
 *
 *   \author Islam Shdaifat, Marco Budde
 */
/**********************************************************/

class _CCTUVISION COperatorScaling
{
public:

    /******************************************************/
    //! Constructor
    /******************************************************/
    COperatorScaling(double scaleFactor = 1.0);

    /******************************************************/
    //! Sets the scaling factor.
    /******************************************************/
    void setScalingFactor (double scaling, double scalingY = -1);

    /******************************************************/
    //! Generate a scaled image (byte gray).
    /******************************************************/
    CImageByteGray *scale (const CImageByteGray &img)
        const;

    /******************************************************/
    //! Generate a scaled image (byte RGB).
    /******************************************************/
    CImageByteRGB *scale (const CImageByteRGB &img) 
        const;

    /******************************************************/
    //! Generate a scaled image (byte YUV).
    /******************************************************/
    CImageByteYUV *scale (const CImageByteYUV &img)
        const;

    /******************************************************/
    //! Generate a scaled image (integer gray).
    /******************************************************/
    CImageIntGray *scale (const CImageIntGray &img)
        const;

    /******************************************************/
    //! Generate a scaled image (integer RGB).
    /******************************************************/
    CImageIntRGB *scale (const CImageIntRGB &img)
        const;

    /******************************************************/
    //! Generate a scaled image (integer YUV).
    /******************************************************/
    CImageIntYUV *scale (const CImageIntYUV &img)
        const;

    /******************************************************/
    //! Generate a scaled image (double gray).
    /******************************************************/
    CImageDoubleGray *scale (const CImageDoubleGray &img)
        const;

    /******************************************************/
    //! Generate a scaled image (double RGB).
    /******************************************************/
    CImageDoubleRGB *scale (const CImageDoubleRGB &img)
        const;

    /******************************************************/
    //! Generate a scaled image (double YUV).
    /******************************************************/
    CImageDoubleYUV *scale (const CImageDoubleYUV &img)
        const;

    /******************************************************/
    //! Generate a scaled image (bilevel).
    /******************************************************/
    CImageByteBilevel *scale (const CImageByteBilevel &img)
        const;


private:

    /******************************************************/
    //! Generate a scaled image (byte).
    /******************************************************/
    void scalingGeneric (const CImageByte &img_in, 
        CImageByte &img_out, 
        int nchannels) const;

    /******************************************************/
    //! Generate a scaled image (integer).
    /******************************************************/
    void scalingGeneric (const CImageInt &img_in, 
        CImageInt &img_out, 
        int nchannels) const;

    /******************************************************/
    //! Generate a scaled image (double).
    /******************************************************/
    void scalingGeneric (const CImageDouble &img_in, 
        CImageDouble &img_out, 
        int nchannels) const;

private:
    double d_scaling;  //!< scaling factor
    double d_scalingY; //!< scaling factor
};
}

#endif  // __TUVISION_COPERATORSCALING_H__
