/***************************************************************************
                          COperatorThreshold.h  -  description
                             -------------------
    begin                : 03.05.2002
    copyright            : (C) 2001 by Islam Shdaifat, Marc Hensel (1)
    email                : tuvisionlib@tuvision.de
    origin               : PhD - Thesis Islam Shdaifat, study thesis Marc Hensel (1)

 ***************************************************************************/

/*
 * (1) Following method added:
 *     void threshold(CImageByteGray *img,unsigned char level);
 */


/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
#ifndef __TUVISION_COPERATORTHRESHOLD_H__
#define __TUVISION_COPERATORTHRESHOLD_H__ 1

#include "TUVisionDef.h"
#include "tuv/CImageDatatypes.h"

namespace TUVision {



/**********************************************************/
//!  Threshold an image.
/*! 
 *   Convert a gray level image to a bilevel image
 *   using a threshold. 
 *
 *   \ingroup operator
 *
 *   \author Marco Budde, Marc Hensel
 *
 *   \class COperatorThreshold COperatorThreshold.h tuv2d/COperatorThreshold.h
 */
/**********************************************************/

class  _CCTUVISION COperatorThreshold {

public:

    /******************************************************/
    //! Threshold gray level image (byte format).
    /******************************************************/
    void threshold (const CImageByteGray &src, 
        CImageByteBilevel **tar,
        unsigned char level);

    /******************************************************/
    //! Threshold gray level image (byte format, source and target identical).
    /******************************************************/
    void threshold (CImageByteGray *img,unsigned char level,unsigned char low=0,unsigned char high=255);

    /******************************************************/
    //! Threshold gray level image (integer format).
    /******************************************************/
    void threshold (const CImageIntGray &src, 
        CImageByteBilevel **tar,
        int level);

    /******************************************************/
    //! Threshold gray level image (double format).
    /******************************************************/
    void threshold (const CImageDoubleGray &src, 
        CImageByteBilevel **tar,
        double level);

};
} 

#endif // __TUVISION_COPERATORTHRESHOLD_H__
