/***************************************************************************
                          COperatorTransform.h  -  description
                             -------------------
    begin                : 1.1.2000
    copyright            : (C) 2000 by Islam Shdaifat
    email                : tuvisionlib@tuvision.de
    origin               : PhD - Thesis Islam Shdaifat

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
#ifndef __TUVISION_COPERATORTRANSFORM_H__
#define __TUVISION_COPERATORTRANSFORM_H__ 1


#include "TUVisionDef.h"
#include "tuv/CImageDatatypes.h"

namespace TUVision {

/*
enum {
XSHEAR,
YSHEAR,
ROTATE
// The other cases
} TransformType;
*/

/**********************************************************/
//!  Image Transformation
/*!
 *   This class can be used in the geometrical transformation
 *   The output of the transformation is an image of the same
 *   type like the input image
 *   The transformations are IntegralRotation, XShear, YShear,
 *   Rotation. 
 *   This calss is mainly adopted for the rotation around the
 *   image's center.
 *   Later Projective, BiLinear etc.
 *
 *   \ingroup operator
 *
 *   \author Islam Shdaifat
 */
/**********************************************************/

class _CCTUVISION COperatorTransform
{
public:

    /******************************************************/
    //! Constructor
    /******************************************************/
    COperatorTransform();

    static CImageByteGray*
        integralRotateImageByte(const CImageByteGray& image,
        unsigned int rotations);
    static CImageByteRGB*
        integralRotateImageRGB(const CImageByteRGB& image,
        unsigned int rotations);

    static CImageByteGray*
        rotateImageByte(const CImageByteGray& image,
        double angle);

    static CImageByteRGB*
        rotateImageRGB(const CImageByteRGB& image,
        double angle);

    static CImageByteGray*
        rotateImageByte1(const CImageByteGray& image,
        double angle);

    static CImageByteGray*
        mirrorImageByte(const CImageByteGray& image);

    static CImageByteGray*
        yshearImageByte(const CImageByteGray& image,
        double angle, int y_offset);			

    static CImageByteGray*
        yshearImageByte(const CImageByteGray& image,
        double angle);

    static CImageByteGray*
        xshearImageByte(const CImageByteGray& image,
        double angle);

    void static
        padImageByte(CImageByteGray& rotated_image,
        int x_pad, int y_pad);


private:


private:
    //double d_scaling;  //!< scaling factor

};
}

#endif  // __TUVISION_COPERATORTRANSFORM_H__
