/***************************************************************************
                          CPointXY.h  -  description
                             -------------------
    begin                : 1.1.2000
    copyright            : (C) 2000 by Islam Shdaifat
    email                : tuvisionlib@tuvision.de
    origin               : PhD - Thesis Islam Shdaifat
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
#ifndef __TUVISION_CPOINTXY_H_
#define __TUVISION_CPOINTXY_H_ 1

#include "TUVisionDef.h"

namespace TUVision {

/**********************************************************/
/*! 
 *    This base class defines a point on the image.
 *
 *    \author Islam Shdaifat
 */
/**********************************************************/


class _CCTUVISION CPointXY {

public:

    //! construct with x0 and y0
    CPointXY(int x0 = 0, int y0 = 0){
        x = x0;
        y = y0;
    }

    //! construct a point from a point
    CPointXY(const CPointXY& pt){
        x = pt.x;
        y = pt.y;
    }


    /******************************************************/
    /*! 
     *   x coordinate of the point
     */
    /******************************************************/
    int x;

    /******************************************************/
    /*! 
     *   y coordinate of the point
     */
    /******************************************************/
    int y;

    /******************************************************/
    /*! 
     *   adding two points
     */
    /******************************************************/
    CPointXY &operator +(const CPointXY&);

    /******************************************************/
    /*! 
     *   adding two points
     */
    /******************************************************/
    CPointXY &operator +=(const CPointXY&);

    /******************************************************/
    /*! 
     *   check if the two points are the same
     */
    /******************************************************/
    bool operator ==(const CPointXY&);

    /******************************************************/
    /*! 
     *   check if the two points are near
     */
    /******************************************************/
    bool isNear(const CPointXY& , const int dx, const int dy);
};

} // TUVision

#endif // __TUVISION_CPOINTXY_H_
