/***************************************************************************
                          CEssentialMatrix.h  -  description
                             -------------------
    begin                : 02.04.2003
    copyright            : (C) 2003 by Thomas Beilicke
    email                : tuvisionlib@tuvision.de
    origin               : Diplomarbeit Thomas Beilicke
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/

#ifndef __TUVISION_CESSENTIALMATRIX_H_
#define __TUVISION_CESSENTIALMATRIX_H_ 1

#include "TUVisionDef.h"

#include "newmat/newmatap.h"
#include "tuv3d/CIntrinsicMatrix.h"
#include "tuv3d/CCameraMatrix.h"
#include "tuv3d/CFundamentalmatrix.h"
#include "tuv3d/CCorrespondence.h"
#include "tuv/CExceptionEssential.h"

namespace TUVision {

/***********************************************************/
//!  Class to contain an essential matrix.
/*! 
 *   This class is a NEWMAT matrix to contain a 3x3 essential
 *   matrix. It supports calculation out of a fundamental matrix
 *   well as assignment from an existing NEWMAT matrix.   
 *
 *   \ingroup 3d
 * 
 *   \author Thomas Beilicke 
 */
/***********************************************************/

class _CCTUVISION CEssentialMatrix : public NEWMAT::Matrix {
public:	
    CEssentialMatrix();	
    //!< The default constructor constructs a "canonical" essential matrix.
    CEssentialMatrix(const CFundamentalMatrix &F, const CIntrinsicMatrix &K1, const CIntrinsicMatrix &K2);
    //!< Creates an essential matrix out of the fundamental matrix and two intrinsic matrices.
    CEssentialMatrix(const CFundamentalMatrix &F, const CCameraMatrix &P1, const CCameraMatrix &P2);
    //!< Creates an essential matrix out of the fundamental matrix and two camera matrices.
    CEssentialMatrix(const NEWMAT::Matrix &E); 
    //!< Creates a new essential matrix out of an existing NEWMAT matrix.
    CEssentialMatrix &operator=(const NEWMAT::Matrix &E);
    //!< Copies an existing NEWMAT matrix.

    void calculateMatrix(const CFundamentalMatrix &F, const CIntrinsicMatrix &K1, const CIntrinsicMatrix &K2);
    //!< Calculates an essential matrix out of the fundamental matrix and two intrinsic matrices.
    void calculateMatrix(const CFundamentalMatrix &F, const CCameraMatrix &P1, const CCameraMatrix &P2);
    //!< Calculates an essential matrix out of the fundamental matrix and two camera matrices.
    void closestEssential();
    //!< Calculates the closest essential matrix and replaces the current one.
    CCameraMatrix CEssentialMatrix::calculateCamera(const TUVision::CIntrinsicMatrix &K, 
        const TUVision::CCorrespondence &normalizedcorresp, 
        unsigned int camNo1=0, unsigned int camNo2=1) const;
    //!< Calculates a camera pair corresponding to this essential matrix.

};

}

#endif
