/***************************************************************************
                          cpointpairlistnormalizer.h  -  description
                             -------------------
    begin                : Mon Apr 15 2002
    copyright            : (C) 2002 by Fabian Wenzel
    email                : wenzel@tu-harburg.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/

#ifndef __TUVISION_CPOINTPAIRLISTNORMALIZER_H
#define __TUVISION_CPOINTPAIRLISTNORMALIZER_H

#include "newmat/newmat.h"
#include "tuv3d/CCorrespondence.h"
#include <vector>

/***************************************************************************/
//! Normalizes pointlists and stores transformations for later processing
/*!
 *  The theory is based on "Multiple View Geometry" by Hartley/Zisserman
 *  page 91 ff (has also been shown in Eric's Diplomarbeit). For fundamental
 *  matrix computation and homography estimation based on linear algorithms
 *  it has been shown that prior normalization leads to better conditioned
 *  problems and more stable results. Passing a pointpairlist to the
 *  class enables access to a normalized pointlist and the transformations
 *  which are necessary to correct the desired model (homography or
 *  fundamental matrix). For the first pointlist, the plain transformation
 *  is needed (see Algorithm 3.2 on page 92), for the second pointlist the
 *  inverse transformation.
 *
 * \ingroup 3d
 *
 * @author  Fabian Wenzel
 *
 ***************************************************************************/
namespace TUVision{

class _CCTUVISION CPointPairListNormalizer {
public:

    typedef std::vector< TUVision::CCorrespondence > Tpointpairlist;

    //! Constructor does all
    CPointPairListNormalizer(const Tpointpairlist &unnormalizedlist, unsigned int firstindex  = 0,
        unsigned int secondindex = 1);

    //! Returns normalized pointpairlist
    Tpointpairlist &getNormalizedlist()
    {
        return normalizedlist;
    }

    //! Returns firstside transformation for correcting the model
    NEWMAT::Matrix &getfirstsidetransformation()
    {
        return firstsidetransformation;
    }

    //! Returns the inverse secondside transformation for correcting the model
    NEWMAT::Matrix &getsecondsidetransformation()
    {
        return secondsidetransformation;
    }
private:

    NEWMAT::Matrix firstsidetransformation;
    NEWMAT::Matrix secondsidetransformation;
    Tpointpairlist normalizedlist;
};

}
#endif
