/***************************************************************************
                          CTriangulation.h  -  description
                             -------------------
    begin                : 03.04.2003
    copyright            : (C) 2003 by Thomas Beilicke
    email                : tuvisionlib@tuvision.de
    origin               : Diplomarbeit Thomas Beilicke
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/

#ifndef __TUVISION_CTRIANGULATION_H_
#define __TUVISION_CTRIANGULATION_H_ 1

#include "TUVisionDef.h"

#include "tuv3d/CCorrespondence.h"
#include "tuv3d/CCameraMatrix.h"

namespace TUVision {

/***********************************************************/
//!  Class for 3d reconstruction by triangulation.
/*! 
 *   This class offers different methods to reconstruct
 *   a 3d-point out of two correspondences. The quality and
 *   speed of the methods has been tested and is mentioned in
 *   the corresponding documentation. The reconstructed 3d-point
 *   is always set in the given #CWorldCorrespondence.
 *
 *   \ingroup 3d
 * 
 *   \author Thomas Beilicke 
 */
/***********************************************************/
class _CCTUVISION CTriangulation {
public:
    static void midpoint(const CCameraMatrix &P1, const CCameraMatrix &P2, CCorrespondence &corresp, 
        unsigned int camNo1=0, unsigned int camNo2=1);
    //!< The midpoint method calculates the midpoint of the two backprojected rays.
    static void linearEigen(const CCameraMatrix &P1, const CCameraMatrix &P2, CCorrespondence &corresp,
        unsigned int camNo1=0, unsigned int camNo2=1);
    //!< The linear-eigen method minimizes a system of equations A*x=0 to the condition abs(x)=1.
    static unsigned int iterativeEigen(const CCameraMatrix &P1, const CCameraMatrix &P2, 
        CCorrespondence &corresp, unsigned int iterations=15, 
        unsigned int camNo1=0, unsigned int camNo2=1);
    //!< The iterative-eigen method uses adaptive weights to minimize the difference 
    //!< between correspondence and projection.
};

}
#endif
