/***************************************************************************
                          TsaiStructs.h  -  description
                             -------------------
    begin                : 23.04.2001
    copyright            : (C) 2001 by Fabian Wenzel
    email                : tuvisionlib@tuvision.de, f.wenzel@gmx.net
    origin               : Diplomarbeit Fabian Wenzel

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
#ifndef __TUVISION_TSAISTRUCTS_H__
#define __TUVISION_TSAISTRUCTS_H__ 1

namespace TUVision{

/*******************************************************************************
//! This struct encapsulates Tsai's camera model together with SCalibrationConstants
 * Camera parameters are usually the fixed parameters of the given camera
 * system, typically obtained from manufacturers specifications.
 *
 * Cy and Cy (the center of radial lens distortion), may be calibrated
 * separately or as part of the coplanar/noncoplanar calibration
 * The same with sx (x scale uncertainty factor).
 *
 * \ingroup 3d
 *
 \*******************************************************************************/
struct SCameraParameters {
    double    Ncx;		/* [sel]     Number of sensor elements in camera's x direction */
    double    Nfx;		/* [pix]     Number of pixels in frame grabber's x direction   */
    double    dx;		/* [mm/sel]  X dimension of camera's sensor element (in mm)    */
    double    dy;		/* [mm/sel]  Y dimension of camera's sensor element (in mm)    */
    double    dpx;		/* [mm/pix]  Effective X dimension of pixel in frame grabber   */
    double    dpy;		/* [mm/pix]  Effective Y dimension of pixel in frame grabber   */
    double    Cx;		/* [pix]     Z axis intercept of camera coordinate system      */
    double    Cy;		/* [pix]     Z axis intercept of camera coordinate system      */
    double    sx;		/* []        Scale factor to compensate for any error in dpx   */
};

/*******************************************************************************
//! This struct encapsulates Tsai's camera model together with SCameraParameters
 *
 * Calibration constants are the model constants that are determined from the
 * calibration data.
 *
 \*******************************************************************************/
/*******************************************************************************
//! This struct encapsulates Tsai's camera model together with SCameraParameters
 *
 * Calibration constants are the model constants that are determined from the
 * calibration data.
 *
 * \ingroup 3d
 *
 \*******************************************************************************/
struct SCalibrationConstants {
    double    f;		/* [mm]          */
    double    kappa1;		/* [1/mm^2]      */
    double    p1;		/* [1/mm]        */
    double    p2;		/* [1/mm]        */
    double    Tx;		/* [mm]          */
    double    Ty;		/* [mm]          */
    double    Tz;		/* [mm]          */
    double    Rx;		/* [rad]	 */
    double    Ry;		/* [rad]	 */
    double    Rz;		/* [rad]	 */
    double    r1;		/* []            */
    double    r2;		/* []            */
    double    r3;		/* []            */
    double    r4;		/* []            */
    double    r5;		/* []            */
    double    r6;		/* []            */
    double    r7;		/* []            */
    double    r8;		/* []            */
    double    r9;		/* []            */
};


}

#endif // __TUVISION_TSAISTRUCTS_H__
