/***************************************************************************
                           CPlane.cpp  -  description
                              -------------------
     begin                : 23.08.2002
     copyright            : (C) 2002 Marc Hensel
     email                : tuvisionlib@tuvision.de, marc.hensel@web.de
     origin               : Diploma thesis Marc Hensel
 
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/

#include <tuv/CPlane.h>

//using namespace std;
using namespace TUVision;


/**************************************************
 *
 * Constructor and destructor
 *
 **************************************************/

/*/
//! Constructor setting d=0, nx=ny=0, and nz=1.
/*/

CPlane::CPlane()
{
    d	= 0;
    nx	= 0;
    ny	= 0;
    nz	= 1;
}


/*/
//! Constructor setting plane by distance and orthonormal vector
/*/

CPlane::CPlane(double d_in,double nx_in,double ny_in,double nz_in,bool normalize)
{
    d	= d_in;
    nx	= nx_in;
    ny	= ny_in;
    nz	= nz_in;
    if (normalize) normVector();
}


/*/
//! Constructor setting plane using three given points in plane
/*/

CPlane::CPlane(CVertex3d P1,CVertex3d P2,CVertex3d P3)
{
    set3PointPlane(P1,P2,P3);
}


/*/
//! Destructor.
/*/

CPlane::~CPlane()
{
}



/**************************************************
 *
 * Set plane
 *
 **************************************************/

/*/
//! Set plane using three given points in plane
/*/

bool CPlane::set3PointPlane(CVertex3d P1,CVertex3d P2,CVertex3d P3)
{
    // Vector orthonormal to plane (cross product of directions normalized to length 1)
    nx = (P1.y - P3.y) * (P2.z - P3.z) - (P1.z - P3.z) * (P2.y - P3.y);
    ny = (P1.z - P3.z) * (P2.x - P3.x) - (P1.x - P3.x) * (P2.z - P3.z);
    nz = (P1.x - P3.x) * (P2.y - P3.y) - (P1.y - P3.y) * (P2.x - P3.x);
    if ( !normVector() ) return false;

    // Distance to origin
    d = P1.x * nx + P1.y * ny + P1.z * nz;

    return true;
}



/**************************************************
 *
 * Test if 3d point is in plane
 *
 **************************************************/

/*/
//! Returns "true" if x lies in plane.
/*/

bool CPlane::isInPlane(CVertex3d x)
{
    if (d != x.x * nx + x.y * ny + x.z * nz) return false;
    return true;
}


/*/
//! Returns "true" if (x,y,z)^T lies in plane.
/*/

bool CPlane::isInPlane(double x,double y,double z)
{
    if (d != x * nx + y * ny + z * nz) return false;
    return true;
}



/**************************************************
 *
 * Normalize vector n
 *
 **************************************************/

/*/
//! Normalize n to euclidean length = 1 after changing nx, ny, and/or nz.
/*/

bool CPlane::normVector()
{
    double factor = sqrt(nx * nx + ny * ny + nz * nz);
    if (factor == 0)	return false;				// Vector n is nullvector (nx = ny = nz = 0)
    else				factor = 1.0 / factor;
    nx = factor * nx;
    ny = factor * ny;
    nz = factor * nz;
    return true;
}

