/***************************************************************************
                          CFileReadAll.cpp  -  description
                             -------------------
    begin                : 01.01.2001
    copyright            : (C) 2001 by Eric Sommerlade
    email                : tuvisionlib@tuvision.de
    origin               : Studienarbeit/Diplomarbeit Eric Sommerlade

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
#include "tuv/CFileReadAll.h"
#include "tuv/CFileRead.h"
#include "tuv/CExceptionFile.h"

#include "tuv/CFileReadRegistry.h"
#include "tuv/CFileReadRegistrator.h"
#include "tuv/CFileReadPng.h"
#include "tuv/CFileReadBmp.h"
#include "tuv/CFileReadMpeg.h"
#include "tuv/CFileReadJpeg.h"
#include "tuv/CFileReadJpeg2k.h"
#include "tuv/CFileReadTiff.h"
#include "tuv/CFileReadRas.h"
#include "tuv/CFileReadPnm.h"

#include <cstdio>


using namespace TUVision;

CFileReadAll::CFileReadAll()
{
    d_fileread=0;
    InitializeDefaultRegistrators();
}


void CFileReadAll::InitializeDefaultRegistrators(){
    static CFileReadBmpRegistrator bmpregistrator;
    static CFileReadJpegRegistrator jpegregistrator;
    static CFileReadJpeg2kRegistrator jpeg2kregistrator;
    static CFileReadMpegRegistrator mpegregistrator;
    static CFileReadPngRegistrator pngregistrator;
    static CFileReadRasRegistrator rasregistrator;
    static CFileReadTiffBigEndianRegistrator tiffbigendianregistrator;
    static CFileReadTiffLittleEndianRegistrator tifflittleendianregistrator;
    static CFileReadPnmRegistratorASCII pnmregistratorascii;
    static CFileReadPnmRegistratorBinary pnmregistratorbinary;
}

CFileReadAll::~CFileReadAll()
{
    close();
}

void 
CFileReadAll::open(const std::string& filename)
{
    d_filename = filename;

    FILE* f=fopen(d_filename.c_str(),"rb");
    if (f == NULL)
        throw CExceptionFileOpen (d_filename);

    unsigned char dst[100];

    size_t read_size=fread(dst,1,100,f);

    fclose(f);
    CFileReadRegistry& reg=CFileReadRegistry::getRegistry();

    CFileReadRegistrator* r=reg.getReader(read_size,dst);
    if(r!=0) {
        d_fileread=r->construct();
        d_fileread->open(d_filename);
    }
    else
        throw CExceptionFileHandle (d_filename, "No handler in FileRead Registry.");
}


void 
CFileReadAll::close()
{
    if(d_fileread==0)
        return;

    delete d_fileread;
    d_fileread=0;
}

CImage* 
CFileReadAll::read(int img_number)
{
    if (d_fileread == 0)
        throw CExceptionFileOpen (d_filename);

    return(d_fileread->read(img_number));
}

int 
CFileReadAll::getNumberImages()
{
    if (d_fileread == 0)
        throw CExceptionFileOpen (d_filename);

    return(d_fileread->getNumberImages());
}
