/***************************************************************************
                          CFileReadBmp.cpp  -  description
                             -------------------
    begin                : 01.01.2001
    copyright            : (C) 2001 by Islam Shdaifat
    email                : tuvisionlib@tuvision.de
    origin               : PhD - Thesis Islam Shdaifat

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/

#include <cmath>

#include "tuv/CFileReadBmp.h"

#include "tuv/CImageByteGray.h"
#include "tuv/CImageByteRGB.h"
#include "tuv/CException.h"
#include "tuv/CExceptionFile.h"

extern "C" {
#include "jasper/jasper.h"
}


using namespace std;
using namespace TUVision;


const unsigned char CFileReadBmpRegistrator::d_id[]={'B','M'};
const int CFileReadBmpRegistrator::d_len=2;


CFileReadBmp::CFileReadBmp (void)
{
}

void CFileReadBmp::open(const std::string &filename)
throw (CException)
{
    jas_init();
    d_filename=filename;
    jfile = jas_stream_fopen (d_filename.c_str(),"rb");
}

void CFileReadBmp::close()
{
    jas_stream_close( (jas_stream_t *) jfile);
}

int CFileReadBmp::getNumberImages(void){
    return 1;
}

CFileReadBmp::~CFileReadBmp()
{
}

/*
 * read the BMP from a file an RGB image
 *
 * \return CImage of the bmp
 */
CImage* CFileReadBmp::read (int img_number)
throw (CException, std::exception)
{
    jas_image_t *jimage = bmp_decode( (jas_stream_t *) jfile, 0);

    int width =jas_image_width(jimage);
    int height=jas_image_height(jimage);

    // We have a Gray image
    if (jas_image_numcmpts(jimage) == 1){
        jas_matrix_t *jasdata = jas_matrix_create(height, width);
        jas_image_readcmpt(jimage,0,0,0,width,height,jasdata);
        CImageByteGray *result = new CImageByteGray(width,height);
        unsigned char *dst = result->getRow(0);
        jas_seqent_t  *src = jasdata->data_;
        for(long k =0;k <width *height;k++)
            *dst++ = (unsigned char) *src++;
        jas_matrix_destroy(jasdata);
        jas_image_destroy(jimage);
        return result;
    }
    // We have an RGB image
    else if (jas_image_numcmpts(jimage) == 3 || jas_image_numcmpts(jimage) == 4){
        jas_matrix_t *jas_r = jas_matrix_create(height,width);
        jas_matrix_t *jas_g = jas_matrix_create(height,width);
        jas_matrix_t *jas_b = jas_matrix_create(height,width);

        jas_image_readcmpt(jimage,0,0,0,width,height,jas_r);
        jas_image_readcmpt(jimage,1,0,0,width,height,jas_g);
        jas_image_readcmpt(jimage,2,0,0,width,height,jas_b);

        CImageByteRGB *result = new CImageByteRGB(width,height);
        unsigned char *dst = result->getRow(0);
        jas_seqent_t  *r   = jas_r->data_;
        jas_seqent_t  *g   = jas_g->data_;
        jas_seqent_t  *b   = jas_b->data_;
        for(long k=0;k<width*height;k++){
            *dst++=(unsigned char) *r++;
            *dst++=(unsigned char) *g++;
            *dst++=(unsigned char) *b++;
        }
        jas_matrix_destroy(jas_r);
        jas_matrix_destroy(jas_g);
        jas_matrix_destroy(jas_b);
        jas_image_destroy(jimage);
        return result;
    }

    return 0;
}
