/***************************************************************************
                          CFileWriteGnuplot3D.cpp  -  description
                             -------------------
    begin                : 01.01.2001
    copyright            : (C) 2001 by Marco Budde
    email                : tuvisionlib@tuvision.de
    origin               : Diplomarbeit Marco Budde

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
#include <fstream>
#include <string>

#include "tuv/CImageByteGray.h"
#include "tuv/CFileWriteGnuplot3D.h"
#include "tuv/CException.h"
#include "tuv/CExceptionFile.h"


using namespace std;
using namespace TUVision;



CFileWriteGnuplot3D::CFileWriteGnuplot3D (void)
{
    d_filename = "";
}


CFileWriteGnuplot3D::~CFileWriteGnuplot3D()
{
}


void CFileWriteGnuplot3D::open (const string &filename)
throw (CException)
{
    d_filename = filename;

    hd.open (d_filename.c_str());
    if (!hd.good())
        throw CExceptionFileOpen (d_filename);
}


void CFileWriteGnuplot3D::close (void)
throw()
{
    hd.close();
}


/******************************************************/
/*! 
 *   This function saves the given image (8 bit gray)
 *   as file readable by gnuplot (splot command). 
 *
 *   \param img image to be saved
 *
 *   \bug Exception handling (ios_base not supported
 *        by the libstdc++ at the moment) - MB
 */
/******************************************************/

void CFileWriteGnuplot3D::write (const CImageByteGray &img)
throw (CException)
{
    const unsigned char  *img_buffer;
    int                  y, x;


    // Stream object valid?
    if (!hd.good())
        throw CExceptionFileWrite (d_filename);

    // fixme:
    // add exception handler using ios_base
    // map exceptions to CException

    for (y = 0; y < img.getH(); y++)
    {
        img_buffer = img.getRow (y);
        for (x = 0; x < img.getW(); x++)
            hd << x << " " << y << " " 
            << (unsigned int) img_buffer[x] << endl;
        hd << endl;
    }
}
