/***************************************************************************
                          CFileWritePbm.cpp  -  description
                             -------------------
    begin                : 01.01.2001
    copyright            : (C) 2001 by Marco Budde
    email                : tuvisionlib@tuvision.de
    origin               : Diplomarbeit Marco Budde

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
#include <fstream>
#include <iomanip>
#include <string>

#include "tuv/CFileWritePbm.h"
#include "tuv/CImageByteBilevel.h"
#include "tuv/CImageByteGray.h"
#include "tuv/CImageByteRGB.h"
#include "tuv/CException.h"
#include "tuv/CExceptionFile.h"


using namespace std;
using namespace TUVision;



CFileWritePbm::CFileWritePbm (void)
{
    d_filename = "";
}


CFileWritePbm::~CFileWritePbm()
{
}


void CFileWritePbm::open (const string &filename)
throw (CException)
{
    d_filename = filename;

    hd.open (d_filename.c_str());
    if (!hd.good())
        throw CExceptionFileOpen (d_filename);
}


void CFileWritePbm::close (void)
throw ()
{
    hd.close();
}


/******************************************************/
/*! 
 *   This function saves the given image (8 bit gray)
 *   as pgm file. 
 *
 *   \param img image to be saved
 *
 *   \bug Exception handling (ios_base not supported
 *        by the libstdc++ at the moment) - MB
 */
/******************************************************/

void CFileWritePbm::write (const CImageByteGray &img)
throw (CException)
{
    const unsigned char  *img_buffer;
    int                  y, x;
    int                  cr = 0;


    // Stream object valid?
    if (!hd.good())
        throw CExceptionFileWrite (d_filename);

    // fixme:
    // add exception handler using ios_base
    // map exceptions to CException

    hd << "P2" << endl;
    hd << img.getW() << " " << img.getH() << endl;
    hd << "255" << endl;

    for (y = 0; y < img.getH(); y++)
    {
        img_buffer = img.getRow (y);
        for (x = 0; x < img.getW(); x++)
        {
            hd << setw(3) << (unsigned int) img_buffer[x];

            if (cr > 15)
            {
                hd << endl;
                cr = 0;
            }
            else
            {
                hd << " ";
                cr++;
            }
        }

        hd << endl;
        cr = 0;
    }
}


/******************************************************/
/*! 
 *   This function saves the given image (RGB)
 *   as ppm file. 
 *
 *   \param img image to be saved
 *
 *   \bug Exception handling (ios_base not supported
 *        by the libstdc++ at the moment) - MB
 */
/******************************************************/

void CFileWritePbm::write (const CImageByteRGB &img)
throw (CException)
{
    const unsigned char  *img_buffer;
    int                  y, x;
    int                  cr = 0;


    // Stream object valid?
    if (!hd.good())
        throw CExceptionFileWrite (d_filename);

    // fixme:
    // add exception handler using ios_base
    // map exceptions to CException

    hd << "P3" << endl;
    hd << img.getW() << " " << img.getH() << endl;
    hd << "255" << endl;

    for (y = 0; y < img.getH(); y++)
    {
        img_buffer = img.getRow (y);
        for (x = 0; x < img.getW(); x++)
        {
            hd << setw(3) << (unsigned int) img_buffer[x * 3] << " ";
            hd << setw(3) << (unsigned int) img_buffer[x * 3 + 1] << " ";
            hd << setw(3) << (unsigned int) img_buffer[x * 3 + 2];

            if (cr > 3)
            {
                hd << endl;
                cr = 0;
            }
            else
            {
                hd << "  ";
                cr++;
            }
        }

        hd << endl;
        cr = 0;
    }
}


/******************************************************/
/*! 
 *   This function saves the given image (bilevel)
 *   as pbm file. 
 *
 *   \param img image to be saved
 *
 *   \bug Exception handling (ios_base not supported
 *        by the libstdc++ at the moment) - MB
 */
/******************************************************/

void CFileWritePbm::write (const CImageByteBilevel &img)
throw (CException)
{
    const unsigned char  *img_buffer;
    int                  y, x;
    int                  cr = 0;


    if (!hd.good())
        throw CExceptionFileWrite (d_filename);

    // fixme:
    // add exception handler using ios_base
    // map exceptions to CException

    hd << "P1" << endl;
    hd << img.getW() << " " << img.getH() << endl;

    for (y = 0; y < img.getH(); y++)
    {
        img_buffer = img.getRow (y);
        for (x = 0; x < img.getW(); x++)
        {
            if (img_buffer[x] == 0)
                hd << "1";
            else
                hd << "0";

            if (cr > 33)
            {
                hd << endl;
                cr = 0;
            }
            else
            {
                hd << " ";
                cr++;
            }
        }

        hd << endl;
        cr = 0;
    }
}
