/***************************************************************************
                          CFileWritePng.cpp  -  description
                             -------------------
    begin                : 20.09.2001
    copyright            : (C) 2001 by Fabian Wenzel
    email                : tuvisionlib@tuvision.de
    origin               : Preparation Bildverarbeitung 1 WS 2001

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
#include <cstdio>
#include <string>

#include <png.h>

#include "tuv/CImageByteGray.h"
#include "tuv/CImageByteRGB.h"
#include "tuv/CFileWritePng.h"
#include "tuv/CException.h"
#include "tuv/CExceptionFile.h"


using namespace std;
using namespace TUVision;


CFileWritePng::CFileWritePng (void)
{
    d_filename = "";
    d_hd = NULL;
}


CFileWritePng::~CFileWritePng()
{
    if (d_hd != NULL)
        fclose (d_hd);
}


void CFileWritePng::open (const string &filename)
throw (CException)
{
    d_filename = filename;

    d_hd = fopen (d_filename.c_str(), "wb");
    if (d_hd == NULL)
        throw CExceptionFileOpen (d_filename);
}


void CFileWritePng::close (void)
throw()
{
    if (d_hd != NULL)
    {
        fclose (d_hd);
        d_hd = NULL;
    }
}


/******************************************************/
/*! 
 *   This function writes the given image to a PNG
 *   graphic file. 
 *
 *   \param img         image to be saved
 *   \param color_space color space of the image:
 *                      grayscale (1) or RGB (3)
 */
/******************************************************/

void CFileWritePng::write (const CImageByte &img, 
                           int color_space)
                           throw (CException)
{
    png_structp png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING,NULL,NULL,NULL);
    png_infop info_ptr = png_create_info_struct(png_ptr);
    png_infop end_info = png_create_info_struct(png_ptr);
    png_bytep *row_pointers = NULL;
    int                         y;     


    // Valid file handle?
    if (d_hd == NULL)
        throw CExceptionFileWrite (d_filename);

    // Install our own error handler. This handler is used
    // to get a description of the problem and throw an
    // exception.
    if(setjmp(png_jmpbuf(png_ptr))){
        png_destroy_info_struct(png_ptr,&end_info);
        png_destroy_write_struct(&png_ptr,&info_ptr);
        throw CExceptionFileWrite( d_filename, "Error occured while writing file");
    }

    png_init_io(png_ptr,d_hd);
    png_set_IHDR(png_ptr,info_ptr,img.getW(),img.getH(),8,
        (color_space==1?PNG_COLOR_TYPE_GRAY:PNG_COLOR_TYPE_RGB),
        PNG_INTERLACE_NONE,PNG_COMPRESSION_TYPE_DEFAULT,
        PNG_FILTER_TYPE_DEFAULT);

    row_pointers = new png_bytep[img.getH()];
    for (y = 0; y < img.getH(); y++){
        row_pointers[y] = const_cast<unsigned char*> (img.getRow (y));
    }
    png_set_rows(png_ptr,info_ptr,row_pointers);
    png_write_png(png_ptr,info_ptr,PNG_TRANSFORM_IDENTITY,NULL);
    png_destroy_info_struct(png_ptr,&end_info);
    png_destroy_write_struct(&png_ptr,&info_ptr);

    delete[] row_pointers;
}


/******************************************************/
/*! 
 *   This function writes the given image (8 bit gray)
 *   to a PNG graphic file. 
 *
 *   \param img image to be saved
 */
/******************************************************/

void CFileWritePng::write (const CImageByteGray &img)
throw (CException)
{
    write (img, 1);
}


/******************************************************/
/*! 
 *   This function writes the given image (8 bit RGB)
 *   to a PNG graphic file. 
 *
 *   \param img image to be saved
 */
/******************************************************/

void CFileWritePng::write (const CImageByteRGB &img)
throw (CException)
{
    write (img, 3);
}
