/***************************************************************************
                          CFileWritePnm.cpp  -  description
                             -------------------
    begin                : 08.04.2003
    copyright            : (C) 2003 by Fabian Wenzel
    email                : tuvisionlib@tuvision.de
    origin               : Maintainance

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/

#include <cmath>


#include "tuv/CFileWritePnm.h"

#include "tuv/CImageByteGray.h"
#include "tuv/CImageByteRGB.h"
#include "tuv/CException.h"
#include "tuv/CExceptionFile.h"

extern "C" {
#include "jasper/jasper.h"
}

using namespace std;
using namespace TUVision;


CFileWritePnm::CFileWritePnm (void)
{
    d_filename = "";
    jfile = NULL;
}


CFileWritePnm::~CFileWritePnm()
{
    if(jfile)
        close();
}

void CFileWritePnm::open(const std::string &filename)
throw (CException)
{
    jas_init();
    d_filename = filename;
    jfile = jas_stream_fopen( d_filename.c_str(), "wb");
}

void CFileWritePnm::close()
throw ()
{
    jas_stream_close((jas_stream_t *) jfile);
    jfile=NULL;
}

/*
 * write a image to bmp file
 */
void
CFileWritePnm::write (const CImageByteGray &img)
throw (CException)
{
    int height = img.getH();
    int width  = img.getW();
    jas_matrix_t *jas_data = jas_matrix_create(height,width);

    const unsigned char *src = img.getRow(0);
    jas_seqent_t *dst = jas_data->data_;

    for(long k=0;k<width*height;k++)
        *dst++=*src++;

    jas_image_cmptparm_t cmptparms;
    cmptparms.tlx = 0;
    cmptparms.tly = 0;
    cmptparms.hstep = 1;
    cmptparms.vstep = 1;
    cmptparms.width = width;
    cmptparms.height= height;
    cmptparms.prec  = 8; // 8 BPP
    cmptparms.sgnd  = 0;

    jas_image_t *newimage = jas_image_create(1,&cmptparms,JAS_CLRSPC_SGRAY);
    jas_image_writecmpt(newimage,0,0,0,width,height,jas_data);
    jas_image_setcmpttype(newimage, 0,JAS_IMAGE_CT_COLOR(JAS_CLRSPC_CHANIND_GRAY_Y));


    pnm_encode(newimage,(jas_stream_t *) jfile,0);

    jas_matrix_destroy(jas_data);
    jas_image_destroy(newimage);
}

void
CFileWritePnm::write (const CImageByteRGB &img)
throw (CException)
{
    int height = img.getH();
    int width  = img.getW();
    jas_matrix_t *jas_r = jas_matrix_create(height,width);
    jas_matrix_t *jas_g = jas_matrix_create(height,width);
    jas_matrix_t *jas_b = jas_matrix_create(height,width);

    const unsigned char *src = img.getRow(0);
    jas_seqent_t *r = jas_r->data_;
    jas_seqent_t *g = jas_g->data_;
    jas_seqent_t *b = jas_b->data_;
    for(long k=0;k<width*height;k++){
        *r++=*src++;
        *g++=*src++;
        *b++=*src++;
    }
    jas_image_cmptparm_t cmptparms[3];
    for(int i=0;i<3;++i){
        cmptparms[i].tlx = 0;
        cmptparms[i].tly = 0;
        cmptparms[i].hstep = 1;
        cmptparms[i].vstep = 1;
        cmptparms[i].width = width;
        cmptparms[i].height= height;
        cmptparms[i].prec  = 8; // 8 BPP
        cmptparms[i].sgnd  = 0;
    }

    jas_image_t *newimage = jas_image_create(3,cmptparms,JAS_CLRSPC_SRGB);
    jas_image_writecmpt(newimage,0,0,0,width,height,jas_r);
    jas_image_setcmpttype(newimage, 0,JAS_IMAGE_CT_COLOR(JAS_CLRSPC_CHANIND_RGB_R));

    jas_image_writecmpt(newimage,1,0,0,width,height,jas_g);
    jas_image_setcmpttype(newimage, 1,JAS_IMAGE_CT_COLOR(JAS_CLRSPC_CHANIND_RGB_G));

    jas_image_writecmpt(newimage,2,0,0,width,height,jas_b);
    jas_image_setcmpttype(newimage, 2,JAS_IMAGE_CT_COLOR(JAS_CLRSPC_CHANIND_RGB_B));

    pnm_encode(newimage,(jas_stream_t *) jfile,0);

    jas_matrix_destroy(jas_r);
    jas_matrix_destroy(jas_g);
    jas_matrix_destroy(jas_b);
    jas_image_destroy(newimage);
}
