/***************************************************************************
                          CLine.cpp  -  description
                             -------------------
    begin                : 1.1.2000
    copyright            : (C) 2000 by Islam Shdaifat
    email                : tuvisionlib@tuvision.de
    origin               : PhD - Thesis Islam Shdaifat
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
#include "tuv2d/CLine.h"
#include "tuv2d/CPointXY.h"
#include "tuv2d/CCurve.h"

#define Pi 3.1415926535

using namespace TUVision;
using namespace std;


/********************************************************/
/*!
 *	construct a multi line curve object, The line passes
 *  through the origen and a window frame w, with slope
 *  angle resolution of res. The line starts at slope angle
 *  t0 til t1.
 *
 *  \param w   the window size
 *  \param res the slope angle resolution
 *  \param t0 the initial slope angle
 *  \param t1 the final slope angle
 */
/********************************************************/
CLine::CLine(int w ,double res, double t0, double t1)
{
    setLineShape(w,res,t0,t1);
    //	setWindowShape(w);
}

CLine::~CLine()
{
}

/********************************************************/
/*!
 *	using the y sin(thita)+ x cos(thita) = rho generate 
 *  lines.
 *
 *  \param w   the window size
 *  \param res the slope angle resolution
 *  \param t0 the initial slope angle
 *  \param t1 the final slope angle
 */
/********************************************************/
void 
CLine::setLineShape(int w, double res, double t0, double t1)
{
    // w is the square window side
    // res is the resolution in degree
    int x1,y1;
    double x,y,thita;
    int CurveLength;
    for(thita=t0;thita<t1;thita+=res)
    {
        CurveLength=0;
        x=(double(w)*cos(double(thita)*(Pi/180.0))); // x on a circle circumfrance
        y=(double(w)*sin(double(thita)*(Pi/180.0))); // y on a circle circumfrance
        int sg=1; // the sign of x or y
        if(abs(y)>abs(x))
        {
            if(y<0)sg=-1;
            for(y1=0;y1<=w;y1++)
            {
                d_Pxy.y=sg*y1; // the point y coordinate
                double s=(double)(y1*x)/(double)y;
                d_Pxy.x=(int) rint(s); // the point x coordinate
                d_Cxy.vecCurvePoints.push_back(d_Pxy); //insert the point in the curve
                CurveLength++; // increase the length of the curve
            }
        }
        else
        {
            if(x<0)sg=-1;
            for(x1=0;x1<=w;x1++)
            {
                d_Pxy.x=sg*x1;// the point x coordinate
                double s= (double)(x1*y)/(double)x;
                d_Pxy.y=(int) (sg*rint(s));// the point y coordinate
                d_Cxy.vecCurvePoints.push_back(d_Pxy); //insert the point in the curve
                CurveLength++; // increase the length of the curve
            }

        }
        d_Cxy.attribute = thita;
        d_Cxy.CurveEndPoint = d_Pxy;  // set the curve end point
        d_Cxy.CurveLength=CurveLength; // set the curve length
        vecCurvesLUT.push_back(d_Cxy); //insert the curve in the LUT
        d_Cxy.vecCurvePoints.clear(); // clear the temp curve for new curve
    }
    getRidOfDuplicatedCurves(); // get rid of duplicated points
    // this will generates the frame of the shape
}


/********************************************************/
/*!
 *  Given a point x,y this function returns an iterator (pointer) to
 *  the expected line (curve) which passes through the origin and the
 *  given point
 */
/********************************************************/
inline vector<CCurve>::iterator 
CLine::itToLineOf(int x,int y)
{
    static std::vector<CCurve>::iterator pos;
    for(pos=vecCurvesLUT.begin();pos<vecCurvesLUT.end();++pos)
    {
        if( (*pos).CurveEndPoint.x==x && (*pos).CurveEndPoint.y==y )
            return pos;
    }
    //Not found, so return end()
    return vecCurvesLUT.end();
}



/********************************************************/
/*!
 *  Given a point PointXY, this function returns an iterator (pointer) to
 *  the expected line (curve) which passes through the origin and the
 *  given point PointXY
 */
/********************************************************/
inline std::vector<CCurve>::iterator 
CLine::itToLineOf(const CPointXY& pt)
{
    static std::vector<CCurve>::iterator pos;
    for(pos=vecCurvesLUT.begin();pos<vecCurvesLUT.end();++pos)
    {
        if( (*pos).CurveEndPoint == pt)
            return pos;
    }
    // Not found, so return end()
    return vecCurvesLUT.end();
}

/********************************************************/
//! to be removed
/*!
 *	Fill the vector of points that contains the window boundary
 *  points.
 *
 *  \param w   the window size
 *  \param t0 the initial slope angle
 *  \param t1 the final slope angle
 */
/********************************************************/
void 
CLine::setWindowShape(int w)
{
}
