/***************************************************************************
                          CLineSearch.cpp  -  description
                            -------------------
    begin                : 1.1.2000
    copyright            : (C) 2000 by Islam Shdaifat
    email                : tuvisionlib@tuvision.de
    origin               : PhD - Thesis Islam Shdaifat
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
#include "tuv2d/CLineSearch.h"

#include "tuv/CImageByteGray.h"
#include "tuv2d/CLine.h"

#include <iostream>

using namespace TUVision;
using namespace std;


CLineSearch::CLineSearch(CLine& line,
                         double dThr0 ,
                         double dThr1 )
                         :CCurveSearch(line,dThr0,dThr1)
{
    //vecCurvesLUT = line.vecCurvesLUT;      // get the curves from the line shapes
    //setItCurveStart(vecCurvesLUT.begin()); // the start of the search at
    //setItCurveEnd(vecCurvesLUT.end());     // the end of the search at
    d_Min_HitCount = (int) (dThr1 * (vecCurvesLUT.begin())->CurveLength);    // the min hit point
    d_Max_NonHitCount = (int) (dThr0 * (vecCurvesLUT.begin())->CurveLength); // the max non hit 
}



//! write the output to the OutputVector 
void
CLineSearch::writeToFeatureStream()
{
    // TODO: In this function the line can be deleted if e.g. 90%
    //write to output
    COut tmpout;
    //if(d_Hit >= d_Min_HitCount )
    {
        /*
        cout
        <<"\n is found at x:          "<< d_x
        <<"\n and y:                  "<< d_y
        <<"\n with hit points:        "<< d_Hit
        <<"\n with NON hit points:    "<< d_NonHit
        <<"\n the curve pointer:      "<< d_it_crv
        <<"\n the curve attribute:    "<< d_it_crv->attribute
        <<"\n"
        ;
        */
        tmpout.crv = &(*d_it_crv);
        tmpout.x = d_x;
        tmpout.y = d_y;
        tmpout.xf = d_it_crv->CurveEndPoint.x + d_x;
        tmpout.yf = d_it_crv->CurveEndPoint.y + d_y;
        tmpout.Hits = d_Hit;
        tmpout.NonHits = d_NonHit;
        tmpout.condition = 1;

        vecSearchOut.push_back(tmpout); // insert it in the output curve
        if(delCurveFromImge == true)  // to delete the curve from the image
            drawCurveInImage(*d_image, *d_it_crv,
            d_x, d_y, 230);
    } // a real curve was found
}

bool 
CLineSearch::checkSpecialCondition ()
{
    // TODO make it inline!!
    //if the pixel is on 
    if(d_image->getPixel(d_x,d_y) == 255 ) return true;

    return false;
};

/**********************************************************/
/*! 
 * search the Multi Curves in the Image
 *
 * \param img a pointer to the image to search in
 */
/**********************************************************/
void
CLineSearch::search(CImageByteGray *img, int x0, int y0)
{ 
    setSearchInImage( *img ); // set the search in the image
    vecSearchOut.clear(); // clear the output vector

    int w = d_image->getW() - x0; // image width
    int h = d_image->getH() - y0; // image height
    int i, j, posmax, maxhits;
    unsigned char tmp;
    int *pXY;

    // use setWindow() to be inside the image or Pad the image
    // if necessary, this shoul be done before SEARCH
    for(d_y = 0 ; d_y < h ; d_y++) // loop through height
    { 
        d_image_ptr = d_image->getRow(d_y) + x0;
        for(d_x = x0 ; d_x < w  ; d_x++) // loop through width
        {
            if( *d_image_ptr == 255 )
            {
                posmax = 0; maxhits = 0;
                // for the whole curve lut compare all the curves
                for(i = 0; i < nofLines; i++)
                {
                    d_Hit = 0; // reset counters
                    d_NonHit = 0;
                    pXY = LinePointsXY[i];
                    for( j = 0 ; j < length[i] ; j++)
                    {
                        // the value of the image at that point
                        tmp = *(d_image_ptr + *pXY++);
                        if( tmp == 255) 	d_Hit++; // it is a hit point			
                        else
                        { // non hit point
                            d_NonHit++;
                            if(d_NonHit > d_Max_NonHitCount )
                            {
                                // it is better to break now rather to continue searching
                                break;
                            }
                        }// non hit point
                        /*
                         */
                    } // end the curve j

                    if( d_Hit >= d_Min_HitCount)
                        if( d_Hit >= maxhits)
                        {
                            maxhits = d_Hit;
                            posmax = i;
                        }
                }
                if( maxhits >= d_Min_HitCount)
                {
                    d_it_crv = vecCurvesLUT.begin()+posmax;
                    //d_curve = &(vecCurvesLUT.at(posmax));
                    writeToFeatureStream();
                }
                //if(d_NonHit <= d_Max_NonHitCount && d_Hit >= d_Min_HitCount)
            }
            d_image_ptr++;
        } // end x, width
    } // end y, height
}


//!search method special for lines
void
CLineSearch::searchSpecial()
{
    // special condition we are on a hit point
    //if(d_image->getPixel(d_x,d_y,0) == 255 )
    //if( *d_image_ptr == 255 ) // if an edge point
    int i, j;
    unsigned char tmp;
    int *pXY;
    // for the whole curve lut compare all the curves
    for(i = 0; i < nofLines; i++)
    {
        d_Hit = 0; // reset counters
        d_NonHit = 0;
        pXY = LinePointsXY[i];
        for( j = 0 ; j < length[i] ; j++)
        {
            // the value of the image at that point
            tmp = *(d_image_ptr + *pXY++);
            if( tmp == 255)
            { // it is a hit point
                d_Hit++;
            } // hit point
            else
            { // non hit point
                d_NonHit++;
                if(d_NonHit > d_Max_NonHitCount )
                {
                    // it is better to break now rather to continue searching
                    break;
                }
            }// non hit point
        } // end the curve
        if(d_NonHit <= d_Max_NonHitCount && d_Hit >= d_Min_HitCount)
        {
            d_it_crv = vecCurvesLUT.begin()+i;
            d_curve = &(*d_it_crv);
            writeToFeatureStream();
        }
    }
}

/**********************************************************/
/*! 
 * count hit and non hit points
 */
/**********************************************************/
void
CLineSearch::getHitAndNonHitCount(int pos) {
    /*
    int j;
    unsigned char tmp;
    //int length = d_curve->vecCurvePoints.size();

    d_Hit = 0; // reset counters
    d_NonHit = 0;

    for( j = 0 ; j < length[pos] ; j++)
    {
    // the value of the image at that point
    tmp = *(d_image_ptr + LinePointsXY[pos][j]);
    if( tmp == 255) { // it is a hit point
    d_Hit++;
    } // hit point
    else { // non hit point
    d_NonHit++;
    if(d_NonHit > d_Max_NonHitCount ) {
    // it is better to break now rather to continue searching
    break;
    }
    }// non hit point
    } // end the curve
    */
}


/*
 * after line search the line segments are connected.
 */
void
CLineSearch::connectLineSegments()
{
    /*
    this->vecSearchOut<COut>
    class _CCTUVISION COut {
    public:
    int x;       //! the x coordinate of the curve start
    int y;       //! the y coordinate of the curve start
    int xf;       //! the x coordinate of the curve start
    int yf;       //! the y coordinate of the curve start
    int Hits;    //! the hit points of the curve
    int NonHits; //! the non hit points of the curve
    CCurve *crv; //! apointer to the curve itself

    int condition; //! this variable is used for post processing
    };

    */
    int dx = 5;
    int dy = 5;
    COut *line1, *line2;
    std::vector<COut>::iterator CurrentLine;
    std::vector<COut>::iterator NextLine;

    for(CurrentLine  = vecSearchOut.begin();
        CurrentLine != vecSearchOut.end()-1;
        ++CurrentLine)
    {
        line1 = &(*CurrentLine);
        if(CurrentLine->condition == 1)
            for(NextLine  = CurrentLine+1 ;
                NextLine != vecSearchOut.end();
                ++NextLine)
            {
                line2 = &(*NextLine);
                //check if the line end points are close
                if( abs(line1->xf - line2->x) < dx ) //
                    if(abs(line1->yf - line2->y) < dy) //
                    {
                        // copare the angles if they are close
                        if(abs(
                            abs(line1->crv->attribute) - 
                            abs(line2->crv->attribute)
                            ) < 3) 
                        {
                            // connect the two lines
                            line2->condition = 0;
                            line1->xf = line2->xf;
                            line1->yf = line2->yf;
                            line1->Hits += line2->Hits;
                            line1->NonHits += line2->NonHits;
                        }
                    }
            }		
    }
    // remove the lines marked with 0 condition
    std::vector<COut> vecTmp;
    for(CurrentLine  = vecSearchOut.begin();
        CurrentLine != vecSearchOut.end();
        ++CurrentLine)
    {
        if(CurrentLine->condition == 1)
        {
            vecTmp.push_back(*CurrentLine);
        }
    }
    vecSearchOut = vecTmp;
}
