/***************************************************************************
                          CParabolaSearch.cpp  -  description
                             -------------------
    begin                : 1.1.2000
    copyright            : (C) 2000 by Islam Shdaifat
    email                : tuvisionlib@tuvision.de
    origin               : PhD - Thesis Islam Shdaifat
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
#include "tuv2d/CParabolaSearch.h"
#include "tuv/CImageByteGray.h"
#include "tuv2d/CParabola.h"
#include <iostream>

using namespace TUVision;
using namespace std;


CParabolaSearch::CParabolaSearch(CParabola& parabola,
                                 double dThr0 ,
                                 double dThr1 )
                                 :CCurveSearch(parabola,dThr0,dThr1)
{
    //vecCurvesLUT = line.vecCurvesLUT;      // get the curves from the line shapes
    //setItCurveStart(vecCurvesLUT.begin()); // the start of the search at
    //setItCurveEnd(vecCurvesLUT.end());     // the end of the search at
    d_Min_HitCount = (int) (dThr1 * (vecCurvesLUT.begin())->CurveLength);    // the min hit point
    d_Max_NonHitCount = (int) (dThr0 * (vecCurvesLUT.begin())->CurveLength); // the max non hit 
}



//! write the output to the OutputVector 
void
CParabolaSearch::writeToFeatureStream()
{
    // TODO: In this function the line can be deleted if e.g. 90%
    //write to output
    COut tmpout;
    if(d_Hit >= d_Min_HitCount ) {
        cout
            <<"\n is found at x:          "<< d_x
            <<"\n and y:                  "<< d_y
            <<"\n with hit points:        "<< d_Hit
            <<"\n with NON hit points:    "<< d_NonHit
            <<"\n the curve pointer:      "<< &(*d_it_crv)
            <<"\n the curve attribute:    "<< d_it_crv->attribute
            <<"\n"
            ;
        /*
         */
        tmpout.crv = &(*d_it_crv);
        tmpout.x = d_x;
        tmpout.y = d_y;
        tmpout.xf = d_it_crv->CurveEndPoint.x;
        tmpout.yf = d_it_crv->CurveEndPoint.y;
        tmpout.Hits = d_Hit;
        tmpout.NonHits = d_NonHit;

        vecSearchOut.push_back(tmpout); // insert it in the output curve
        drawCurveInImage(*d_image, *d_it_crv, d_x, d_y);
    } // a real curve was found
}

bool 
CParabolaSearch::checkSpecialCondition ()
{
    // TODO make it inline!!
    //if the pixel is on 
    if(d_image->getPixel(d_x,d_y) == 255 ) return true;

    return false;
};

void
CParabolaSearch::searchSpecial()
{
    // special condition we are on a hit point
    if(d_image->getPixel(d_x,d_y) == 255 )
    {
        // for the whole curve lut compare all the curves
        for(d_it_crv  = vecCurvesLUT.begin();
            d_it_crv != vecCurvesLUT.end(); ++d_it_crv)
        {
            /*

            if( d_image->getPixel(d_x + d_it_crv->CurveEndPoint.x,
            d_y + d_it_crv->CurveEndPoint.y , 0) == 255 )
            */
            {
                d_curve = &(*d_it_crv);
                getHitAndNonHitCount(); // count the hit and non hit points
                if( isSuffecientHitPoints() )
                    writeToFeatureStream();
            }
        }
    }
}

