/***************************************************************************
                          CShape.cpp  -  description
                             -------------------
    begin                : 1.1.2000
    copyright            : (C) 2000 by Islam Shdaifat
    email                : tuvisionlib@tuvision.de
    origin               : PhD - Thesis Islam Shdaifat
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
#include "tuv2d/CShape.h"
#include "tuv2d/CCurve.h"
#include "tuv2d/CPointXY.h"
#include <iostream>
#include <vector>

using namespace std;
using namespace TUVision;

CShape::CShape()
{
    LinePointsX = 0;
    LinePointsY = 0;
    LinePointsXY = 0;
    length = 0;
    nofLines = 0;
}

CShape::~CShape()
{
    for(int i = 0; i < nofLines; i++)
    {
        delete [] LinePointsX[i]; 
        delete [] LinePointsY[i];
        delete [] LinePointsXY[i]; 
    }
    delete [] LinePointsX;
    delete [] LinePointsY;
    delete [] LinePointsXY; 
    delete [] length;
};
/**********************************************************/
/*!
 * This functions checks whether a duplicated curves exist
 * in vecCurvesLUT and removes them.
 */
/**********************************************************/
void
CShape::getRidOfDuplicatedCurves()
{
    if(vecCurvesLUT.size() < 2) return;// no need to process for one curve
    std::vector < CCurve > vecCurves;	/* a temp lut for curves */
    std::vector < CCurve >::iterator curveIt1, curveIt2; /* iterators through the curves */
    std::vector < CPointXY>::iterator ptIt1, ptIt2; /* iterator through the points of curves*/

    // for the whole lut compare all the curves
    for(curveIt1 = vecCurvesLUT.begin(); curveIt1 != vecCurvesLUT.end()-1; ++curveIt1)
    {
        for(curveIt2 = curveIt1+1; curveIt2 != vecCurvesLUT.end(); ++curveIt2)
        {
            int ct=0; // counter for the curve points
            ptIt2 = curveIt2->vecCurvePoints.begin(); // iterator for the first curve points
            if( curveIt1->vecCurvePoints.size() == curveIt2->vecCurvePoints.size())
                for(ptIt1 = curveIt1->vecCurvePoints.begin();
                    ptIt1 != curveIt1->vecCurvePoints.end(); ++ptIt1)
                {
                    if( ptIt1->x == ptIt2->x && ptIt1->y == ptIt2->y) ct++; 
                    //if the two are the same
                    ++ptIt2; // increase the second iterator for the curve
                    // the first one is incremented automatically with the for loop
                }
                if(ct==curveIt1->vecCurvePoints.size()) // if they identical set 
                {										// signify this by -1 curve length
                    (*curveIt2).CurveLength = -1;
                    //(*curveIt2).vecCurvePoints.clear();
                }
        }
    }
    for(curveIt1 = vecCurvesLUT.begin(); curveIt1 != vecCurvesLUT.end()-1; ++curveIt1)
    {
        //if((*curveIt1).vecCurvePoints.size()!=0) // if the curve length is 0 it is duplica noneed for
        if((*curveIt1).CurveLength!= -1)
            vecCurves.push_back(*curveIt1);
    }
    vecCurvesLUT.clear(); // clear the LUT  and full fill it with the refined one
    vecCurvesLUT.insert(vecCurvesLUT.begin(),vecCurves.begin() ,vecCurves.end());
    setShapeFrame();// always call this after get rid of duplicated curves

    //Fill in the fast pointers	
    fillInLinePoints();
}

/*!
 * from the vector shape fill in the pointer
 */
void
CShape::fillInLinePoints()
{
    int y = vecCurvesLUT.size();
    int i,j ;
    for(i = 0; i < nofLines; i++)
    {
        delete [] LinePointsX[i]; 
        delete [] LinePointsY[i];
        delete [] LinePointsXY[i]; 
    }
    delete [] LinePointsX;
    delete [] LinePointsY;
    delete [] LinePointsXY;
    delete [] length;

    nofLines = y;
    int x;
    LinePointsX = new int*[nofLines];
    LinePointsY = new int*[nofLines];
    LinePointsXY = new int*[nofLines];
    length = new int[nofLines];

    for(i = 0; i < y; i++)
    {
        d_Cxy = vecCurvesLUT[i];
        x = d_Cxy.vecCurvePoints.size();
        length[i] = d_Cxy.vecCurvePoints.size();
        LinePointsX[i] = new int[x];
        LinePointsY[i] = new int[x];
        LinePointsXY[i] = new int[x];
        for(j = 0; j < x; j++)
        {
            LinePointsX[i][j] = d_Cxy.vecCurvePoints[j].x;
            LinePointsY[i][j] = d_Cxy.vecCurvePoints[j].y;
            // a linear relation between image pointed and curve points
            LinePointsXY[i][j] = dp*LinePointsY[i][j] + LinePointsX[i][j]; 

        }
    }
}


/**********************************************************/
/*!
 * This functions generates the frame of the shapes, i.e
 * the last point of each curve in the shape vector into
 * one vector of points.
 */
/**********************************************************/
void
CShape::setShapeFrame()
{
    std::vector < CCurve >::iterator curveIt1; // iterators through the whole curves 
    vecShapeFrame.clear(); // clear the vector of frame
    for(curveIt1 = vecCurvesLUT.begin(); curveIt1 != vecCurvesLUT.end()-1; ++curveIt1)
    {
        vecShapeFrame.push_back((*curveIt1).CurveEndPoint); // push back the end point of each curve
    }
}


/**********************************************************/
/*!
 * prints the shape
 */
/**********************************************************/
ostream & 
operator << (ostream& out,const TUVision::CShape& sp)
{
    CCurve cv; // a curve
    CPointXY pt; // a point
    //out << sp.ShapeName<<"\n";
    for (int i = 0; i < sp.vecCurvesLUT.size(); i++) // for all curves
    {
        cv = sp.vecCurvesLUT[i]; // get one curve
        out <<"Curve: "<< i <<" size "<< cv.vecCurvePoints.size(); // size of the curve
        out<<" Length "<< cv.vecCurvePoints.size() << endl; // size of the curve
        out<<" Curve attrib "<< cv.attribute<< endl; // size of the curve

        int j;

        for ( j = 0; j < cv.vecCurvePoints.size(); j++) // for each point in the curve
        {
            pt=cv.vecCurvePoints[j];
            out<<pt.x<<" "; // print the point x
        }
        out<<endl;
        for ( j = 0; j < cv.vecCurvePoints.size(); j++) 
        {
            pt=cv.vecCurvePoints[j];
            out<<pt.y<<" "; // print the point y
        }
        out << endl;
    }
    return out;
}

