/***************************************************************************
                           CFilterContrast.cpp  -  description
                              -------------------
     begin                : 30.08.2002
     copyright            : (C) 2002 Marc Hensel
     email                : tuvisionlib@tuvision.de, marc.hensel@web.de
     origin               : Diploma thesis Marc Hensel
 
***************************************************************************/


/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/

#include <tuv2d/CFilterContrast.h>

// using namespace std;
using namespace TUVision;


/**************************************************
 *
 * Constructor and destructor
 *
 **************************************************/

//
//! Constructor.
//

CFilterContrast::CFilterContrast()
{
}


//
//! Destructor.
//

CFilterContrast::~CFilterContrast()
{
}


/**************************************************
 *
 * Public methods
 *
 **************************************************/

//
//! Filter, i.e. replace pixel intensity with local contrast (max_value - min_value) in n x n window.
//

bool CFilterContrast::filter(CImageByteGray *img,unsigned char filterWidth)
{
    if(!(filterWidth%2)) return false;		// Filter length has to be odd

    int		width,height;					//!< Image dimensions
    int		x,y;							//!< Image coordinate to be filtered
    int		dx,dy;							//!< Relative image coordinate in filter window
    int		value,max_value,min_value;
    unsigned char	offset = filterWidth/2;	//!< Max. relative position in centered filter window
    unsigned char	*row;					//!< Pointer: 1st pixel in row (image matrix)
    CImageByteGray	*src_img;				//!< Image to be processed

    width	= img->getW();
    height	= img->getH();
    src_img = new CImageByteGray (*img);
    img->clearImage();

    // Run through pixels (positions to be filtered)
    for (y=offset; y<height-offset; y++)
    {
        for (x=offset; x<width-offset; x++)
        {
            // Evaluate filter window
            max_value	= 0;
            min_value	= 255;
            for (dy=-offset; dy<=offset; dy++)
            {
                row = src_img->getRow(y+dy);
                for (dx=-offset; dx<=offset; dx++)
                {
                    value = src_img->getPixel(x+dx,y+dy,0);
                    if (value>max_value) max_value = value;
                    if (value<min_value) min_value = value;
                }
            }
            img->setPixel(x,y,max_value-min_value,0);	// Store contrast in output image
        }
    }
    delete src_img;
    return true;
}


