/***************************************************************************
                          CFilterSobel.cpp  -  description
                             -------------------
    begin                : 01.01.2001
    copyright            : (C) 2001 by Marco Budde
    email                : tuvisionlib@tuvision.de
    origin               : Diplomarbeit Marco Budde

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
#include "tuv2d/CFilterSobel.h"
#include <cstdlib>
#include <cmath>

#include "tuv/CImageByteGray.h"
#include "tuv/CImageIntGray.h"
#include "tuv/CImageDoubleGray.h"


using namespace TUVision;
//using namespace std;


void CFilterSobel::filter (CImageByteGray &img)
{
    CImageByteGray  *orig_img;
    unsigned char   *row[3];
    unsigned char   *row_target;
    int             x, y; 
    int             tmp_x, tmp_y;
    unsigned int    tmp_abs;


    orig_img = new CImageByteGray (img);

    img.clearImage();

    for (y = 1; y < orig_img->getH() - 1; y++)
    {
        row[0] = orig_img->getRow (y-1);   
        row[1] = orig_img->getRow (y);
        row[2] = orig_img->getRow (y+1);
        row_target = img.getRow (y);

        for (x = 1; x < orig_img->getW() - 1; x++)
        {
            tmp_x = row [0][x-1] -
                row [0][x+1] +
                row [2][x-1] -
                row [2][x+1] +
                2 * (row [1][x-1] - row [1][x+1]);
            tmp_y = row [2][x-1] +
                row [2][x+1] -
                row [0][x-1] -
                row [0][x+1] + 
                2 * (row [2][x] - row [0][x]);

            tmp_abs = abs (tmp_x / 4) + abs (tmp_y / 4);

            row_target[x] = tmp_abs >> 1;
        }
    }

    delete orig_img;
}


void CFilterSobel::filter (CImageIntGray &img)
{
    CImageIntGray   *orig_img;
    int             *row[3];
    int             *row_target;
    int             x, y; 
    int             tmp_x, tmp_y, tmp_abs;


    orig_img = new CImageIntGray (img);

    img.clearImage();

    for (y = 1; y < orig_img->getH() - 1; y++)
    {
        row[0] = orig_img->getRow (y-1);   
        row[1] = orig_img->getRow (y);
        row[2] = orig_img->getRow (y+1);
        row_target = img.getRow (y);

        for (x = 1; x < orig_img->getW() - 1; x++)
        {
            tmp_x = row [0][x-1] -
                row [0][x+1] +
                row [2][x-1] -
                row [2][x+1] +
                2 * (row [1][x-1] - row [1][x+1]);
            tmp_y = row [2][x-1] +
                row [2][x+1] -
                row [0][x-1] -
                row [0][x+1] + 
                2 * (row [2][x] - row [0][x]);

            tmp_abs = abs (tmp_x / 4) + abs (tmp_y / 4);

            row_target[x] = tmp_abs / 2;
        }
    }

    delete orig_img;
}


void CFilterSobel::filter (CImageDoubleGray &img)
{
    CImageDoubleGray   *orig_img;
    double             *row[3];
    double             *row_target;
    int                x, y; 
    double             tmp_x, tmp_y, tmp_abs;


    orig_img = new CImageDoubleGray (img);

    img.clearImage();

    for (y = 1; y < orig_img->getH() - 1; y++)
    {
        row[0] = orig_img->getRow (y-1);   
        row[1] = orig_img->getRow (y);
        row[2] = orig_img->getRow (y+1);
        row_target = img.getRow (y);

        for (x = 1; x < orig_img->getW() - 1; x++)
        {
            tmp_x = row [0][x-1] -
                row [0][x+1] +
                row [2][x-1] -
                row [2][x+1] +
                2.0 * (row [1][x-1] - row [1][x+1]);
            tmp_y = row [2][x-1] +
                row [2][x+1] -
                row [0][x-1] -
                row [0][x+1] + 
                2.0 * (row [2][x] - row [0][x]);

            tmp_abs = abs ((int) (tmp_x / 4.0)) + abs ((int) (tmp_y / 4.0));

            row_target[x] = tmp_abs / 2.0;
        }
    }

    delete orig_img;
}
