/***************************************************************************
                          CFilterThinning.cpp  -  description
                             -------------------
    begin                : 01.01.2001
    copyright            : (C) 2001 by Marco Budde
    email                : tuvisionlib@tuvision.de
    origin               : Diplomarbeit Marco Budde

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
#include "tuv/CImageByteBilevel.h"
#include "tuv2d/CFilterThinning.h"


//using namespace std;
using namespace TUVision;


void CFilterThinning::filter (CImageByteBilevel &img)
{
    CImageByteBilevel  *tmp;
    unsigned char      tmp_list [8];
    int                x, y;
    unsigned int       i, n_p, s_p;


    tmp = new CImageByteBilevel (img);

    // first step 
    for (y = 1; y < img.getH()-1; y++)
        for (x = 1; x < img.getW()-1; x++)
            if (img.getPixel (x, y, 0) == 1)   // contour point 
            {
                tmp_list[0] = img.getPixel (x, y-1, 0);
                tmp_list[1] = img.getPixel (x+1, y-1, 0);
                tmp_list[2] = img.getPixel (x+1, y, 0);
                tmp_list[3] = img.getPixel (x+1, y+1, 0);
                tmp_list[4] = img.getPixel (x, y+1, 0);
                tmp_list[5] = img.getPixel (x-1, y+1, 0);
                tmp_list[6] = img.getPixel (x-1, y, 0);
                tmp_list[7] = img.getPixel (x-1, y-1, 0);

                // calculate number of nonzero neighbours: N(p1) 
                n_p = 0;
                for (i=0; i < 8; i++)
                    if (tmp_list[i] == 1)
                        n_p++;

                // calculate number of 0-1 transitions: S(p1)  
                s_p = 0;
                for (i=0; i < 8; i++)
                    if (tmp_list[i] == 0 && tmp_list[(i+1)%8] == 1)
                        s_p++;

                if ((n_p >= 2) && (n_p <= 6) && (s_p == 1) &&
                    (tmp_list[0]*tmp_list[2]*tmp_list[4] == 0) &&
                    (tmp_list[2]*tmp_list[4]*tmp_list[6] == 0))
                    tmp->setPixel (x, y, 0, 0);
                else
                    tmp->setPixel (x, y, 1, 0);
            }
            else
                tmp->setPixel (x, y, 0, 0);

    /* second step */
    for (y = 1; y < tmp->getH()-1; y++)
        for (x = 1; x < tmp->getW()-1; x++)
            if (tmp->getPixel (x, y, 0) == 1)   // contour point 
            {
                tmp_list[0] = tmp->getPixel (x, y-1, 0);
                tmp_list[1] = tmp->getPixel (x+1, y-1, 0);
                tmp_list[2] = tmp->getPixel (x+1, y, 0);
                tmp_list[3] = tmp->getPixel (x+1, y+1, 0);
                tmp_list[4] = tmp->getPixel (x, y+1, 0);
                tmp_list[5] = tmp->getPixel (x-1, y+1, 0);
                tmp_list[6] = tmp->getPixel (x-1, y, 0);
                tmp_list[7] = tmp->getPixel (x-1, y-1, 0);

                // calculate number of nonzero neighbours: N(p1)   
                n_p = 0;
                for (i=0; i < 8; i++)
                    if (tmp_list[i] == 1)
                        n_p++;

                // calculate number of 0-1 transitions: S(p1)   
                s_p = 0;
                for (i=0; i < 8; i++)
                    if (tmp_list[i] == 0 && tmp_list[(i+1)%8] == 1)
                        s_p++;

                if ((n_p >= 2) && (n_p <= 6) && (s_p == 1) &&
                    (tmp_list[0]*tmp_list[2]*tmp_list[6] == 0) &&
                    (tmp_list[0]*tmp_list[4]*tmp_list[6] == 0))
                    img.setPixel (x, y, 0, 0);
                else
                    img.setPixel (x, y, 1, 0);
            }
            else
                img.setPixel (x, y, 0, 0);

    delete tmp;
}
