/***************************************************************************
                          COperatorGray.cpp  -  description
                             -------------------
    begin                : 01.01.2001
    copyright            : (C) 2001 by Marco Budde
    email                : tuvisionlib@tuvision.de
    origin               : Diplomarbeit Marco Budde

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
#include "tuv2d/COperatorGray.h"
#include "tuv/CImageByteGray.h"
#include "tuv/CImageIntGray.h"

using namespace TUVision;

/* *******************************************************/
/*! 
 *   subtract rhs from lhs pixel by pixel.
 *
 *      \param lhs the image to be subtracted from.
 *      \param rhs the image that is subtracted from lhs.
 *
 *      \return the integer valued image. 
 *              if the images are
 *              not equal in size, the returned image
 *              has the minimum size of both images.
 *
 *   \ingroup operator
 *
 */
/* *******************************************************/
CImageIntGray 
TUVision::operator- (const CImageByteGray &lhs,const CImageByteGray &rhs)
{
    int min_w=lhs.getW();
    /* compare the width */
    if (min_w > rhs.getW())
        min_w=rhs.getW();

    int min_h=lhs.getH();
    /* compare the height */
    if (min_h > rhs.getH())
        min_h=rhs.getH();

    CImageIntGray img_dst(min_w,min_h);

    int                  x_cnt, y_cnt;  
    const unsigned char  *src_r;
    const unsigned char  *src_l;
    int* dst;
    /* compare the values of the pixels */
    for (y_cnt = 0; y_cnt < min_h; y_cnt++)
    {
        src_r = rhs.getRow (y_cnt);
        src_l = lhs.getRow (y_cnt);

        dst = img_dst.getRow (y_cnt);

        for (x_cnt = 0; x_cnt < min_w; x_cnt++)
            *dst++=(int)*src_l++ - *src_r++;
    }

    return (CImageIntGray(img_dst,false));
}

/* *******************************************************/
/*! 
 *   subtract rhs from lhs pixel by pixel.
 *
 *      \param lhs the image to be subtracted from.
 *      \param rhs the image that is subtracted from lhs.
 *
 *      \return the integer valued image. 
 *              if the images are
 *              not equal in size, the returned image
 *              has the minimum size of both images.
 *
 *   \ingroup operator
 *
 */
/* *******************************************************/
CImageIntGray 
TUVision::operator- (const CImageIntGray &lhs,const CImageIntGray &rhs)
{
    int min_w=lhs.getW();
    /* compare the width */
    if (min_w > rhs.getW())
        min_w=rhs.getW();

    int min_h=lhs.getH();
    /* compare the height */
    if (min_h > rhs.getH())
        min_h=rhs.getH();

    CImageIntGray img_dst(min_w,min_h);

    int        x_cnt, y_cnt;  
    const int  *src_r, *src_l;
    int        *dst;
    /* compare the values of the pixels */
    for (y_cnt = 0; y_cnt < min_h; y_cnt++)
    {
        src_r = rhs.getRow (y_cnt);
        src_l = lhs.getRow (y_cnt);

        dst = img_dst.getRow (y_cnt);

        for (x_cnt = 0; x_cnt < min_w; x_cnt++)
            *dst++=*src_l++ - *src_r++;
    }

    return (CImageIntGray(img_dst,false));
}

