/***************************************************************************
                          COperatorMaxColDiff.cpp  -  description
                             -------------------
    begin                : 01.01.2001
    copyright            : (C) 2001 by Islam Shdaifat
    email                : tuvisionlib@tuvision.de
    origin               : PhD - Thesis Islam Shdaifat

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
#include "tuv2d/COperatorMaxColDiff.h"
#include "tuv/CImageByteRGB.h"
#include "tuv/CImageByteGray.h"
#include "tuv/CImageIntGray.h"
#include <cmath>
#include <cstdlib>


/* *******************************************************/
/*! 
 *   calculate the maximum color difference
 *
 *      \param rgb the image to be converted.
 *
 *      \return the gray valued image. 
 *
 *   \ingroup operator
 *
 */
/* *******************************************************/
TUVision::CImageByteGray 
TUVision::getMaxColDiff (const TUVision::CImageByteRGB &rgb)
{   
    int w;
    int h;

    w=rgb.getW();
    h=rgb.getH();

    CImageByteGray gray(w,h); // the output image
    gray.clearImage(); // reset the input image

    int curh; // current row
    int curw; // current colomn

    const unsigned char *src_1, *src_2 , *src_3; // for each color channel
    const unsigned char *src1, *src2 , *src3;    // and for +3 -3 rows
    const unsigned char *src0;                   // the current row

    unsigned char* dst;                          // we store here

    for (curh = 3;curh< h-3; curh++) {  // for all the rows in the image
        src3  = rgb.getRow(curh+3);     // vertical +3
        src2  = rgb.getRow(curh+2);     // vertical +2
        src1  = rgb.getRow(curh+1);     // vertical +1
        src0  = rgb.getRow(curh)  ;     // vertical 0
        src_1 = rgb.getRow(curh-1);     // vertical -1
        src_2 = rgb.getRow(curh-2);     // vertical -2
        src_3 = rgb.getRow(curh-3);     // vertical -3

        dst=gray.getRow(curh);          // the distination image

        for (curw = 3;curw< w-3; curw++) {  // for all the rows in the image
            int diff = 0, avg1, avg2;

            for(int i=0; i<3;i++) { //  for all the color channels
                avg1 = (*src1 + *src2 + *src3)/3;    // vertical+ average
                avg2 = (*src_1 + *src_2 + *src_3)/3; //vertical- average
                diff += abs(avg1 - avg2);   // accumlate the color difference

                avg1 = ( *(src0+3) + *(src0+6) + *(src0+9) ) /3; //horizontal+ avg
                avg2 = ( *(src0-3) + *(src0-6) + *(src0-9) ) /3; //horizontal- avg
                diff += abs(avg1 - avg2);   // accumlate the color difference

                src0++;                     // one step to the other channel
                src1++; src2++; src3++;     // one step to the other channel
                src_1++; src_2++; src_3++;  // one step to the other channel
            }
            if(diff > 255 ) diff = 255; // limit the output
            *dst++=(unsigned char)diff;
        } // next pixel
    } // next row
    return(CImageByteGray(gray,false));
}


TUVision::CImageIntGray 
TUVision::getMaxColDiffInt (const TUVision::CImageByteRGB &rgb)
{   
    int w;
    int h;

    w=rgb.getW();
    h=rgb.getH();

    CImageIntGray gray(w,h); // the output image
    gray.clearImage(); // reset the input image

    int curh; // current row
    int curw; // current colomn

    const unsigned char *src_1, *src_2 , *src_3; // for each color channel
    const unsigned char *src1, *src2 , *src3;    // and for +3 -3 rows
    const unsigned char *src0;                   // the current row

    int* dst;                          // we store here

    for (curh = 3;curh< h-3; curh++) {  // for all the rows in the image
        src3  = rgb.getRow(curh+3);     // vertical +3
        src2  = rgb.getRow(curh+2);     // vertical +2
        src1  = rgb.getRow(curh+1);     // vertical +1
        src0  = rgb.getRow(curh)  ;     // vertical 0
        src_1 = rgb.getRow(curh-1);     // vertical -1
        src_2 = rgb.getRow(curh-2);     // vertical -2
        src_3 = rgb.getRow(curh-3);     // vertical -3

        dst=gray.getRow(curh);          // the distination image

        for (curw = 3;curw< w-3; curw++) {  // for all the rows in the image
            int diff = 0, avg1, avg2;

            for(int i=0; i<3;i++) { //  for all the color channels
                avg1 = (*src1 + *src2 + *src3)/3;    // vertical+ average
                avg2 = (*src_1 + *src_2 + *src_3)/3; //vertical- average
                diff += abs(avg1 - avg2);   // accumlate the color difference

                avg1 = ( *(src0+3) + *(src0+6) + *(src0+9) ) /3; //horizontal+ avg
                avg2 = ( *(src0-3) + *(src0-6) + *(src0-9) ) /3; //horizontal- avg
                diff += abs(avg1 - avg2);   // accumlate the color difference

                src0++;                     // one step to the other channel
                src1++; src2++; src3++;     // one step to the other channel
                src_1++; src_2++; src_3++;  // one step to the other channel
            }
            *dst++= diff;
        } // next pixel
    } // next row
    return(CImageIntGray(gray,false));
}


TUVision::CImageByteGray 
TUVision::getMaxDiffGray (const TUVision::CImageByteGray &rgb)
{   
    int w;
    int h;

    w=rgb.getW();
    h=rgb.getH();

    CImageByteGray gray(w,h); // the output image
    gray.clearImage(); // reset the input image

    int curh; // current row
    int curw; // current colomn

    const unsigned char *src_1, *src_2 , *src_3; // for each color channel
    const unsigned char *src1, *src2 , *src3;    // and for +3 -3 rows
    const unsigned char *src0;                   // the current row

    unsigned char* dst;                          // we store here

    for (curh = 3;curh< h-3; curh++) {  // for all the rows in the image
        src3  = rgb.getRow(curh+3);     // vertical +3
        src2  = rgb.getRow(curh+2);     // vertical +2
        src1  = rgb.getRow(curh+1);     // vertical +1
        src0  = rgb.getRow(curh)  ;     // vertical 0
        src_1 = rgb.getRow(curh-1);     // vertical -1
        src_2 = rgb.getRow(curh-2);     // vertical -2
        src_3 = rgb.getRow(curh-3);     // vertical -3

        dst=gray.getRow(curh);          // the distination image

        for (curw = 3;curw< w-3; curw++) {  // for all the colomns in the image
            int diff = 0, avg1, avg2;

            avg1 = (*src1 + *src2 + *src3)/3;    // vertical+ average
            avg2 = (*src_1 + *src_2 + *src_3)/3; //vertical- average
            diff += abs(avg1 - avg2);   // accumlate the color difference
            avg1 = ( *(src0+3) + *(src0+6) + *(src0+9) ) /3; //horizontal+ avg
            avg2 = ( *(src0-3) + *(src0-6) + *(src0-9) ) /3; //horizontal- avg
            diff += abs(avg1 - avg2);   // accumlate the color difference
            src0++;                     // one step to the other channel
            src1++; src2++; src3++;     // one step to the other channel
            src_1++; src_2++; src_3++;  // one step to the other channel
            if(diff > 255 ) diff = 255; // limit the output
            *dst++=(unsigned char)diff;
        } // next pixel
    } // next row
    return(CImageByteGray(gray,false));
}
