/***************************************************************************
                          COperatorTransform.cpp  -  description
                             -------------------
    begin                : 01.01.2001
    copyright            : (C) 2001 by Islam Shdaifat
    email                : tuvisionlib@tuvision.de
    origin               : PhD - Thesis Islam Shdaifat

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
#include "tuv2d/COperatorTransform.h"
#include "tuv/CImageByte.h"
#include "tuv/CImageByteGray.h"
#include "tuv/CImageByteRGB.h"
#include "tuv/CImageByteYUV.h"
#include "tuv/CImageInt.h"
#include "tuv/CImageIntGray.h"
#include "tuv/CImageIntRGB.h"
#include "tuv/CImageIntYUV.h"
#include "tuv/CImageDouble.h"
#include "tuv/CImageDoubleGray.h"
#include "tuv/CImageDoubleRGB.h"
#include "tuv/CImageDoubleYUV.h"
#include "tuv/CImageByteBilevel.h"
#include <cmath>
#include <memory>
#include <cstdlib>

using namespace std;
using namespace TUVision;

#if !defined(M_PI)
#define M_PI  3.14159265358979323846
#endif


/**********************************************************/
/*!
 *   Constructor
 */
/**********************************************************/

COperatorTransform::COperatorTransform( )
{

}

/**********************************************************/
/*!
 * Flip the image in multiples of 90
 * i.e. 0, 90, 180, .....
 * 
 *   \param image    input image to be flipped
 *   \param rotation the 90 degree multiplier
 *
 *   \param return flipped image
 */
/**********************************************************/
CImageByteGray*
COperatorTransform::integralRotateImageByte(const CImageByteGray& image,
                                            unsigned int rotations)
{
    CImageByteGray
        *rotated_image; // the rotated image to be returned
    int w_im = image.getW();
    int h_im = image.getH();

    register int
        x ,  y;

    register unsigned char const
        *p_in_image;

    register unsigned char 
        *p_rot_image;

    /*
    Initialize rotated image attributes.
    */
    rotations%=4; // we here get 0,1,2,3 the quadrant

    switch (rotations)
    {
    case 0:
        {
            /*
             * Rotate 0 degrees. just copy the two images do nothing
             */
            rotated_image= new CImageByteGray(image);

            if (rotated_image == (CImage *) 0)
            {
                // TODO:have to throw exeption
                return(0);
            }

            break;
        }
    case 1:
        {
            /*
            Rotate 90 degrees. interchange the image axis
            */
            rotated_image= new CImageByteGray(image.getH(), image.getW());
            int w_rot = rotated_image->getW();
            int h_rot = rotated_image->getH();

            if (rotated_image == (CImage *) 0)
            {
                // TODO:have to throw exeption
                return(0);
            }

            int d_m =abs(image.getRow(0)-image.getRow(1));

            const unsigned char *p_tmp = image.getRow(0) +w_im -1;
            p_in_image = p_tmp;
            for (y=0; y < h_rot; y++)
            {
                p_rot_image = rotated_image->getRow(y);

                for (x=0; x < w_rot; x++)
                {
                    (*p_rot_image) = (*p_in_image);
                    p_rot_image++ ;
                    p_in_image += d_m;
                }
                p_in_image = p_tmp--;
            }
            break;
        }
    case 2:
        {
            /*
            Rotate 180 degrees.
            */
            rotated_image= new CImageByteGray(image.getW(), image.getH());
            int w_rot = rotated_image->getW();
            int h_rot = rotated_image->getH();

            if (rotated_image == 0)
            {
                // TODO:have to throw exeption
                return(0);
            }

            for (y=1; y <= h_rot; y++)
            {
                p_rot_image = rotated_image->getRow(h_rot-y);
                p_in_image = image.getRow(y-1) + w_im;

                for (x=0; x < w_rot; x++)
                {
                    *p_rot_image++ = (*p_in_image);
                    p_in_image--;
                }
            }

            break;
        }
    case 3:
        {
            /*
            Rotate 270 degrees.
            */
            rotated_image= new CImageByteGray(image.getH(), image.getW());
            int w_rot = rotated_image->getW();
            int h_rot = rotated_image->getH();

            if (rotated_image == 0)
            {
                // TODO:have to throw exeption
                return(0);
            }

            int d_m =abs(image.getRow(0)-image.getRow(1));

            const unsigned char *p_tmp = image.getRow(h_im-1);
            p_in_image = p_tmp;
            for (y=0; y < h_rot; y++)
            {
                p_rot_image = rotated_image->getRow(y);

                for (x=0; x < w_rot; x++)
                {
                    *p_rot_image++ = (*p_in_image);
                    p_in_image -= d_m;
                }
                p_in_image = p_tmp++;
            }
            break;
        }
    }
    return(rotated_image);
}

/**********************************************************/
/*!
 * Flip the image RGB in multiples of 90
 * i.e. 0, 90, 180, .....
 * 
 *   \param image    input image to be flipped
 *   \param rotation the 90 degree multiplier
 *
 *   \param return flipped image
 */
/**********************************************************/
CImageByteRGB*
COperatorTransform::integralRotateImageRGB(const CImageByteRGB& image,
                                           unsigned int rotations)
{
    CImageByteRGB
        *rotated_image; // the rotated image to be returned
    int w_im = image.getW();
    int h_im = image.getH();

    register int
        x ,  y;

    register unsigned char const
        *p_in_image;

    register unsigned char 
        *p_rot_image;

    /*
    Initialize rotated image attributes.
    */
    rotations%=4; // we here get 0,1,2,3 the quadrant

    switch (rotations)
    {
    case 0:
        {
            /*
             * Rotate 0 degrees. just copy the two images do nothing
             */
            rotated_image= new CImageByteRGB(image);

            if (rotated_image == (CImage *) 0)
            {
                // TODO:have to throw exeption
                return(0);
            }

            break;
        }
    case 1:
        {
            /*
            Rotate 90 degrees. interchange the image axis
            */
            rotated_image= new CImageByteRGB(image.getH(), image.getW());
            int w_rot = rotated_image->getW();
            int h_rot = rotated_image->getH();

            if (rotated_image == (CImage *) 0)
            {
                // TODO:have to throw exeption
                return(0);
            }

            int d_m =abs(image.getRow(0)-image.getRow(1));

            const unsigned char *p_tmp = image.getRow(0) +w_im -1;
            p_in_image = p_tmp;
            for (y=0; y < h_rot; y++)
            {
                p_rot_image = rotated_image->getRow(y);

                for (x=0; x < w_rot; x++)
                {
                    (*p_rot_image) = (*p_in_image);
                    p_rot_image++ ;
                    p_in_image += d_m;
                }
                p_in_image = p_tmp--;
            }
            break;
        }
    case 2:
        {
            /*
            Rotate 180 degrees.
            */
            rotated_image= new CImageByteRGB(image.getW(), image.getH());
            int w_rot = rotated_image->getW();
            int h_rot = rotated_image->getH();

            if (rotated_image == 0)
            {
                // TODO:have to throw exeption
                return(0);
            }

            for (y=1; y <= h_rot; y++)
            {
                p_rot_image = rotated_image->getRow(h_rot-y);
                p_in_image = image.getRow(y-1) + 3*w_im;

                for (x=0; x < w_rot; x++)
                {
                    *p_rot_image++ = *p_in_image++;
                    *p_rot_image++ = *p_in_image++;
                    *p_rot_image++ = *p_in_image++;
                    p_in_image -= 6;
                }
            }

            break;
        }
    case 3:
        {
            /*
            Rotate 270 degrees.
            */
            rotated_image= new CImageByteRGB(image.getH(), image.getW());
            int w_rot = rotated_image->getW();
            int h_rot = rotated_image->getH();

            if (rotated_image == 0)
            {
                // TODO:have to throw exeption
                return(0);
            }

            int d_m =abs(image.getRow(0)-image.getRow(1));

            const unsigned char *p_tmp = image.getRow(h_im-1);
            p_in_image = p_tmp;
            for (y=0; y < h_rot; y++)
            {
                p_rot_image = rotated_image->getRow(y);

                for (x=0; x < w_rot; x++)
                {
                    *p_rot_image++ = (*p_in_image);
                    p_in_image -= d_m;
                }
                p_in_image = p_tmp++;
            }
            break;
        }
    }
    return(rotated_image);
}

#include "tuv2d/COperatorMultiplex.h"

CImageByteRGB*
COperatorTransform::rotateImageRGB(const CImageByteRGB& image,
                                   double angle)
{
    COperatorMultiplex mux;
    // mux gray <-> RGB mux demux
    vector <CImageByteGray*> vecRGB, vectRGBrotated;
    mux.demultiplex (image,vecRGB);

    int j;

    // rotate all of the image
    for(j = 0; j < 3; j++)
    {
        CImageByteGray *rotatedImage = rotateImageByte(*vecRGB[j],angle);
        delete vecRGB[j];
        vectRGBrotated.push_back(rotatedImage);
    }
    // construct an RGB from R, G, and B
    CImageByteRGB *rotatedRGB;
    mux.multiplex (vectRGBrotated, &rotatedRGB);

    // delete the intermeiate images
    for(j = 0; j < 3; j++)
    {
        delete vectRGBrotated[j];
    }

    return rotatedRGB;
}

CImageByteGray*
COperatorTransform::rotateImageByte(const CImageByteGray& image,
                                    double angle)
{
    CImageByteGray
        *rotated_image; // the rotated image to be returned
    const int shf  = 10;
    const int shf2 = 2*shf;
    int x ,  y;

    register unsigned char const
        *p_in_image;

    register unsigned char 
        *p_rot_image;

    // the angle must be between 0 - 360
    while (angle < 0)
        angle+=2*M_PI;
    while (angle > 2*M_PI)
        angle-=2*M_PI;


    register int cost = (int) ((cos(angle))*(1<<shf));
    register int sint = (int) ((sin(angle))*(1<<shf));

    register int w_rot = (image.getW()*abs(cost) + image.getH()*abs(sint))>>shf;
    register int h_rot = (image.getW()*abs(sint) + image.getH()*abs(cost))>>shf;

    rotated_image= new CImageByteGray(w_rot, h_rot);

    if (rotated_image == (CImage *) 0)
    {
        // TODO:have to throw exeption
        return(0);
    }

    register int w_im = image.getW();
    register int h_im = image.getH();

    register int x1, y1, x0, y0;
    int x_crop, y_crop;
    // find the origin
    // the first quadrant case
    if(angle >=0 && angle < M_PI/2)
    {
        x0 = (w_im*sint*sint)>>shf;
        y0 = (-w_im*sint*cost)>>shf;
        x_crop = (x0>>shf);
        y_crop = -(y0>>shf);
    }

    // the second quadrant case
    if(angle >=M_PI/2 && angle < M_PI)
    {
        x0 = ((w_im*cost*cost)>>shf) + ((h_rot*sint));
        y0 = ((w_im*cost*sint)>>shf) - ((h_rot*cost));
    }

    // the third quadrant case
    if(angle >=M_PI && angle < 1.5*M_PI)
    {
        x0 = ((-h_im*cost*sint)>>shf) - ((w_rot*cost));
        y0 = ((h_im*cost*cost)>>shf) - ((w_rot*sint));
    }

    // the fourth quadrant case
    if(angle >=1.5*M_PI && angle < 2.0*M_PI)
    {
        x0 = ((h_im*sint*cost)>>shf);
        y0 = ((h_im*sint*sint)>>shf);
    }

    register int tx1;
    register int ty1;
    register int tys=0;
    register int tyc=0;
    /*
    x1 = x*cost - y*sint + x0
    y1 = x*sint + y*cost + y0
    is implemented here throw addition and substraction
    */

    int d_m_im  = abs(image.getRow(0)-image.getRow(1));

    p_in_image = image.getRow(0);

    for (y=0; y < h_rot; y++)
    {
        tx1=x0+tys;
        ty1=(y0+tyc);
        p_rot_image = rotated_image->getRow(y);

        for (x=0; x < w_rot; x++)
        {
            x1=tx1>>shf;
            y1=ty1>>shf;

            if(x1<0)
            {
                *p_rot_image = *(p_in_image + y1*d_m_im) ;
            }
            else
            {
                if(x1>= w_im)
                {
                    *p_rot_image = *(p_in_image + w_im+ y1*d_m_im-1) ;
                }
                else
                {
                    if(y1<0)
                    {
                        *p_rot_image = *(p_in_image + x1) ;
                    }
                    else
                    {
                        if(y1>=h_im)
                        {
                            *p_rot_image = *(p_in_image + x1 + (h_im-1)*d_m_im) ;
                        }
                        else
                        {
                            *p_rot_image = *(p_in_image + x1 +y1*d_m_im) ;
                        }
                    }
                }
            }

            p_rot_image++ ;

            tx1+=cost;
            ty1+=sint;

        }
        tys-=sint;
        tyc+=cost;
    }

    return(rotated_image);
}


void
COperatorTransform::padImageByte(CImageByteGray& rotated_image,
                                 int x_pad, int y_pad)
{
    //	int w_im = image.getW();
    //	int h_im = image.getH();
    int w_rot = rotated_image.getW();
    int h_rot = rotated_image.getH();
    x_pad = abs(x_pad);
    y_pad = abs(y_pad);

    int x ,  y;
    unsigned char tmp;//*pad_data, tmp;

    if(x_pad > y_pad)
    {
        //pad_data = new unsigned char[x_pad +1];
        for (x=0; x < x_pad; x++)
        {
            int yf = y_pad - (x*y_pad)/x_pad ;
            tmp = rotated_image.getPixel(x,yf+10);
            for (y=0; y <= yf ; y++)
            {
                rotated_image.setPixel(x,y,tmp);
            }
        }
    }
}

/*
 * -pi/2 < angle < pi/2
 */
CImageByteGray*
COperatorTransform::yshearImageByte(const CImageByteGray& image,
                                    double angle)
{
    const int shf  = 10;
    const int shf2 = 2*shf;

    int w_im = image.getW();
    int h_im = image.getH();

    int tant = (int) (tan(angle))*(1<<shf);

    int w_rot = w_im + abs((h_im*tant)>>shf);
    int h_rot = h_im;

    CImageByteGray
        *rotated_image; // the rotated image to be returned
    register int  y;

    register unsigned char const
        *p_in_image;

    register unsigned char 
        *p_rot_image;

    // the angle must be between 0 - 360
    while (angle < -M_PI/2)
        angle+=2*M_PI;
    while (angle >= 3*M_PI/2)
        angle-=2*M_PI;

    rotated_image= new CImageByteGray(w_rot, h_rot);

    if (rotated_image == (CImage *) 0)
    {
        // TODO:have to throw exeption
        return(0);
    }

    enum {LEFT,
        RIGHT} shear_direction;
    int x0 ;

    x0 = abs(((h_im*tant)>>shf));
    // the first quadrant case
    if(angle >=0 && angle < M_PI/2)
        shear_direction = LEFT;

    // the second quadrant case
    if(angle >= -M_PI/2 && angle < 0)
        shear_direction = RIGHT;

    switch(shear_direction)
    {
    case LEFT:
        {
            int xf ;
            for (y=0; y < h_rot; y++)
            {
                xf = ((y*tant)>>shf);
                p_rot_image = rotated_image->getRow(y);
                p_in_image = image.getRow(y);
                memset( p_rot_image, (*p_in_image), xf );
                memcpy( p_rot_image + xf, p_in_image , w_im);
                memset( p_rot_image + xf + w_im, *(p_in_image + w_im-1),x0 - xf );
            }
            break;
        }
    case RIGHT:
        {
            int xf;
            for (y=0; y < h_rot; y++)
            {
                xf = x0+((y*tant)>>shf);
                p_rot_image = rotated_image->getRow(y);
                p_in_image = image.getRow(y);
                memset( p_rot_image, (*p_in_image), xf );
                memcpy( p_rot_image + xf, p_in_image , w_im);
                memset( p_rot_image + xf + w_im, *(p_in_image + w_im-1),x0 - xf );
            }
            break;
        }
    }
    return(rotated_image);
}


CImageByteGray*
COperatorTransform::xshearImageByte(const CImageByteGray& image,
                                    double angle)
{
    const int shf  = 10;
    const int shf2 = 2*shf;

    int w_im = image.getW();
    int h_im = image.getH();

    int tant = (int) (tan(angle))*(1<<shf);

    int w_rot = w_im ;
    int h_rot = h_im + abs((w_im*tant)>>shf);

    CImageByteGray
        *rotated_image; // the rotated image to be returned
    register int x ,  y;

    register unsigned char const
        *p_in_image;

    register unsigned char 
        *p_rot_image;

    // the angle must be between 0 - 360
    while (angle < -M_PI/2)
        angle+=2*M_PI;
    while (angle >= 3*M_PI/2)
        angle-=2*M_PI;

    rotated_image= new CImageByteGray(w_rot, h_rot);

    if (rotated_image == (CImage *) 0)
    {
        // TODO:have to throw exeption
        return(0);
    }

    enum {UP,
        DOWN} shear_direction;
    int  y1,  y0, y2;

    y0 = abs(((w_im*tant)>>shf));
    // the first quadrant case
    if(angle >=0 && angle < M_PI/2)
        shear_direction = UP;

    // the second quadrant case
    if(angle >=-M_PI/2 && angle < 0)
        shear_direction = DOWN;

    const unsigned char *p_in_top    = image.getRow(0);
    const unsigned char *p_in_bottom = image.getRow(h_im-1);

    unsigned char *p_rot_top    = rotated_image->getRow(0);

    int d_m_im  = abs(image.getRow(0)-image.getRow(1));
    int d_m_rot = abs(rotated_image->getRow(0)-rotated_image->getRow(1));
    p_rot_image = rotated_image->getRow(0);


    switch(shear_direction)
    {
    case UP:
        {
            for (x=0; x < w_rot; x++)
            {
                p_in_image = p_in_top;
                p_rot_image = p_rot_top;
                y1 = y0 - ((x*tant)>>shf);
                y2 = ((x*tant)>>shf);

                for (y=0; y < y1; y++)
                {
                    (*p_rot_image) = *p_in_top;
                    p_rot_image += d_m_rot;
                }
                for (y = y1; y < h_rot - y2; y++)
                {
                    *p_rot_image = *p_in_image;
                    p_in_image += d_m_im;
                    p_rot_image += d_m_rot;
                }
                for (y=h_rot-y2; y < h_rot; y++)
                {
                    *p_rot_image = *p_in_bottom;
                    p_rot_image += d_m_rot;
                }

                p_in_top++;
                p_in_bottom++;
                p_rot_top++;
            }
            break;
        }

    case DOWN:
        {
            for (x=0; x < w_rot; x++)
            {
                p_in_image = p_in_top;
                p_rot_image = p_rot_top;
                y1 = -((x*tant)>>shf);
                y2 = y0 + ((x*tant)>>shf);

                for (y=0; y < y1; y++)
                {
                    (*p_rot_image) = *p_in_top;
                    p_rot_image += d_m_rot;
                }
                for (y = y1; y < h_rot - y2; y++)
                {
                    *p_rot_image = *p_in_image;
                    p_in_image += d_m_im;
                    p_rot_image += d_m_rot;
                }
                for (y=h_rot-y2; y < h_rot; y++)
                {
                    *p_rot_image = *p_in_bottom;
                    p_rot_image += d_m_rot;
                }

                p_in_top++;
                p_in_bottom++;
                p_rot_top++;
            }
            break;
        }
    }
    return(rotated_image);
}



CImageByteGray*
COperatorTransform::rotateImageByte1(const CImageByteGray& image,
                                     double angle)
{

    // the angle must be between 0 - 360
    while (angle < 0)
        angle+=2*M_PI;
    while (angle >= 2*M_PI)
        angle-=2*M_PI;
    // in which quadrant
    int theta = (int) (180*angle/M_PI+45);
    int quadrant = theta/90;
    //	angle = angle-quadrant*M_PI/2;
    CImageByteGray* img1 = integralRotateImageByte(image,quadrant);
    CImageByteGray* img3;
    switch (quadrant)
    {
    case (0):
        {
            img3 = yshearImageByte(*img1,angle);
            //img1->~CImageByteGray();
            //img3 = yshearImageByte(*img2,angle);
            //img2->~CImageByteGray();

            break;
        }
    case (1):
        {
            angle -= M_PI/2;
            break;
        }
    case (2):
        {
            angle -= M_PI;
            break;
        }
    case (3):
        {
            break;
        }
    }

    return img3;
}



CImageByteGray*
COperatorTransform::yshearImageByte(const CImageByteGray& image,
                                    double angle, int y_offset)
{
    const int shf  = 10;
    const int shf2 = 2*shf;

    int w_im = image.getW();
    int h_im = image.getH();

    int tant = (int) ((tan(angle))*(1<<shf));

    int w_rot = w_im ;//+ abs((h_im*tant)>>shf); check this 
    int h_rot = h_im;

    CImageByteGray
        *rotated_image; // the rotated image to be returned
    register int    y;

    register unsigned char const
        *p_in_image;

    register unsigned char 
        *p_rot_image;

    // the angle must be between 0 - 360
    while (angle < -M_PI/2)
        angle+=2*M_PI;
    while (angle >= 3*M_PI/2)
        angle-=2*M_PI;

    rotated_image= new CImageByteGray(w_rot, h_rot);

    if (rotated_image == (CImage *) 0)
    {
        // TODO:have to throw exeption
        return(0);
    }

    enum {LEFT,
        RIGHT} shear_direction;
    int   x0 ;

    x0 = abs(((h_im*tant)>>shf));
    // the first quadrant case
    if(angle >=0 && angle < M_PI/2)
        shear_direction = LEFT;

    // the second quadrant case
    if(angle >= -M_PI/2 && angle < 0)
        shear_direction = RIGHT;

    switch(shear_direction)
    {
    case LEFT:
        {
            int xf ;
            int x_offset =  ((y_offset*tant)>>shf);
            for (y=0; y < y_offset; y++)
            {
                xf = ((y*tant)>>shf);
                p_rot_image = rotated_image->getRow(y);
                p_in_image = image.getRow(y);
                //memset( p_rot_image, (*p_in_image), xf );
                memcpy( p_rot_image , p_in_image + x_offset -xf , w_im - x_offset + xf);
                memset( p_rot_image + w_im - x_offset + xf, *(p_in_image + w_im-1),x_offset - xf );
            }

            for (y = y_offset; y < h_rot; y++)
            {
                xf = ((y*tant)>>shf);
                p_rot_image = rotated_image->getRow(y);
                p_in_image = image.getRow(y);
                memset( p_rot_image, (*p_in_image), xf - x_offset);
                memcpy( p_rot_image + xf - x_offset, p_in_image , w_im - x_offset + xf);
                //memset( p_rot_image + xf + w_im, *(p_in_image + w_im-1),x0 - xf );
            }

            break;
        }
    case RIGHT:
        {
            int xf;
            for (y=0; y < h_rot; y++)
            {
                xf = x0+((y*tant)>>shf);
                p_rot_image = rotated_image->getRow(y);
                p_in_image = image.getRow(y);
                memset( p_rot_image, (*p_in_image), xf );
                memcpy( p_rot_image + xf, p_in_image , w_im);
                memset( p_rot_image + xf + w_im, *(p_in_image + w_im-1),x0 - xf );
            }
            break;
        }
    }
    return(rotated_image);
}


/**********************************************************/
/*!
 *  Return the image reflected, left right and right left
 * 
 *   \param image    input image to be mirrored
 *
 *   \param return reflected image
 */
/**********************************************************/
CImageByteGray*
COperatorTransform::mirrorImageByte(const CImageByteGray& image)
{
    int w = image.getW();
    int h = image.getH();

    CImageByteGray
        *mirror_image = new CImageByteGray(w,h); // the mirror image to be returned
    int x,y;
    const unsigned char *src;
    unsigned char *dst;
    for (y=0; y < h; y++)
    {
        src = image.getRow(y) + w -1;
        dst = mirror_image->getRow(y);
        for (x=0; x < w; x++)
        {
            *dst++ = *src--;
        }
    }
    return mirror_image;
}
