/***************************************************************************
                          cpointpairlistnormalizer.cpp  -  description
                             -------------------
    begin                : Mon Apr 15 2002
    copyright            : (C) 2002 by Fabian Wenzel
    email                : wenzel@tu-harburg.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library. It is free software; you   *
 *   can redistribute it and/or modify it under the terms of the           *
 *   GNU Library General Public License as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option)      *
 *   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/

#include "tuv3d/CPointListNormalizer.h"
#include <tuv/CVertex.h>

using namespace TUVision;
using namespace NEWMAT;

/***************************************************************************/
/*!
 *  Constructor normalizes the pointlist and stores all the values
 *
 ***************************************************************************/
CPointPairListNormalizer::CPointPairListNormalizer(const Tpointpairlist &unnormalizedlist,
                                                   unsigned int firstindex,
                                                   unsigned int secondindex) :
firstsidetransformation(3,3),
secondsidetransformation(3,3),
normalizedlist(unnormalizedlist)
{

    // As we made a copy (see initializsation of normalizedlist), computations
    // are done on this copy only
    Tpointpairlist::iterator currentpointpair;

    // First run: Determine offset
    CPoint3D<double> firstoffset;
    CPoint3D<double> secondoffset;

    for(currentpointpair = normalizedlist.begin();
        currentpointpair != normalizedlist.end();
        ++currentpointpair)
    {
        (*currentpointpair)[firstindex ] /= (*currentpointpair)[firstindex ].z;
        (*currentpointpair)[secondindex] /= (*currentpointpair)[secondindex].z;

        firstoffset +=(*currentpointpair)[firstindex ];
        secondoffset+=(*currentpointpair)[secondindex];
    }

    firstoffset /=(double) normalizedlist.size();
    secondoffset/=(double) normalizedlist.size();

    // Second step: Determine scaling factor by also shifting points first
    double firstscaling =0.0;
    double secondscaling=0.0;

    for(currentpointpair  = normalizedlist.begin();
        currentpointpair != normalizedlist.end();
        ++currentpointpair)
    {
        (*currentpointpair)[firstindex]  -=firstoffset ;
        (*currentpointpair)[secondindex] -=secondoffset;
        firstscaling +=(*currentpointpair)[firstindex] .x*(*currentpointpair)[firstindex] .x+
            (*currentpointpair)[firstindex] .y*(*currentpointpair)[firstindex] .y;
        secondscaling+=(*currentpointpair)[secondindex].x*(*currentpointpair)[secondindex].x+
            (*currentpointpair)[secondindex].y*(*currentpointpair)[secondindex].y;
    }

    firstscaling =sqrt((2.0*(double)normalizedlist.size())/firstscaling );
    secondscaling=sqrt((2.0*(double)normalizedlist.size())/secondscaling);

    // Finally, scale points
    for(currentpointpair  = normalizedlist.begin();
        currentpointpair != normalizedlist.end();
        ++currentpointpair)
    {
        (*currentpointpair)[firstindex ] *=firstscaling;
        (*currentpointpair)[secondindex] *=secondscaling;
        (*currentpointpair)[firstindex ].z = 1.0;
        (*currentpointpair)[secondindex].z = 1.0;
    }

    // Store calculated values in matrices for faster computation later
    firstsidetransformation << firstscaling <<      0       << -firstoffset.x * firstscaling
        <<     0        << firstscaling << -firstoffset.y * firstscaling
        <<     0        <<      0       <<       1;

    secondsidetransformation << secondscaling   <<      0        << -secondoffset.x*secondscaling
        <<        0        << secondscaling << -secondoffset.y*secondscaling
        <<        0        <<      0        <<           1;
}
