/***************************************************************************
                          CCalibrator.cpp  -  description
                             -------------------
    begin                : 26.03.2001
    copyright            : (C) 2001 by Fabian Wenzel
    email                : tuvisionlib@tuvision.de, f.wenzel@gmx.net
    origin               : Diplomarbeit Fabian Wenzel

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of the TUVision Library's internal version. It is   *
 *   not permitted to publish this file at this point of time as the       *
 *   algorithms are meant to be shared between TUHH Vision Systems         *
 *   members only.                                                         *
 *                                                                         *
 ***************************************************************************/
#include "tuv3d/CCalibrator.h"
#include <fstream>

using namespace std;
using namespace TUVision;
using namespace NEWMAT;

CCalibrator::CCalibrator(){
    d_initialized          = false;
    d_calibrated           = false;
    d_CameraParameters     = 0;
    d_CalibrationConstants = 0;
    d_CalibrationData      = 0;
    d_tsaiCalibrator       = 0;
}

CCalibrator::~CCalibrator(){
    if(d_tsaiCalibrator)
        delete d_tsaiCalibrator;

    if(d_CameraParameters)
        delete d_CameraParameters;

    if(d_CalibrationData)
        delete d_CalibrationData;

    if(d_CalibrationConstants)
        delete d_CalibrationConstants;
}

bool CCalibrator::Init(SCameraParameters *cameraParameters, SCalibrationConstants *calibrationConstants){
    // make a local copy of the camera parameters

    if(!d_CameraParameters)
        d_CameraParameters = new SCameraParameters;
    if(!d_CalibrationConstants)
        d_CalibrationConstants = new SCalibrationConstants;

    memcpy(d_CameraParameters,cameraParameters,sizeof(SCameraParameters));

    if(calibrationConstants)
        memcpy(d_CalibrationConstants,calibrationConstants,sizeof(SCalibrationConstants));
    else {
        memset(d_CalibrationConstants,0,sizeof(SCalibrationConstants));
        d_CalibrationConstants->Tx = 100.0;
        d_CalibrationConstants->Ty = 100.0;
        d_CalibrationConstants->Tz = 100.0;
        d_CalibrationConstants->f  = 3;
        d_CalibrationConstants->Rx = 1.5;
        d_CalibrationConstants->Ry = 1.5;
        d_CalibrationConstants->Rz = 1.5;
    }

    d_initialized = true;

    return true;
}

bool CCalibrator::Save(string cameraparameterfilename){
    fstream cameraparameterfile(cameraparameterfilename.c_str(),std::ios::out);
    cameraparameterfile << d_CameraParameters->Ncx;
    cameraparameterfile << d_CameraParameters->Nfx;
    cameraparameterfile << d_CameraParameters->dx;
    cameraparameterfile << d_CameraParameters->dy;
    cameraparameterfile << d_CameraParameters->dpx;
    cameraparameterfile << d_CameraParameters->dpy;
    cameraparameterfile << d_CameraParameters->Cx;
    cameraparameterfile << d_CameraParameters->Cy;
    cameraparameterfile << d_CameraParameters->sx;
    cameraparameterfile << d_CalibrationConstants->f;
    cameraparameterfile << d_CalibrationConstants->kappa1;
    cameraparameterfile << d_CalibrationConstants->Tx;
    cameraparameterfile << d_CalibrationConstants->Ty;
    cameraparameterfile << d_CalibrationConstants->Tz;
    cameraparameterfile << d_CalibrationConstants->Rx;
    cameraparameterfile << d_CalibrationConstants->Ry;
    cameraparameterfile << d_CalibrationConstants->Rz;
    cameraparameterfile.close();
    return true;
}

bool CCalibrator::Init(string cameraparameterfilename){
    if(!d_CameraParameters)
        d_CameraParameters = new SCameraParameters;
    if(!d_CalibrationConstants)
        d_CalibrationConstants = new SCalibrationConstants;

    fstream cameraparameterfile(cameraparameterfilename.c_str(),ios::in);
    cameraparameterfile >> d_CameraParameters->Ncx;
    cameraparameterfile >> d_CameraParameters->Nfx;
    cameraparameterfile >> d_CameraParameters->dx;
    cameraparameterfile >> d_CameraParameters->dy;
    cameraparameterfile >> d_CameraParameters->dpx;
    cameraparameterfile >> d_CameraParameters->dpy;
    cameraparameterfile >> d_CameraParameters->Cx;
    cameraparameterfile >> d_CameraParameters->Cy;
    cameraparameterfile >> d_CameraParameters->sx;
    memset(d_CalibrationConstants,0,sizeof(SCalibrationConstants));
    cameraparameterfile >> d_CalibrationConstants->f;
    cameraparameterfile >> d_CalibrationConstants->kappa1;
    cameraparameterfile >> d_CalibrationConstants->Tx;
    cameraparameterfile >> d_CalibrationConstants->Ty;
    cameraparameterfile >> d_CalibrationConstants->Tz;
    cameraparameterfile >> d_CalibrationConstants->Rx;
    cameraparameterfile >> d_CalibrationConstants->Ry;
    cameraparameterfile >> d_CalibrationConstants->Rz;
    cameraparameterfile.close();
    d_initialized = true;

    memcpy(&d_tsaiCalibrator->cc,d_CalibrationConstants,sizeof(SCalibrationConstants));
    memcpy(&d_tsaiCalibrator->cp,d_CameraParameters,sizeof(SCameraParameters));

    return true;
}


bool CCalibrator::Init(int width, int height){
    if(!d_CameraParameters)
        d_CameraParameters = new SCameraParameters;
    if(!d_CalibrationConstants)
        d_CalibrationConstants = new SCalibrationConstants;

    d_CameraParameters->Ncx = width;
    d_CameraParameters->Nfx = width;
    d_CameraParameters->sx  = 1.0;
    d_CameraParameters->dpy = 1.0E-2;
    d_CameraParameters->dpx = 1.0E-2; // Assume that CCD elements have square shape
    d_CameraParameters->dx  = 1.0E-2*width;
    d_CameraParameters->dy  = 1.0E-2*height; // Assume that there is no resampling
    d_CameraParameters->Cx  = (double) width/2;
    d_CameraParameters->Cy  = (double) height/2; // Assume that the optical center is in the middle of the framebuffer  

    memset(d_CalibrationConstants,0,sizeof(SCalibrationConstants));

    d_CalibrationConstants->Tx = 100.0;
    d_CalibrationConstants->Ty = 100.0;
    d_CalibrationConstants->Tz = 100.0;
    d_CalibrationConstants->f  = 3;
    d_CalibrationConstants->Rx = 1.5;
    d_CalibrationConstants->Ry = 1.5;
    d_CalibrationConstants->Rz = 1.5;

    d_initialized = true;
    return true;
}

bool CCalibrator::InitializeTsai(SCalibrationData *calibrationdata){
    int i;
    bool doCoplanar=true;

    if(!d_initialized)
        return false;

    // First: Check if we can use coplanar case
    for(i=0;i<calibrationdata->point_count;i++){
        if(calibrationdata->worldCoordinates[i].Ncols() == 3)
            doCoplanar &= (calibrationdata->worldCoordinates[i](3) == 0.0);
        if(!doCoplanar)
            break;
    }

    if(doCoplanar){
        d_tsaiCalibrator = new TsaiCalibratorCoplanar();
    } else {
        d_tsaiCalibrator = new TsaiCalibratorNoncoplanar();
    }


    d_tsaiCalibrator->cd.point_count=calibrationdata->point_count;

    for(i=0;i<calibrationdata->point_count;i++){
        d_tsaiCalibrator->cd.Xf[i]=calibrationdata->imageCoordinates[i](1);
        d_tsaiCalibrator->cd.Yf[i]=calibrationdata->imageCoordinates[i](2);
        d_tsaiCalibrator->cd.xw[i]=calibrationdata->worldCoordinates[i](1);
        d_tsaiCalibrator->cd.yw[i]=calibrationdata->worldCoordinates[i](2);
        if(calibrationdata->worldCoordinates[i].Ncols() == 3)
            d_tsaiCalibrator->cd.zw[i]=calibrationdata->worldCoordinates[i](3);
    }

    if(d_CalibrationConstants)
        memcpy(&d_tsaiCalibrator->cc,d_CalibrationConstants,sizeof(SCalibrationConstants));

    memcpy(&d_tsaiCalibrator->cp,d_CameraParameters,sizeof(SCameraParameters));
    return true;
}

bool CCalibrator::Calibrate(SCalibrationData *calibrationdata){
    bool retvalue;

    if(InitializeTsai(calibrationdata))
        retvalue=(d_calibrated=d_tsaiCalibrator->calibration_with_full_optimization());
    else
        retvalue=false;

    if(retvalue){
        memcpy(d_CalibrationConstants,&d_tsaiCalibrator->cc,sizeof(SCalibrationConstants));
        memcpy(d_CameraParameters,&d_tsaiCalibrator->cp,sizeof(SCameraParameters));
    }

    return retvalue;

}

Matrix *CCalibrator::getNewRotationMatrix(){
    if(!d_calibrated)
        return 0;

    Matrix *rotationMatrix = new Matrix(3,3);
    (*rotationMatrix) << d_tsaiCalibrator->cc.r1 << d_tsaiCalibrator->cc.r2 << d_tsaiCalibrator->cc.r3
        << d_tsaiCalibrator->cc.r4 << d_tsaiCalibrator->cc.r5 << d_tsaiCalibrator->cc.r6
        << d_tsaiCalibrator->cc.r7 << d_tsaiCalibrator->cc.r8 << d_tsaiCalibrator->cc.r9;

    return rotationMatrix;
}

void CCalibrator::getRotationMatrix(Matrix &rotationMatrix){
    if(!d_calibrated)
        return;

    rotationMatrix = Matrix(3,3);

    rotationMatrix << d_tsaiCalibrator->cc.r1 << d_tsaiCalibrator->cc.r2 << d_tsaiCalibrator->cc.r3
        << d_tsaiCalibrator->cc.r4 << d_tsaiCalibrator->cc.r5 << d_tsaiCalibrator->cc.r6
        << d_tsaiCalibrator->cc.r7 << d_tsaiCalibrator->cc.r8 << d_tsaiCalibrator->cc.r9;
}

void CCalibrator::getTranslationVector(ColumnVector &translationVector){
    if(!d_calibrated)
        return;

    translationVector = ColumnVector(3);
    translationVector << d_tsaiCalibrator->cc.Tx
        << d_tsaiCalibrator->cc.Ty
        << d_tsaiCalibrator->cc.Tz;
}


Matrix *CCalibrator::getNewTranslationVector(){
    if(!d_calibrated)
        return 0;

    return 0;
}

Matrix *CCalibrator::getNewProjectionMatrix(){
    if(!d_calibrated)
        return 0;

    return 0;
}

void CCalibrator::getProjectionMatrix(Matrix &projectionMatrix){
    if(!d_calibrated)
        return;

    Matrix A(3,3);
    Matrix Rt(3,4);
    getAMatrix(A);
    getRTMatrix(Rt);
    projectionMatrix = A*Rt;

}

void CCalibrator::getRTMatrix(Matrix &Rt){
    Rt=Matrix(3,4);

    Rt << d_tsaiCalibrator->cc.r1 << d_tsaiCalibrator->cc.r2 << d_tsaiCalibrator->cc.r3 << d_tsaiCalibrator->cc.Tx
        << d_tsaiCalibrator->cc.r4 << d_tsaiCalibrator->cc.r5 << d_tsaiCalibrator->cc.r6 << d_tsaiCalibrator->cc.Ty
        << d_tsaiCalibrator->cc.r7 << d_tsaiCalibrator->cc.r8 << d_tsaiCalibrator->cc.r9 << d_tsaiCalibrator->cc.Tz;
}

void CCalibrator::getAMatrix(Matrix &A){
    A=Matrix(3,3);
    A=0.0;    // Initialize matrix! (For elements that are not filled below)

    A(1,1) = d_tsaiCalibrator->cc.f*d_tsaiCalibrator->cp.sx/d_tsaiCalibrator->cp.dpx;
    A(2,2) = d_tsaiCalibrator->cc.f/d_tsaiCalibrator->cp.dpy;
    A(3,3) = 1.0;
    A(1,3) = d_tsaiCalibrator->cp.Cx;
    A(2,3) = d_tsaiCalibrator->cp.Cy;
}

bool CCalibrator::ExtrinsicCalibrate(SCalibrationData *calibrationdata){
    bool retvalue;
    if(InitializeTsai(calibrationdata))
        retvalue = d_calibrated=d_tsaiCalibrator->extrinsic_parameter_estimation();

    if(retvalue){
        memcpy(d_CalibrationConstants,&d_tsaiCalibrator->cc,sizeof(SCalibrationConstants));
        memcpy(d_CameraParameters,&d_tsaiCalibrator->cp,sizeof(SCameraParameters));
    }

    return retvalue;
}

void CCalibrator::getCalibrationErrors(SCalibrationError *undistortedImagePlaneError,SCalibrationError *distortedImagePlaneError,SCalibrationError *objectSpaceError,SCalibrationError *normalizedCalibrationError){
    if(!d_tsaiCalibrator || !d_initialized)
        return;

    d_tsaiCalibrator->distorted_image_plane_error_stats (&undistortedImagePlaneError->mean, &undistortedImagePlaneError->stddev, &undistortedImagePlaneError->max, &undistortedImagePlaneError->sse);
    d_tsaiCalibrator->undistorted_image_plane_error_stats (&distortedImagePlaneError->mean, &distortedImagePlaneError->stddev, &distortedImagePlaneError->max, &distortedImagePlaneError->sse);
    d_tsaiCalibrator->object_space_error_stats (&objectSpaceError->mean, &objectSpaceError->stddev, &objectSpaceError->max, &objectSpaceError->sse);
    d_tsaiCalibrator->normalized_calibration_error (&normalizedCalibrationError->mean, &normalizedCalibrationError->stddev);  
}
